@@ -1,5 +1,10 @@
 #include <stdio.h>
-main()
+
+#ifdef _WIN64
+#define long long long
+#endif
+
+int main()
 {
   if (sizeof(long) == 4)
   {
@@ -14,8 +19,9 @@ main()
   {
     union {double d; unsigned long l;} x;
     x.d = 2.;
-    if (x.l==(1UL<<62)) printf("-\n");
+    if (x.l==((unsigned long)1)<<62) printf("-\n");
     else
       printf("NOT IEEE (64 bit)\n");
   }
+  return 0;
 }
