@@ -109,7 +109,7 @@ Rg_to_raw(GEN x, GEN ff)
     return gel(x,2);
   }
   pari_err_TYPE("Rg_to_raw",x);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 static GEN
@@ -132,23 +132,27 @@ FFX_to_raw(GEN x, GEN ff)
 }
 
 static GEN
-FFC_to_raw(GEN x)
-{
-  long i, lx;
-  GEN y = cgetg_copy(x,&lx);
-  for(i=1; i<lx; i++)
-    gel(y, i) = gel(gel(x, i), 2);
-  return y;
-}
+FFC_to_raw(GEN x, GEN ff)
+{ pari_APPLY_same(Rg_to_raw(gel(x, i), ff)) }
 
 static GEN
-FFM_to_raw(GEN x)
+FFM_to_raw(GEN x, GEN ff)
+{ pari_APPLY_same(FFC_to_raw(gel(x, i), ff)) }
+
+/* in place */
+static GEN
+rawFq_to_FF(GEN x, GEN ff)
 {
-  long i, lx;
-  GEN y = cgetg_copy(x,&lx);
-  for(i=1; i<lx; i++)
-    gel(y, i) = FFC_to_raw(gel(x, i));
-  return y;
+  return mkFF_i(ff, typ(x)==t_INT ? scalarpol(x, varn(gel(ff,3))): x);
+}
+
+/* in place */
+static GEN
+raw_to_FFX(GEN x, GEN ff)
+{
+  long i, lx = lg(x);
+  for (i=2; i<lx; i++) gel(x,i) = rawFq_to_FF(gel(x,i), ff);
+  return x;
 }
 
 /* in place */
@@ -190,16 +194,6 @@ Fq_to_FF(GEN x, GEN ff)
   return _mkFF_i(ff,z,r);
 }
 
-/* in place */
-static GEN
-to_FFX(GEN x, GEN ff)
-{
-  long i, lx = lg(x);
-  if (typ(x) != t_POL) pari_err_TYPE("to_FFX",x);
-  for (i=2; i<lx; i++) gel(x,i) = Fq_to_FF(gel(x,i), ff);
-  return x;
-}
-
 /*************************************************************************/
 /**                                                                     **/
 /**                   Public functions                                  **/
@@ -207,11 +201,17 @@ to_FFX(GEN x, GEN ff)
 /*************************************************************************/
 
 /* Return true if x and y are defined in the same field */
+
+static int
+FF_samechar(GEN x, GEN y)
+{
+  return x[1] == y[1] && equalii(gel(x,4),gel(y,4));
+}
+
 int
 FF_samefield(GEN x, GEN y)
 {
-  return x[1] == y[1] && equalii(gel(x,4),gel(y,4))
-                      && gidentical(gel(x,3),gel(y,3));
+  return FF_samechar(x, y) && gidentical(gel(x,3),gel(y,3));
 }
 
 int
@@ -298,6 +298,27 @@ FF_1(GEN x)
   return _mkFF(x,z,r);
 }
 
+GEN
+FF_gen(GEN x)
+{
+  ulong pp;
+  GEN r, T, p, z=_initFF(x,&T,&p,&pp);
+  switch(x[1])
+  {
+  case t_FF_FpXQ:
+    r = pol_x(varn(T));
+    if (degpol(T)==1) r = FpX_rem(r, T, p);
+    break;
+  case t_FF_F2xq:
+    r = polx_F2x(T[1]);
+    if (F2x_degree(T)==1) r = F2x_rem(r, T);
+    break;
+  default:
+    r = polx_Flx(T[1]);
+    if (degpol(T)==1) r = Flx_rem(r, T, pp);
+  }
+  return _mkFF(x,z,r);
+}
 GEN
 FF_q(GEN x)
 {
@@ -560,27 +581,92 @@ FF_neg_i(GEN x)
   return _mkFF_i(x,z,r);
 }
 
+GEN
+FF_map(GEN m, GEN x)
+{
+  ulong pp;
+  GEN r, T, p, z=_initFF(m,&T,&p,&pp);
+  switch(m[1])
+  {
+  case t_FF_FpXQ:
+    r=FpX_FpXQ_eval(gel(x,2),gel(m,2),T,p);
+    break;
+  case t_FF_F2xq:
+    r=F2x_F2xq_eval(gel(x,2),gel(m,2),T);
+    break;
+  default:
+    r=Flx_Flxq_eval(gel(x,2),gel(m,2),T,pp);
+  }
+  return _mkFF(m,z,r);
+}
+
+GEN
+FF_Frobenius(GEN x, long e)
+{
+  ulong pp;
+  GEN r, T, p, z=_initFF(x,&T,&p,&pp);
+  ulong n = umodsu(e, FF_f(x));
+  if (n==0) return gcopy(x);
+  switch(x[1])
+  {
+  case t_FF_FpXQ:
+    r=FpXQ_pow(gel(x,2),p,T,p);
+    if (n>1) r=FpXQ_autpow(r,n,T,p);
+    break;
+  case t_FF_F2xq:
+    r=F2xq_sqr(gel(x,2),T);
+    if (n>1) r=F2xq_autpow(r,n,T);
+    break;
+  default:
+    r=Flxq_powu(gel(x,2),pp,T,pp);
+    if (n>1) r=Flxq_autpow(r,n,T,pp);
+  }
+  return _mkFF(x,z,r);
+}
+
+GEN
+FFX_preimage(GEN x, GEN F, GEN y)
+{
+  GEN r, T, p, z;
+  ulong pp;
+  if (FF_equal0(x)) return FF_zero(y);
+  z=_initFF(y,&T,&p,&pp);
+  F = FFX_to_raw(F, y);
+  switch(y[1])
+  {
+  case t_FF_FpXQ:
+    r = FpXQX_rem(gel(x,2), F, T, p);
+    break;
+  case t_FF_F2xq:
+    r = F2xqX_rem(F2x_to_F2xX(gel(x,2),T[1]), F, T);
+    break;
+  default:
+    r = FlxqX_rem(Flx_to_FlxX(gel(x,2),T[1]), F, T, pp);
+  }
+  if (degpol(r) > 0) return NULL;
+  r = (y[1] == t_FF_FpXQ)? Fq_to_FpXQ(gel(r,2),T, p): gel(r,2);
+  return _mkFF(y,z,r);
+}
+
 GEN
 FF_mul(GEN x, GEN y)
 {
   ulong pp;
   GEN r, T, p, z=_initFF(x,&T,&p,&pp);
+  pari_sp av=avma;
   _checkFF(x,y,"*");
   switch(x[1])
   {
   case t_FF_FpXQ:
-    {
-      pari_sp av=avma;
-      r=gerepileupto(av,FpXQ_mul(gel(x,2),gel(y,2),T,p));
-      break;
-    }
+    r=FpXQ_mul(gel(x,2),gel(y,2),T,p);
+    break;
   case t_FF_F2xq:
     r=F2xq_mul(gel(x,2),gel(y,2),T);
     break;
   default:
     r=Flxq_mul(gel(x,2),gel(y,2),T,pp);
   }
-  return _mkFF(x,z,r);
+  return _mkFF(x,z,gerepileupto(av, r));
 }
 
 GEN
@@ -1110,14 +1196,13 @@ F2xq_ell_to_a4a6(GEN E, GEN T)
     GEN d6 = F2xq_mul(F2x_add(F2x_add(F2xq_mul(F2x_add(F2xq_mul(da2,d,T),a4),d,T),a6),F2xq_sqr(e,T)),a1i6,T);
     retmkvec3(d2, d6, mkvec4(a1i,dd,pol0_F2x(v),ee));
   }
-  else if (lgpol(a3))
-  {
+  else
+  { /* allow a1 = a3 = 0: singular curve */
     GEN d4 = F2x_add(F2xq_sqr(a2,T),a4);
     GEN d6 = F2x_add(F2xq_mul(a2,a4,T),a6);
-    GEN a3i = F2xq_inv(a3,T);
+    GEN a3i = lgpol(a3)? F2xq_inv(a3,T): a3;
     retmkvec3(mkvec3(a3,d4,a3i), d6, mkvec4(pol1_F2x(v),a2,pol0_F2x(T[1]),pol0_F2x(T[1])));
   }
-  return NULL;
 }
 
 static GEN
@@ -1156,7 +1241,7 @@ FF_ellcard(GEN E)
 }
 
 GEN
-FF_ellcard_SEA(GEN E, ulong smallfact)
+FF_ellcard_SEA(GEN E, long smallfact)
 {
   pari_sp av = avma;
   GEN T,p;
@@ -1181,11 +1266,11 @@ FF_ellcard_SEA(GEN E, ulong smallfact)
   return gerepileuptoint(av, card);
 }
 
-/* return [G,m] */
+/* return G, set m */
 GEN
-FF_ellgroup(GEN E)
+FF_ellgroup(GEN E, GEN *pm)
 {
-  GEN T, p, e, fg, N, G, m;
+  GEN T, p, e, fg, N;
   ulong pp;
 
   N = ellff_get_card(E);
@@ -1195,37 +1280,37 @@ FF_ellgroup(GEN E)
   switch(fg[1])
   {
   case t_FF_FpXQ:
-    G = FpXQ_ellgroup(Fq_to_FpXQ(gel(e,1),T,p),Fq_to_FpXQ(gel(e,2),T,p),N,T,p,&m);
-    break;
+    return FpXQ_ellgroup(Fq_to_FpXQ(gel(e,1),T,p),
+                         Fq_to_FpXQ(gel(e,2),T,p),N,T,p,pm);
   case t_FF_F2xq:
-    G = F2xq_ellgroup(gel(e,1),gel(e,2),N,T,&m); break;
+    return F2xq_ellgroup(gel(e,1),gel(e,2),N,T,pm);
   default:
-    G = Flxq_ellgroup(gel(e,1),gel(e,2),N,T,pp,&m); break;
+    return Flxq_ellgroup(gel(e,1),gel(e,2),N,T,pp,pm);
   }
-  return mkvec2(G,m);
 }
 
 GEN
 FF_ellgens(GEN E)
 {
-  GEN fg, e, Gm, G, m, T, p, F, e3;
+  GEN D, fg, e, m, T, p, F, e3;
   ulong pp;
 
   fg = ellff_get_field(E);
   e = ellff_get_a4a6(E);
-  Gm = ellff_get_group(E); G = gel(Gm,1); m = gel(Gm,2);
+  m = ellff_get_m(E);
+  D = ellff_get_D(E);
   _getFF(fg,&T,&p,&pp);
   switch(fg[1])
   {
   case t_FF_FpXQ:
     e3 = FqV_to_FpXQV(gel(e,3),T);
-    F = FpXQ_ellgens(Fq_to_FpXQ(gel(e,1),T,p),Fq_to_FpXQ(gel(e,2),T,p),e3,G,m,T,p);
+    F = FpXQ_ellgens(Fq_to_FpXQ(gel(e,1),T,p),Fq_to_FpXQ(gel(e,2),T,p),e3,D,m,T,p);
     break;
   case t_FF_F2xq:
-    F = F2xq_ellgens(gel(e,1),gel(e,2),gel(e,3),G,m,T);
+    F = F2xq_ellgens(gel(e,1),gel(e,2),gel(e,3),D,m,T);
     break;
   default:
-    F = Flxq_ellgens(gel(e,1),gel(e,2),gel(e,3),G,m,T, pp);
+    F = Flxq_ellgens(gel(e,1),gel(e,2),gel(e,3),D,m,T, pp);
     break;
   }
   return to_FFE_vec(F,fg);
@@ -1249,39 +1334,32 @@ FF_elldata(GEN E, GEN fg)
   return mkvec2(fg,e);
 }
 
+/* allow singular E, set E.j = 0 in this case */
 GEN
 FF_ellinit(GEN E, GEN fg)
 {
-  GEN T,p,e;
+  GEN T,p,e, D, c4, y = E;
   ulong pp;
   long i;
-  GEN y=E;
   _getFF(fg,&T,&p,&pp);
   switch(fg[1])
   {
   case t_FF_FpXQ:
     e = FpXQ_ell_to_a4a6(E,T,p);
-    for(i=1;i<=12;i++)
-      gel(y,i) = mkFF_i(fg,Rg_to_FpXQ(gel(E,i),T,p));
-    if (FF_equal0(gel(y,12))) return NULL;
-    gel(y,i) = mkFF_i(fg,Rg_to_FpXQ(gel(E,i),T,p));
+    for(i=1;i<=12;i++) gel(y,i) = mkFF_i(fg,Rg_to_FpXQ(gel(E,i),T,p));
     break;
   case t_FF_F2xq:
     e = F2xq_ell_to_a4a6(E,T);
-    if (!e) return NULL;
-    for(i=1;i<=12;i++)
-      gel(y,i) = mkFF_i(fg,Rg_to_F2xq(gel(E,i),T));
-    if (FF_equal0(gel(y,12))) return NULL;
-    gel(y,i) = mkFF_i(fg,Rg_to_F2xq(gel(E,i),T));
+    for(i=1;i<=12;i++) gel(y,i) = mkFF_i(fg,Rg_to_F2xq(gel(E,i),T));
     break;
   default:
     e = Flxq_ell_to_a4a6(E,T,pp);
-    for(i=1;i<=12;i++)
-      gel(y,i) = mkFF_i(fg,Rg_to_Flxq(gel(E,i),T,pp));
-    if (FF_equal0(gel(y,12))) return NULL;
-    gel(y,i) = mkFF_i(fg,Rg_to_Flxq(gel(E,i),T,pp));
+    for(i=1;i<=12;i++) gel(y,i) = mkFF_i(fg,Rg_to_Flxq(gel(E,i),T,pp));
     break;
   }
+  D = ell_get_disc(y);
+  c4 = ell_get_c4(y);
+  gel(y,13) = FF_equal0(D)? D: gdiv(gpowgs(c4,3), D);
   gel(y,14) = mkvecsmall(t_ELL_Fq);
   gel(y,15) = mkvec2(fg,e);
   return y;
@@ -1525,7 +1603,241 @@ FFX_roots(GEN Pf, GEN ff)
   default:
     r = FlxqX_roots(P, T, pp);
   }
-  r = raw_to_FFC(r, ff);
+  return gerepilecopy(av, raw_to_FFC(r, ff));
+}
+
+static GEN
+raw_to_FFXC(GEN x, GEN ff)
+{ pari_APPLY_type(t_COL, raw_to_FFX(gel(x,i), ff)); }
+
+static GEN
+raw_to_FFX_fact(GEN F, GEN ff)
+{
+  GEN y, u, v;
+  GEN P = gel(F,1), E = gel(F,2);
+  long j, l = lg(P);
+  y = cgetg(3,t_MAT);
+  u = cgetg(l,t_COL); gel(y,1) = u;
+  v = cgetg(l,t_COL); gel(y,2) = v;
+  for (j=1; j<l; j++)
+  {
+    gel(u,j) = raw_to_FFX(gel(P,j), ff);
+    gel(v,j) = utoi(uel(E,j));
+  }
+  return y;
+}
+
+static GEN
+FFX_zero(GEN ff, long v)
+{
+  GEN r = cgetg(3,t_POL);
+  r[1] = evalvarn(v);
+  gel(r,2) = FF_zero(ff);
+  return r;
+}
+
+static GEN
+FFX_wrap2(GEN Pf, GEN Qf, GEN ff, GEN FpXQX(GEN, GEN, GEN, GEN),
+          GEN F2xqX(GEN, GEN, GEN), GEN FlxqX(GEN, GEN, GEN, ulong))
+{
+  pari_sp av = avma;
+  GEN r,T,p;
+  ulong pp;
+  GEN P = FFX_to_raw(Pf, ff);
+  GEN Q = FFX_to_raw(Qf, ff);
+  _getFF(ff,&T,&p,&pp);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ:
+    r = FpXQX(P, Q, T, p);
+    break;
+  case t_FF_F2xq:
+    r = F2xqX(P, Q, T);
+    break;
+  default:
+    r = FlxqX(P, Q, T, pp);
+  }
+  if (!lgpol(r)) { avma = av; return FFX_zero(ff, varn(Pf)); }
+  return gerepilecopy(av, raw_to_FFX(r, ff));
+}
+
+GEN
+FFX_mul(GEN Pf, GEN Qf, GEN ff)
+{ return FFX_wrap2(Pf, Qf, ff, FpXQX_mul, F2xqX_mul, FlxqX_mul); }
+
+GEN
+FFX_sqr(GEN Pf, GEN ff)
+{
+  pari_sp av = avma;
+  GEN r,T,p;
+  ulong pp;
+  GEN P = FFX_to_raw(Pf, ff);
+  _getFF(ff,&T,&p,&pp);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ:
+    r = FpXQX_sqr(P, T, p);
+    break;
+  case t_FF_F2xq:
+    r = F2xqX_sqr(P, T);
+    break;
+  default:
+    r = FlxqX_sqr(P, T, pp);
+  }
+  if (!lgpol(r)) { avma = av; return FFX_zero(ff, varn(Pf)); }
+  return gerepilecopy(av, raw_to_FFX(r, ff));
+}
+
+GEN
+FFX_rem(GEN Pf, GEN Qf, GEN ff)
+{ return FFX_wrap2(Pf, Qf, ff, FpXQX_rem, F2xqX_rem, FlxqX_rem); }
+
+GEN
+FFXQ_sqr(GEN Pf, GEN Qf, GEN ff)
+{ return FFX_wrap2(Pf, Qf, ff, FpXQXQ_sqr, F2xqXQ_sqr, FlxqXQ_sqr); }
+
+GEN
+FFXQ_inv(GEN Pf, GEN Qf, GEN ff)
+{ return FFX_wrap2(Pf, Qf, ff, FpXQXQ_inv, F2xqXQ_inv, FlxqXQ_inv); }
+
+GEN
+FFXQ_mul(GEN Pf, GEN Qf, GEN Sf, GEN ff)
+{
+  pari_sp av = avma;
+  GEN r,T,p;
+  ulong pp;
+  GEN P = FFX_to_raw(Pf, ff);
+  GEN Q = FFX_to_raw(Qf, ff);
+  GEN S = FFX_to_raw(Sf, ff);
+  _getFF(ff,&T,&p,&pp);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ:
+    r = FpXQXQ_mul(P, Q, S, T, p);
+    break;
+  case t_FF_F2xq:
+    r = F2xqXQ_mul(P, Q, S, T);
+    break;
+  default:
+    r = FlxqXQ_mul(P, Q, S, T, pp);
+  }
+  if (!lgpol(r)) { avma = av; return FFX_zero(ff, varn(Pf)); }
+  return gerepilecopy(av, raw_to_FFX(r, ff));
+}
+
+long
+FFX_ispower(GEN Pf, long k, GEN ff, GEN *pt_r)
+{
+  pari_sp av = avma;
+  GEN P,T,p;
+  ulong pp;
+  long s;
+  if (degpol(Pf) % k) return 0;
+  P = FFX_to_raw(Pf, ff);
+  _getFF(ff,&T,&p,&pp);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ:
+    s = FpXQX_ispower(P, k, T, p, pt_r);
+    break;
+  case t_FF_F2xq:
+    s = F2xqX_ispower(P, k, T, pt_r);
+    break;
+  default:
+    s = FlxqX_ispower(P, k, T, pp, pt_r);
+  }
+  if (s==0) { avma = av; return 0; }
+  if (pt_r)
+    *pt_r = gerepilecopy(av, raw_to_FFX(*pt_r, ff));
+  else avma = av;
+  return 1;
+}
+
+GEN
+FFX_factor(GEN Pf, GEN ff)
+{
+  pari_sp av = avma;
+  GEN r,T,p;
+  ulong pp;
+  GEN P = FFX_to_raw(Pf, ff);
+  _getFF(ff,&T,&p,&pp);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ:
+    r = FpXQX_factor(P, T, p);
+    break;
+  case t_FF_F2xq:
+    r = F2xqX_factor(P, T);
+    break;
+  default:
+    r = FlxqX_factor(P, T, pp);
+  }
+  return gerepilecopy(av, raw_to_FFX_fact(r, ff));
+}
+
+GEN
+FFX_factor_squarefree(GEN Pf, GEN ff)
+{
+  pari_sp av = avma;
+  GEN r,T,p;
+  ulong pp;
+  GEN P = FFX_to_raw(Pf, ff);
+  _getFF(ff,&T,&p,&pp);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ:
+    r = FpXQX_factor_squarefree(P, T, p);
+    break;
+  case t_FF_F2xq:
+    r = F2xqX_factor_squarefree(P, T);
+    break;
+  default:
+    r = FlxqX_factor_squarefree(P, T, pp);
+  }
+  return gerepilecopy(av, raw_to_FFXC(r, ff));
+}
+
+GEN
+FFX_ddf(GEN Pf, GEN ff)
+{
+  pari_sp av = avma;
+  GEN r,T,p;
+  ulong pp;
+  GEN P = FFX_to_raw(Pf, ff);
+  _getFF(ff,&T,&p,&pp);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ:
+    r = FpXQX_ddf(P, T, p);
+    break;
+  case t_FF_F2xq:
+    r = F2xqX_ddf(P, T);
+    break;
+  default:
+    r = FlxqX_ddf(P, T, pp);
+  }
+  return gerepilecopy(av, raw_to_FFX_fact(r, ff));
+}
+
+GEN
+FFX_degfact(GEN Pf, GEN ff)
+{
+  pari_sp av = avma;
+  GEN r,T,p;
+  ulong pp;
+  GEN P = FFX_to_raw(Pf, ff);
+  _getFF(ff, &T, &p, &pp);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ:
+    r = FpXQX_degfact(P, T, p);
+    break;
+  case t_FF_F2xq:
+    r = F2xqX_degfact(P, T);
+    break;
+  default:
+    r = FlxqX_degfact(P, T, pp);
+  }
   return gerepilecopy(av, r);
 }
 
@@ -1539,69 +1851,6 @@ FqX_to_FFX(GEN x, GEN ff)
   return y;
 }
 
-/* P vector of t_POL, E t_VECSMALL of exponents, ff a t_FFELT. Update elts of
- * P so that 1) variable number is vP, 2) coefficients are ff-compatible.
- * Collect garbage wrt av */
-static GEN
-to_FF_fact(long vP, GEN P, GEN E, GEN ff, pari_sp av)
-{
-  GEN y = cgetg(3,t_MAT), u, v, zf;
-  long j, l = lg(P), nbf = lg(P);
-
-  u = cgetg(nbf,t_COL); gel(y,1) = u;
-  v = cgetg(nbf,t_COL); gel(y,2) = v;
-  for (j=1; j<l; j++)
-  {
-    GEN Q = simplify_shallow(gel(P,j)); /* may contain pols of degree 0 */
-    if (typ(Q) == t_POL) setvarn(Q, vP);
-    gel(u,j) = Q;
-    gel(v,j) = utoi(uel(E,j));
-  }
-  y = gerepilecopy(av, y); u = gel(y,1);
-  zf = FF_zero(ff);
-  for (j=1; j<nbf; j++) gel(u,j) = to_FFX(gel(u,j), zf);
-  return y;
-}
-
-static GEN
-FFX_init_fix_varn(GEN P, GEN x, GEN *pT, GEN *pp)
-{
-  ulong junk;
-  GEN Q, T, p;
-
-  _getFF(x, &T, &p, &junk);
-  switch(x[1])
-  {
-  case t_FF_FpXQ:
-    T=shallowcopy(T);
-    break;
-  case t_FF_F2xq:
-    T=F2x_to_ZX(T);
-    break;
-  default:
-    T=Flx_to_ZX(T);
-  }
-  setvarn(T, 1);
-  Q = RgX_to_FqX(P, T,p);
-  setvarn(Q, 0);
-
-  *pT = T;
-  *pp = p; return Q;
-}
-
-/* Factor P over the field of definition of x */
-GEN
-FFX_factor(GEN P, GEN x)
-{
-  long vP = varn(P);
-  GEN r, T, p;
-  pari_sp av=avma;
-
-  P = FFX_init_fix_varn(P, x, &T, &p);
-  r = FpXQX_factor(P, T,p);
-  return to_FF_fact(vP, gel(r,1),gel(r,2), x,av);
-}
-
 GEN
 ffgen(GEN T, long v)
 {
@@ -1609,6 +1858,9 @@ ffgen(GEN T, long v)
   long d;
   switch(typ(T))
   {
+    case t_FFELT:
+      p = FF_p_i(T); T = FF_mod(T); d = degpol(T);
+      break;
     case t_POL:
       d = degpol(T); p = NULL;
       if (d < 1 || !RgX_is_FpX(T, &p) || !p) pari_err_TYPE("ffgen",T);
@@ -1836,6 +2088,7 @@ FqM_to_FpXQM(GEN x, GEN T, GEN p)
   return y;
 }
 
+/* for functions t_MAT -> t_MAT */
 static GEN
 FFM_wrap(GEN M, GEN ff, GEN (*Fq)(GEN,GEN,GEN),
                        GEN (*Flxq)(GEN,GEN,ulong),
@@ -1844,7 +2097,7 @@ FFM_wrap(GEN M, GEN ff, GEN (*Fq)(GEN,GEN,GEN),
   pari_sp av = avma;
   ulong pp;
   GEN T, p;
-  _getFF(ff,&T,&p,&pp); M = FFM_to_raw(M);
+  _getFF(ff,&T,&p,&pp); M = FFM_to_raw(M, ff);
   switch(ff[1])
   {
   case t_FF_FpXQ: M = Fq(M,T,p); if (M) M = FqM_to_FpXQM(M,T,p);
@@ -1855,6 +2108,56 @@ FFM_wrap(GEN M, GEN ff, GEN (*Fq)(GEN,GEN,GEN),
   if (!M) { avma = av; return NULL; }
   return gerepilecopy(av, raw_to_FFM(M, ff));
 }
+
+/* for functions (t_MAT, t_MAT) -> t_MAT */
+static GEN
+FFM_FFM_wrap(GEN M, GEN N, GEN ff,
+             GEN (*Fq)(GEN, GEN, GEN, GEN),
+             GEN (*Flxq)(GEN, GEN, GEN, ulong),
+             GEN (*F2xq)(GEN, GEN, GEN))
+{
+  pari_sp av = avma;
+  ulong pp;
+  GEN T, p;
+  int is_sqr = M==N;
+  _getFF(ff, &T, &p, &pp);
+  M = FFM_to_raw(M, ff);
+  N = is_sqr? M: FFM_to_raw(N, ff);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ: M = Fq(M, N, T, p); if (M) M = FqM_to_FpXQM(M, T, p);
+                  break;
+  case t_FF_F2xq: M = F2xq(M, N, T); break;
+  default: M = Flxq(M, N, T, pp); break;
+  }
+  if (!M) { avma = av; return NULL; }
+  return gerepilecopy(av, raw_to_FFM(M, ff));
+}
+
+/* for functions (t_MAT, t_COL) -> t_COL */
+static GEN
+FFM_FFC_wrap(GEN M, GEN C, GEN ff,
+             GEN (*Fq)(GEN, GEN, GEN, GEN),
+             GEN (*Flxq)(GEN, GEN, GEN, ulong),
+             GEN (*F2xq)(GEN, GEN, GEN))
+{
+  pari_sp av = avma;
+  ulong pp;
+  GEN T, p;
+  _getFF(ff, &T, &p, &pp);
+  M = FFM_to_raw(M, ff);
+  C = FFC_to_raw(C, ff);
+  switch(ff[1])
+  {
+  case t_FF_FpXQ: C = Fq(M, C, T, p); if (C) C = FqC_to_FpXQC(C, T, p);
+                  break;
+  case t_FF_F2xq: C = F2xq(M, C, T); break;
+  default: C = Flxq(M, C, T, pp); break;
+  }
+  if (!C) { avma = av; return NULL; }
+  return gerepilecopy(av, raw_to_FFC(C, ff));
+}
+
 GEN
 FFM_ker(GEN M, GEN ff)
 { return FFM_wrap(M,ff, &FqM_ker,&FlxqM_ker,&F2xqM_ker); }
@@ -1864,6 +2167,42 @@ FFM_image(GEN M, GEN ff)
 GEN
 FFM_inv(GEN M, GEN ff)
 { return FFM_wrap(M,ff, &FqM_inv,&FlxqM_inv,&F2xqM_inv); }
+GEN
+FFM_suppl(GEN M, GEN ff)
+{ return FFM_wrap(M,ff, &FqM_suppl,&FlxqM_suppl,&F2xqM_suppl); }
+
+GEN
+FFM_deplin(GEN M, GEN ff)
+{
+  pari_sp av = avma;
+  ulong pp;
+  GEN C, T, p;
+  _getFF(ff, &T, &p, &pp); M = FFM_to_raw(M, ff);
+  switch(ff[1]) {
+  case t_FF_FpXQ: C = FqM_deplin(M, T, p);
+    if (C) C = FqC_to_FpXQC(C, T, p); break;
+  case t_FF_F2xq: C = F2xqM_deplin(M, T); break;
+  default: C = FlxqM_deplin(M, T, pp); break;
+  }
+  if (!C) { avma = av; return NULL; }
+  return gerepilecopy(av, raw_to_FFC(C, ff));
+}
+
+GEN
+FFM_indexrank(GEN M, GEN ff)
+{
+  pari_sp av = avma;
+  ulong pp;
+  GEN R, T, p;
+  _getFF(ff,&T,&p,&pp); M = FFM_to_raw(M, ff);
+  switch(ff[1]) {
+  case t_FF_FpXQ: R = FqM_indexrank(M,T,p); break;
+  case t_FF_F2xq: R = F2xqM_indexrank(M,T); break;
+  default: R = FlxqM_indexrank(M,T,pp); break;
+  }
+  return gerepileupto(av, R);
+}
+
 long
 FFM_rank(GEN M, GEN ff)
 {
@@ -1871,7 +2210,7 @@ FFM_rank(GEN M, GEN ff)
   long r;
   ulong pp;
   GEN T, p;
-  _getFF(ff,&T,&p,&pp); M = FFM_to_raw(M);
+  _getFF(ff,&T,&p,&pp); M = FFM_to_raw(M, ff);
   switch(ff[1])
   {
   case t_FF_FpXQ: r = FqM_rank(M,T,p); break;
@@ -1886,7 +2225,7 @@ FFM_det(GEN M, GEN ff)
   pari_sp av = avma;
   ulong pp;
   GEN d, T, p;
-  _getFF(ff,&T,&p,&pp); M = FFM_to_raw(M);
+  _getFF(ff,&T,&p,&pp); M = FFM_to_raw(M, ff);
   switch(ff[1])
   {
   case t_FF_FpXQ: d = FqM_det(M,T,p); break;
@@ -1896,39 +2235,43 @@ FFM_det(GEN M, GEN ff)
   return gerepilecopy(av, mkFF_i(ff, d));
 }
 
+GEN
+FFM_FFC_gauss(GEN M, GEN C, GEN ff)
+{
+  return FFM_FFC_wrap(M, C, ff, FqM_FqC_gauss,
+                      FlxqM_FlxqC_gauss, F2xqM_F2xqC_gauss);
+}
+
+GEN
+FFM_gauss(GEN M, GEN N, GEN ff)
+{
+  return FFM_FFM_wrap(M, N, ff, FqM_gauss,
+                      FlxqM_gauss, F2xqM_gauss);
+}
+
+GEN
+FFM_FFC_invimage(GEN M, GEN C, GEN ff)
+{
+  return FFM_FFC_wrap(M, C, ff, FqM_FqC_invimage,
+                      FlxqM_FlxqC_invimage, F2xqM_F2xqC_invimage);
+}
+
+GEN
+FFM_invimage(GEN M, GEN N, GEN ff)
+{
+  return FFM_FFM_wrap(M, N, ff, FqM_invimage,
+                      FlxqM_invimage, F2xqM_invimage);
+}
+
 GEN
 FFM_FFC_mul(GEN M, GEN C, GEN ff)
 {
-  pari_sp av = avma;
-  ulong pp;
-  GEN P, T, p;
-  _getFF(ff, &T, &p, &pp);
-  M = FFM_to_raw(M);
-  C = FFC_to_raw(C);
-  switch (ff[1])
-  {
-  case t_FF_FpXQ: P = FqM_FqC_mul(M, C, T, p); break;
-  case t_FF_F2xq: P = F2xqM_F2xqC_mul(M, C, T); break;
-  default: P = FlxqM_FlxqC_mul(M, C, T, pp); break;
-  }
-  return gerepilecopy(av, raw_to_FFC(P, ff));
+  return FFM_FFC_wrap(M, C, ff, FqM_FqC_mul,
+                      FlxqM_FlxqC_mul, F2xqM_F2xqC_mul);
 }
 
 GEN
 FFM_mul(GEN M, GEN N, GEN ff)
 {
-  pari_sp av = avma;
-  ulong pp;
-  GEN P, T, p;
-  int is_sqr = M==N;
-  _getFF(ff, &T, &p, &pp);
-  M = FFM_to_raw(M);
-  N = is_sqr? M: FFM_to_raw(N);
-  switch (ff[1])
-  {
-  case t_FF_FpXQ: P = FqM_mul(M, N, T, p); break;
-  case t_FF_F2xq: P = F2xqM_mul(M, N, T); break;
-  default: P = FlxqM_mul(M, N, T, pp); break;
-  }
-  return gerepilecopy(av, raw_to_FFM(P, ff));
+  return FFM_FFM_wrap(M, N, ff, FqM_mul, FlxqM_mul, F2xqM_mul);
 }
