@@ -154,7 +154,7 @@ naf_repr(naf_t *x, ulong a)
   t = c0 - (c1 << 1);
   /* Note that we don't need to check whether t < 0, since a >= 0 implies
    * that this most significant signed bit must be non-negative. */
-  if (t > 0) pbits |= (1UL << i);
+  if (t > 0 && i != BITS_IN_LONG) pbits |= (1UL << i);
 
   x->pbits = pbits;
   x->nbits = nbits;
@@ -162,7 +162,7 @@ naf_repr(naf_t *x, ulong a)
    * like the bit-scan-rev instruction on Intel architectures. */
   /* Using pbits here is justified by the fact that a >= 0, so the
    * most significant bit must be positive. */
-  x->lnzb = expu(pbits) - 2;
+  x->lnzb = t? i-2: i-3;
 }
 
 /* Standard left-to-right signed double-and-add to compute [n]P. */
