@@ -513,7 +513,7 @@ Flxq_ellgroup(GEN a4, GEN a6, GEN N, GEN T, ulong p, GEN *pt_m)
   struct _FlxqE e;
   GEN q = powuu(p, get_Flx_degree(T));
   e.a4=a4; e.a6=a6; e.T=T; e.p=p;
-  return gen_ellgroup(N, subis(q,1), pt_m, (void*)&e, &FlxqE_group, _FlxqE_pairorder);
+  return gen_ellgroup(N, subiu(q,1), pt_m, (void*)&e, &FlxqE_group, _FlxqE_pairorder);
 }
 
 GEN
@@ -525,6 +525,8 @@ Flxq_ellgens(GEN a4, GEN a6, GEN ch, GEN D, GEN m, GEN T, ulong p)
   e.a4=a4; e.a6=a6; e.T=T; e.p=p;
   switch(lg(D)-1)
   {
+  case 0:
+    return cgetg(1,t_VEC);
   case 1:
     P = gen_gener(gel(D,1), (void*)&e, &FlxqE_group);
     P = mkvec(FlxqE_changepoint(P, ch, T, p));
@@ -1018,8 +1020,7 @@ get_Kohel_polynomials(ulong p, GEN *act, long *dj)
     *dj = 8;
     return fill_pols(13,mat13,11,num13,den13,act);
   }
-  *dj=0; *act = NULL;
-  return NULL;
+  *dj=0; *act = NULL; return NULL; /* LCOV_EXCL_LINE */
 }
 
 long
@@ -1088,8 +1089,8 @@ Flxq_ellcard_Kohel(GEN a4, GEN a6, GEN T, ulong p)
     if (DEBUGLEVEL) timer_printf(&ti,"Teichmuller/Fp");
     Nc2 = Fp_mul(Nc2,tNc2,q);
   }
-  t = Fp_center(Fp_mul(Nc2,Np,q),q,shifti(q,-1));
-  return gerepileupto(av, subii(addis(powuu(p,n),1),t));
+  t = Fp_center_i(Fp_mul(Nc2,Np,q),q,shifti(q,-1));
+  return gerepileupto(av, subii(addiu(powuu(p,n),1),t));
 }
 
 static void
@@ -1123,7 +1124,7 @@ getc5(GEN H, GEN A40, GEN A60, GEN A41, GEN A61, GEN T, GEN q, ulong p, long N)
 static GEN
 ZpXQX_liftrootmod_vald(GEN f, GEN H, long v, GEN T, GEN p, long e)
 {
-  pari_sp av = avma, av2, lim;
+  pari_sp av = avma, av2;
   GEN pv = p, q, qv, W, df, Tq, fr, dfr;
   ulong mask;
   pari_timer ti;
@@ -1136,8 +1137,7 @@ ZpXQX_liftrootmod_vald(GEN f, GEN H, long v, GEN T, GEN p, long e)
   if (DEBUGLEVEL) timer_start(&ti);
   W = FpXQXQ_inv(FpXQX_rem(dfr, H, Tq, p), H, Tq, p); /* 1/f'(a) mod (T,p) */
   if (DEBUGLEVEL) timer_printf(&ti,"FpXQXQ_inv");
-  q = p;
-  av2 = avma; lim = stack_lim(av2, 2);
+  q = p; av2 = avma;
   for (;;)
   {
     GEN u, fa, qv, q2v, Tq2, fadH;
@@ -1157,7 +1157,7 @@ ZpXQX_liftrootmod_vald(GEN f, GEN H, long v, GEN T, GEN p, long e)
     dfr = FpXQX_rem(FpXQX_red(df, Tq, q),H,Tq,q);
     u = ZXX_Z_divexact(ZXX_Z_add_shallow(FpXQXQ_mul(W,dfr,H,Tq,q),gen_m1),q2);
     W = gsub(W,gmul(FpXQXQ_mul(u,W,H2,Tq2,q2),q2));
-    if (low_stack(lim, stack_lim(av2,2)))
+    if (gc_needed(av2,2))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"ZpXQX_liftroot, e = %ld", e);
       gerepileall(av2, 3, &H, &W, &q);
@@ -1237,8 +1237,8 @@ Flxq_ellcard_Harley(GEN a4, GEN a6, GEN T, ulong p)
     if (DEBUGLEVEL) timer_printf(&ti,"Teichmuller/Fp");
     Nc2 = Fp_mul(Nc2,tNc2,q);
   }
-  t = Fp_center(Fp_mul(Nc2,Np,q),q,shifti(q,-1));
-  return gerepileupto(av, subii(addis(powuu(p,n),1),t));
+  t = Fp_center_i(Fp_mul(Nc2,Np,q),q,shifti(q,-1));
+  return gerepileupto(av, subii(addiu(powuu(p,n),1),t));
 }
 
 /***************************************************************************/
@@ -1423,8 +1423,8 @@ Flxq_ellcard_Shanks(GEN a4, GEN a6, GEN q, GEN T, ulong p)
   pari_sp av = avma;
   long vn = get_Flx_var(T), n = get_Flx_degree(T), KRO = -1;
   GEN h,f, ta4, A, B, m;
-  GEN q1p = addsi(1, q), q2p = shifti(q1p, 1);
-  GEN bound = addis(sqrti(gmul2n(q,4)), 1); /* ceil( 4sqrt(q) ) */
+  GEN q1p = addiu(q,1), q2p = shifti(q1p, 1);
+  GEN bound = addiu(sqrti(gmul2n(q,4)), 1); /* ceil( 4sqrt(q) ) */
   /* once #E(Flxq) is know mod B >= bound, it is completely determined */
   /* how many 2-torsion points ? */
   switch(FlxqX_nbroots(mkpoln(4, pol1_Flx(vn), pol0_Flx(vn), a4, a6), T, p))
@@ -1467,7 +1467,7 @@ F3xq_ellcard(GEN a2, GEN a6, GEN T)
     return utoi(F3xq_ellcard_naive(a2, a6, T));
   else
   {
-    GEN q1 = addis(powuu(3, get_Flx_degree(T)), 1), t;
+    GEN q1 = addiu(powuu(3, get_Flx_degree(T)), 1), t;
     GEN a = Flxq_div(a6,Flxq_powu(a2,3,T,3),T,3);
     if (Flx_equal1(Flxq_powu(a, 8, T, 3)))
     {
@@ -1500,7 +1500,7 @@ Flxq_ellcard_Satoh(GEN a4, GEN a6, GEN j, GEN T, ulong p)
       if (degpol(m) <= 0)
       {
         GEN q = sqru(p);
-        GEN q1 = addis(powuu(p, get_Flx_degree(T)), 1);
+        GEN q1 = addiu(powuu(p, get_Flx_degree(T)), 1);
         GEN sk = Flx_Fl_add(Flx_neg(j, p), 1728%p, p);
         GEN sA4 = Flx_triple(Flxq_mul(sk, j, T, p), p);
         GEN u = Flxq_div(a4, sA4, T, p);
@@ -1529,8 +1529,8 @@ Flxq_ellcard_Kedlaya(GEN a4, GEN a6, GEN T, ulong p)
   GEN N = ZpXQM_prodFrobenius(M, Tp, utoi(p), e);
   GEN q = powuu(p, e);
   GEN tp = Fq_add(gcoeff(N,1,1), gcoeff(N,2,2), Tp, q);
-  GEN t = Fp_center(typ(tp)==t_INT ? tp: leading_coeff(tp), q, shifti(q,-1));
-  return gerepileupto(av, subii(addis(powuu(p, n), 1), t));
+  GEN t = Fp_center_i(typ(tp)==t_INT ? tp: leading_coeff(tp), q, shifti(q,-1));
+  return gerepileupto(av, subii(addiu(powuu(p, n), 1), t));
 }
 
 GEN
@@ -1578,7 +1578,7 @@ F3xq_ellcardj(GEN a4, GEN a6, GEN T, GEN q, long n)
 {
   const ulong p = 3;
   ulong t;
-  GEN q1 = addis(q,1);
+  GEN q1 = addiu(q,1);
   GEN na4 = Flx_neg(a4,p), ra4;
   if (!Flxq_issquare(na4,T,p))
     return q1;
@@ -1605,7 +1605,7 @@ F3xq_ellcardj(GEN a4, GEN a6, GEN T, GEN q, long n)
 static GEN
 Flxq_ellcardj(GEN a4, GEN a6, ulong j, GEN T, GEN q, ulong p, long n)
 {
-  GEN q1 = addis(q,1);
+  GEN q1 = addiu(q,1);
   if (j==0)
   {
     ulong w;
