@@ -111,7 +111,7 @@ cindex_Flx(long c, long d, ulong p, long v)
 static GEN
 factorel(GEN h, ulong p)
 {
-  GEN F = Flx_factcantor(h, p, 0);
+  GEN F = Flx_factor(h, p);
   GEN F1 = gel(F, 1), F2 = gel(F, 2);
   long i, l1 = lg(F1)-1;
   GEN p2 = cgetg(l1+1, t_VECSMALL);
@@ -169,7 +169,7 @@ Flxq_log_cubic(struct Flxq_log_rel *r, GEN C, GEN R, ulong p)
   GEN a = zero_zv(l); /*We allocate one extra word to catch overflow*/
   GEN b = zero_zv(l);
   pari_sp av = avma;
-  long i,j,k, dh=0;
+  long i,j,k;
   for(i=0; ; i++, Flx_cnext(a, p))
   {
     Flx_renormalize_inplace(a, l+1);
@@ -189,7 +189,6 @@ Flxq_log_cubic(struct Flxq_log_rel *r, GEN C, GEN R, ulong p)
       pabc2= Flx_sub(pab,Flx_sqr(c,p),p);
       h = Flx_add(R,Flx_add(Flx_mul(C,pabc2,p),pabc,p), p);
       h = Flx_normalize(h, p);
-      dh = maxss(dh,degpol(h));
       if (Flx_addifsmooth3(&av, r, h, i, j, k, p)) return;
     }
   }
@@ -246,7 +245,7 @@ smoothness_vec(ulong p, long r, long n)
       pari_sp av2 = avma;
       if (l*k == j)
       {
-        s = binomial(addis(Ik,l-1), l);
+        s = binomial(addiu(Ik,l-1), l);
         l--;
       }
       for (i = 0; i <= l; ++i)
@@ -333,22 +332,25 @@ check_kernel(long r, GEN M, long nbi, long nbrow, GEN T, ulong p, GEN m)
   pari_sp av = avma;
   long N = 3*upowuu(p, r);
   GEN K = FpMs_leftkernel_elt(M, nbrow, m);
-  long i, f=0;
+  long i, f=0, tbs;
   long lm = lgefint(m), u=1;
-  GEN g;
-  GEN idx = diviiexact(subis(powuu(p,degpol(T)),1),m);
+  GEN tab, g;
+  GEN q = powuu(p,degpol(T));
+  GEN idx = diviiexact(subiu(q,1),m);
   pari_timer ti;
   if (DEBUGLEVEL) timer_start(&ti);
   while (signe(gel(K,u))==0)
-   u++;
+    u++;
   K = FpC_Fp_mul(K, Fp_inv(gel(K, u), m), m);
   g = Flxq_pow(cindex_Flx(u, r, p, T[1]), idx, T, p);
+  tbs = maxss(1, expu(nbi/expi(m)));
+  tab = Flxq_pow_init(g, q, tbs, T, p);
   setlg(K, N);
   for (i=1; i<N; i++)
   {
     GEN k = gel(K,i);
     pari_sp av = avma;
-    long t = signe(k) && Flx_equal(Flxq_pow(g, k, T, p),
+    long t = signe(k) && Flx_equal(Flxq_pow_table(tab, k, T, p),
                                    Flxq_pow(cindex_Flx(i,r,p,T[1]), idx, T, p));
     avma = av;
     if (!t)
@@ -475,57 +477,90 @@ rel_Coppersmith(long r, GEN u, GEN v, long h, GEN R, long d, ulong p)
   return famatsmall_reduce(M);
 }
 
+GEN
+Flxq_log_Coppersmith_worker(GEN u, long i, GEN V, GEN R)
+{
+  long r = V[1], h = V[2], d = V[3], p = V[4], dT = V[5];
+  pari_sp ltop = avma;
+  GEN v = zero_zv(dT+2);
+  GEN L = cgetg(2*i+1, t_VEC);
+  pari_sp av = avma;
+  long j;
+  long nbtest=0, rel = 1;
+  ulong lu = Flx_lead(u), lv;
+  for (j=1; j<=i; j++)
+  {
+    GEN z;
+    Flx_cnext(v, p);
+    Flx_renormalize_inplace(v, dT+2);
+    lv = Flx_lead(v);
+    avma = av;
+    if (lu != 1 && lv != 1) continue;
+    if (degpol(Flx_gcd(u, v, p))!=0) continue;
+    if (lu==1)
+    {
+      z = rel_Coppersmith(r, u, v, h, R, d, p);
+      nbtest++;
+      if (z) { gel(L, rel++) = z; av = avma; }
+    }
+    if (i==j) continue;
+    if (lv==1)
+    {
+      z = rel_Coppersmith(r, v, u, h, R, d, p);
+      nbtest++;
+      if (z) { gel(L, rel++) = z; av = avma; }
+    }
+  }
+  setlg(L,rel);
+  return gerepilecopy(ltop, mkvec2(stoi(nbtest), L));
+}
+
 static GEN
 Flxq_log_Coppersmith(long nbrel, long r, GEN T, ulong p)
 {
+  pari_sp av;
   long dT = degpol(T);
   long h = dT/p, d = dT-(h*p);
   GEN R = Flx_sub(Flx_shift(pol1_Flx(T[1]), dT), T, p);
-  GEN u = zero_zv(dT+2), v = zero_zv(dT+2);
+  GEN u = zero_zv(dT+2);
+  GEN done;
   long nbtest = 0, rel = 0;
   GEN M = cgetg(nbrel+1, t_VEC);
-  pari_sp av = avma;
-  long i, j;
+  long i = 1;
+  GEN worker = snm_closure(is_entry("_Flxq_log_Coppersmith_worker"),
+               mkvec2(mkvecsmall5(r,h,d,p,dT), R));
+  struct pari_mt pt;
+  long running, pending = 0, stop=0;
   if (DEBUGLEVEL) err_printf("Coppersmith (R = %ld): ",degpol(R));
-  for (i=1; ; i++)
+  mt_queue_start(&pt, worker);
+  av = avma;
+  while ((running = !stop) || pending)
   {
+    GEN L;
+    long l, j;
     Flx_cnext(u, p);
     Flx_renormalize_inplace(u, dT+2);
-    for(j=2; j<lg(v); j++) v[j] = 0;
-    for(j=1; j<i; j++)
+    mt_queue_submit(&pt, 0, running ? mkvec2(u, stoi(i)): NULL);
+    done = mt_queue_get(&pt, NULL, &pending);
+    if (!done) continue;
+    L = gel(done, 2); nbtest += itos(gel(done,1));
+    l = lg(L);
+    if (l > 1)
     {
-      GEN z;
-      Flx_cnext(v, p);
-      Flx_renormalize_inplace(v, dT+2);
-      avma = av;
-      if (Flx_lead(u)==1)
+      for (j=1; j<l; j++)
       {
-        z = rel_Coppersmith(r, u, v, h, R, d, p);
-        nbtest++;
-        if (z)
-        {
-          gel(M,++rel) = gerepilecopy(av, z); av = avma;
-          if (DEBUGLEVEL && (rel&511UL)==0)
-            err_printf("%ld%%[%ld] ",rel*100/nbrel,i);
-        }
-        if (rel>nbrel) break;
-      }
-      if (i==j) continue;
-      if (Flx_lead(v)==1)
-      {
-        z = rel_Coppersmith(r, v, u, h, R, d, p);
-        nbtest++;
-        if (z)
-        {
-          gel(M,++rel) = gerepilecopy(av, z); av = avma;
-          if (DEBUGLEVEL && (rel&511UL)==0)
-            err_printf("%ld%%[%ld] ",rel*100/nbrel,i);
-        }
         if (rel>nbrel) break;
+        gel(M,++rel) = gel(L,j);
+        if (DEBUGLEVEL && (rel&511UL)==0)
+          err_printf("%ld%%[%ld] ",rel*100/nbrel,i);
       }
+      av = avma;
     }
-    if (rel>nbrel) break;
+    else avma = av;
+    if (rel>nbrel) stop = 1;
+    i++;
   }
+  mt_queue_end(&pt);
   if (DEBUGLEVEL) err_printf(": %ld tests\n", nbtest);
   return M;
 }
@@ -663,7 +698,6 @@ Flxq_log_Coppersmith_rec(GEN W, long r2, GEN a, long r, GEN T, ulong p, GEN m)
   }
 }
 
-
 static GEN
 Flxq_log_index_Coppersmith(GEN a0, GEN b0, GEN m, GEN T0, ulong p)
 {
