@@ -1829,9 +1829,9 @@ GEN
 FpXQ_autpow(GEN x, ulong n, GEN T, GEN p)
 {
   struct _FpXQ D;
+  if (n==0) return FpX_rem(pol_x(varn(x)), T, p);
+  if (n==1) return FpX_rem(x, T, p);
   D.T = FpX_get_red(T, p); D.p = p;
-  if (n==0) return pol_x(varn(x));
-  if (n==1) return ZX_copy(x);
   return gen_powu(x,n,(void*)&D,FpXQ_autpow_sqr,FpXQ_autpow_mul);
 }
 
@@ -2010,9 +2010,10 @@ Fq_ispower(GEN x, GEN K, GEN T, GEN p)
   pari_sp av = avma;
   long d;
   GEN Q;
-  if (!T) return Fp_ispower(x,K,p);
+  if (equaliu(K,2)) return Fq_issquare(x, T, p);
+  if (!T) return Fp_ispower(x, K, p);
   d = get_FpX_degree(T);
-  if (!umodui(d, K)) return 1;
+  if (typ(x) == t_INT && !umodui(d, K)) return 1;
   Q = subiu(powiu(p,d), 1);
   Q = diviiexact(Q, gcdii(Q, K));
   d = gequal1(Fq_pow(x, Q, T,p));
