@@ -152,6 +152,19 @@ FpXX_mulu(GEN P, ulong u, GEN p)
   return FpXX_renormalize(res,lP);
 }
 
+GEN
+FpXX_halve(GEN P, GEN p)
+{
+  long i, lP;
+  GEN res = cgetg_copy(P, &lP); res[1] = P[1];
+  for(i=2; i<lP; i++)
+  {
+    GEN x = gel(P,i);
+    gel(res,i) = typ(x)==t_INT? Fp_halve(x,p): FpX_halve(x,p);
+  }
+  return FpXX_renormalize(res,lP);
+}
+
 GEN
 FpXX_deriv(GEN P, GEN p)
 {
@@ -169,6 +182,25 @@ FpXX_deriv(GEN P, GEN p)
   return FpXX_renormalize(res, l);
 }
 
+GEN
+FpXX_integ(GEN P, GEN p)
+{
+  long i, l = lg(P);
+  GEN res;
+
+  if (l == 2) return pol_0(varn(P));
+  res = cgetg(l+1, t_POL);
+  res[1] = P[1];
+  gel(res,2) = gen_0;
+  for (i=3; i<=l ; i++)
+  {
+    GEN x = gel(P,i-1);
+    GEN i1 = Fp_inv(utoi(i-2), p);
+    gel(res,i) = typ(x)==t_INT? Fp_mul(x,i1,p): FpX_Fp_mul(x,i1,p);
+  }
+  return FpXX_renormalize(res, l+1);
+}
+
 /*******************************************************************/
 /*                                                                 */
 /*                             (Fp[X]/(Q))[Y]                      */
@@ -182,15 +214,6 @@ get_FpXQX_red(GEN T, GEN *B)
   *B = gel(T,1); return gel(T,2);
 }
 
-GEN
-get_FpXQX_mod(GEN T) { return typ(T)==t_VEC? gel(T,2): T; }
-
-long
-get_FpXQX_var(GEN T) { return typ(T)==t_VEC? varn(gel(T,2)): varn(T); }
-
-long
-get_FpXQX_degree(GEN T) { return typ(T)==t_VEC? degpol(gel(T,2)): degpol(T); }
-
 GEN
 random_FpXQX(long d1, long v, GEN T, GEN p)
 {
@@ -267,6 +290,23 @@ FpXQX_red(GEN z, GEN T, GEN p)
   return FpXQX_renormalize(res,l);
 }
 
+/* z in Z[X], return z * Mod(1,p), normalized*/
+GEN
+FpXQX_to_mod(GEN z, GEN T, GEN p)
+{
+  long i,l = lg(z);
+  GEN x = cgetg(l, t_POL);
+  x[1] = z[1];
+  if (l == 2) return x;
+  T = FpX_to_mod(T, p);
+  for (i=2; i<l; i++)
+  {
+    GEN zi = gel(z,i);
+    gel(x,i) = typ(zi) == t_POL? mkpolmod(FpX_to_mod(zi, p), T): icopy(zi);
+  }
+  return normalizepol_lg(x,l);
+}
+
 static int
 ZXX_is_ZX_spec(GEN a,long na)
 {
@@ -702,6 +742,18 @@ FpXQX_extgcd(GEN x, GEN y, GEN T, GEN p, GEN *ptu, GEN *ptv)
   return d;
 }
 
+GEN
+FpXQX_dotproduct(GEN x, GEN y, GEN T, GEN p)
+{
+  long i, l = minss(lg(x), lg(y));
+  pari_sp av;
+  GEN c;
+  if (l == 2) return gen_0;
+  av = avma; c = gmul(gel(x,2),gel(y,2));
+  for (i=3; i<l; i++) c = gadd(c, gmul(gel(x,i),gel(y,i)));
+  return gerepileupto(av, Fq_red(c,T,p));
+}
+
 /***********************************************************************/
 /**                                                                   **/
 /**                       Barrett reduction                           **/
@@ -736,7 +788,7 @@ FpXQX_invBarrett_basecase(GEN S, GEN T, GEN p)
 }
 
 INLINE GEN
-FpXQX_recipspec(GEN x, long l, long n)
+FpXX_recipspec(GEN x, long l, long n)
 {
   return RgX_recipspec_shallow(x, l, n);
 }
@@ -840,9 +892,9 @@ FpXQX_divrem_Barrettspec(GEN x, long l, GEN mg, GEN S, GEN T, GEN p, GEN *pr)
   lm = minss(ld, lgpol(mg));
   lT  = ZXX_lgrenormalizespec(S+2,lt);
   lmg = ZXX_lgrenormalizespec(mg+2,lm);
-  q = FpXQX_recipspec(x+lt,ld,ld);                 /* q = rec(x)     lq<=ld*/
+  q = FpXX_recipspec(x+lt,ld,ld);                 /* q = rec(x)     lq<=ld*/
   q = FpXQX_mulspec(q+2,mg+2,T,p,lgpol(q),lmg);    /* q = rec(x) * mg lq<=ld+lm*/
-  q = FpXQX_recipspec(q+2,minss(ld,lgpol(q)),ld);  /* q = rec (rec(x) * mg) lq<=ld*/
+  q = FpXX_recipspec(q+2,minss(ld,lgpol(q)),ld);  /* q = rec (rec(x) * mg) lq<=ld*/
   if (!pr) return q;
   r = FpXQX_mulspec(q+2,S+2,T,p,lgpol(q),lT);      /* r = q*pol        lr<=ld+lt*/
   r = FpXX_subspec(x,r+2,p,lt,minss(lt,lgpol(r))); /* r = x - r   lr<=lt */
@@ -1079,7 +1131,7 @@ FpXQX_digits(GEN x, GEN B, GEN T, GEN p)
 }
 
 GEN
-FpXQX_fromdigits(GEN x, GEN B, GEN T, GEN p)
+FpXQXV_FpXQX_fromdigits(GEN x, GEN B, GEN T, GEN p)
 {
   pari_sp av = avma;
   struct _FpXQXQ D;
@@ -1352,6 +1404,106 @@ FpXQXQ_powers(GEN x, long l, GEN S, GEN T, GEN p)
   return gen_powers(x, l, use_sqr, (void*)&D, &_FpXQXQ_sqr, &_FpXQXQ_mul,&_FpXQXQ_one);
 }
 
+/* Let v a linear form, return the linear form z->v(tau*z)
+   that is, v*(M_tau) */
+
+INLINE GEN
+FpXQX_recipspec(GEN x, long l, long n)
+{
+  return RgX_recipspec_shallow(x, l, n);
+}
+
+static GEN
+FpXQXQ_transmul_init(GEN tau, GEN S, GEN T, GEN p)
+{
+  GEN bht;
+  GEN h, Sp = get_FpXQX_red(S, &h);
+  long n = degpol(Sp), vT = varn(Sp);
+  GEN ft = FpXQX_recipspec(Sp+2, n+1, n+1);
+  GEN bt = FpXQX_recipspec(tau+2, lgpol(tau), n);
+  setvarn(ft, vT); setvarn(bt, vT);
+  if (h)
+    bht = FpXQXn_mul(bt, h, n-1, T, p);
+  else
+  {
+    GEN bh = FpXQX_div(RgX_shift_shallow(tau, n-1), S, T, p);
+    bht = FpXQX_recipspec(bh+2, lgpol(bh), n-1);
+    setvarn(bht, vT);
+  }
+  return mkvec3(bt, bht, ft);
+}
+
+static GEN
+FpXQXQ_transmul(GEN tau, GEN a, long n, GEN T, GEN p)
+{
+  pari_sp ltop = avma;
+  GEN t1, t2, t3, vec;
+  GEN bt = gel(tau, 1), bht = gel(tau, 2), ft = gel(tau, 3);
+  if (signe(a)==0) return pol_0(varn(a));
+  t2 = RgX_shift_shallow(FpXQX_mul(bt, a, T, p),1-n);
+  if (signe(bht)==0) return gerepilecopy(ltop, t2);
+  t1 = RgX_shift_shallow(FpXQX_mul(ft, a, T, p),-n);
+  t3 = FpXQXn_mul(t1, bht, n-1, T, p);
+  vec = FpXX_sub(t2, RgX_shift_shallow(t3, 1), p);
+  return gerepileupto(ltop, vec);
+}
+
+static GEN
+polxn_FpXX(long n, long v, long vT)
+{
+  long i, a = n+2;
+  GEN p = cgetg(a+1, t_POL);
+  p[1] = evalsigne(1)|evalvarn(v);
+  for (i = 2; i < a; i++) gel(p,i) = pol_0(vT);
+  gel(p,a) = pol_1(vT); return p;
+}
+
+GEN
+FpXQXQ_minpoly(GEN x, GEN S, GEN T, GEN p)
+{
+  pari_sp ltop = avma;
+  long vS, vT, n;
+  GEN v_x, g, tau;
+  vS = get_FpXQX_var(S);
+  vT = get_FpX_var(T);
+  n = get_FpXQX_degree(S);
+  g = pol_1(vS);
+  tau = pol_1(vS);
+  S = FpXQX_get_red(S, T, p);
+  v_x = FpXQXQ_powers(x, usqrt(2*n), S, T, p);
+  while(signe(tau) != 0)
+  {
+    long i, j, m, k1;
+    GEN M, v, tr;
+    GEN g_prime, c;
+    if (degpol(g) == n) { tau = pol_1(vS); g = pol_1(vS); }
+    v = random_FpXQX(n, vS, T, p);
+    tr = FpXQXQ_transmul_init(tau, S, T, p);
+    v = FpXQXQ_transmul(tr, v, n, T, p);
+    m = 2*(n-degpol(g));
+    k1 = usqrt(m);
+    tr = FpXQXQ_transmul_init(gel(v_x,k1+1), S, T, p);
+    c = cgetg(m+2,t_POL);
+    c[1] = evalsigne(1)|evalvarn(vS);
+    for (i=0; i<m; i+=k1)
+    {
+      long mj = minss(m-i, k1);
+      for (j=0; j<mj; j++)
+        gel(c,m+1-(i+j)) = FpXQX_dotproduct(v, gel(v_x,j+1), T, p);
+      v = FpXQXQ_transmul(tr, v, n, T, p);
+    }
+    c = FpXX_renormalize(c, m+2);
+    /* now c contains <v,x^i> , i = 0..m-1  */
+    M = FpXQX_halfgcd(polxn_FpXX(m, vS, vT), c, T, p);
+    g_prime = gmael(M, 2, 2);
+    if (degpol(g_prime) < 1) continue;
+    g = FpXQX_mul(g, g_prime, T, p);
+    tau = FpXQXQ_mul(tau, FpXQX_FpXQXQV_eval(g_prime, v_x, S, T, p), S, T, p);
+  }
+  g = FpXQX_normalize(g,T, p);
+  return gerepilecopy(ltop,g);
+}
+
 GEN
 FpXQXQ_matrix_pow(GEN y, long n, long m, GEN S, GEN T, GEN p)
 {
@@ -1385,13 +1537,13 @@ static GEN
 FpXQXQ_autpow_sqr(void * E, GEN x)
 {
   struct _FpXQXQ *D = (struct _FpXQXQ *)E;
-  GEN T = D->T, p = D->p;
-  GEN phi = gel(x,1), S = gel(x,2);
-  long n = brent_kung_optpow(get_FpX_degree(T)-1,lgpol(S)+1,1);
+  GEN S = D->S, T = D->T, p = D->p;
+  GEN phi = gel(x,1), S1 = gel(x,2);
+  long n = brent_kung_optpow(get_FpX_degree(T)-1,lgpol(S1)+1,1);
   GEN V = FpXQ_powers(phi, n, T, p);
   GEN phi2 = FpX_FpXQV_eval(phi, V, T, p);
-  GEN Sphi = FpXY_FpXQV_evalx(S, V, T, p);
-  GEN S2 = FpXQX_FpXQXQ_eval(Sphi, S, D->S, T, p);
+  GEN Sphi = FpXY_FpXQV_evalx(S1, V, T, p);
+  GEN S2 = FpXQX_FpXQXQ_eval(Sphi, S1, S, T, p);
   return mkvec2(phi2, S2);
 }
 
@@ -1399,19 +1551,19 @@ static GEN
 FpXQXQ_autpow_mul(void * E, GEN x, GEN y)
 {
   struct _FpXQXQ *D = (struct _FpXQXQ *)E;
-  GEN T = D->T, p = D->p;
+  GEN S = D->S, T = D->T, p = D->p;
   GEN phi1 = gel(x,1), S1 = gel(x,2);
   GEN phi2 = gel(y,1), S2 = gel(y,2);
   long n = brent_kung_optpow(get_FpX_degree(T)-1, lgpol(S1)+1, 1);
   GEN V = FpXQ_powers(phi2, n, T, p);
   GEN phi3 = FpX_FpXQV_eval(phi1, V, T, p);
   GEN Sphi = FpXY_FpXQV_evalx(S1, V, T, p);
-  GEN S3 = FpXQX_FpXQXQ_eval(Sphi, S2, D->S, T, p);
+  GEN S3 = FpXQX_FpXQXQ_eval(Sphi, S2, S, T, p);
   return mkvec2(phi3, S3);
 }
 
 GEN
-FpXQXQV_autpow(GEN aut, long n, GEN S, GEN T, GEN p)
+FpXQXQ_autpow(GEN aut, long n, GEN S, GEN T, GEN p)
 {
   struct _FpXQXQ D;
   T = FpX_get_red(T, p);
@@ -1423,21 +1575,17 @@ FpXQXQV_autpow(GEN aut, long n, GEN S, GEN T, GEN p)
 static GEN
 FpXQXQ_auttrace_mul(void *E, GEN x, GEN y)
 {
-  struct _FpXQXQ *D = (struct _FpXQXQ *) E;
-  GEN T = D->T, p = D->p;
-  GEN phi1 = gel(x,1), S1 = gel(x,2), a1 = gel(x,3);
-  GEN phi2 = gel(y,1), S2 = gel(y,2), a2 = gel(y,3);
-  long n2 = brent_kung_optpow(get_FpX_degree(T)-1, lgpol(S1)+lgpol(a1)+1, 1);
-  GEN V2 = FpXQ_powers(phi2, n2, T, p);
-  GEN phi3 = FpX_FpXQV_eval(phi1, V2, T, p);
-  GEN Sphi = FpXY_FpXQV_evalx(S1, V2, T, p);
-  GEN aphi = FpXY_FpXQV_evalx(a1, V2, T, p);
-  long n = brent_kung_optpow(maxss(degpol(Sphi),degpol(aphi)),2,1);
-  GEN V = FpXQXQ_powers(S2, n, D->S, T, p);
-  GEN S3 = FpXQX_FpXQXQV_eval(Sphi, V, D->S, T, p);
-  GEN aS = FpXQX_FpXQXQV_eval(aphi, V, D->S, T, p);
+  struct _FpXQXQ *D = (struct _FpXQXQ *)E;
+  GEN S = D->S, T = D->T;
+  GEN p = D->p;
+  GEN S1 = gel(x,1), a1 = gel(x,2);
+  GEN S2 = gel(y,1), a2 = gel(y,2);
+  long n = brent_kung_optpow(maxss(degpol(S1),degpol(a1)),2,1);
+  GEN V = FpXQXQ_powers(S2, n, S, T, p);
+  GEN S3 = FpXQX_FpXQXQV_eval(S1, V, S, T, p);
+  GEN aS = FpXQX_FpXQXQV_eval(a1, V, S, T, p);
   GEN a3 = FpXX_add(aS, a2, p);
-  return mkvec3(phi3, S3, a3);
+  return mkvec2(S3, a3);
 }
 
 static GEN
@@ -1445,7 +1593,7 @@ FpXQXQ_auttrace_sqr(void *E, GEN x)
 { return FpXQXQ_auttrace_mul(E, x, x); }
 
 GEN
-FpXQXQV_auttrace(GEN aut, long n, GEN S, GEN T, GEN p)
+FpXQXQ_auttrace(GEN aut, long n, GEN S, GEN T, GEN p)
 {
   struct _FpXQXQ D;
   T = FpX_get_red(T, p);
@@ -1458,7 +1606,7 @@ static GEN
 FpXQXQ_autsum_mul(void *E, GEN x, GEN y)
 {
   struct _FpXQXQ *D = (struct _FpXQXQ *) E;
-  GEN T = D->T, p = D->p;
+  GEN S = D->S, T = D->T, p = D->p;
   GEN phi1 = gel(x,1), S1 = gel(x,2), a1 = gel(x,3);
   GEN phi2 = gel(y,1), S2 = gel(y,2), a2 = gel(y,3);
   long n2 = brent_kung_optpow(get_FpX_degree(T)-1, lgpol(S1)+lgpol(a1)+1, 1);
@@ -1467,10 +1615,10 @@ FpXQXQ_autsum_mul(void *E, GEN x, GEN y)
   GEN Sphi = FpXY_FpXQV_evalx(S1, V2, T, p);
   GEN aphi = FpXY_FpXQV_evalx(a1, V2, T, p);
   long n = brent_kung_optpow(maxss(degpol(Sphi),degpol(aphi)),2,1);
-  GEN V = FpXQXQ_powers(S2, n, D->S, T, p);
-  GEN S3 = FpXQX_FpXQXQV_eval(Sphi, V, D->S, T, p);
-  GEN aS = FpXQX_FpXQXQV_eval(aphi, V, D->S, T, p);
-  GEN a3 = FpXQXQ_mul(aS,a2,D->S, T, p);
+  GEN V = FpXQXQ_powers(S2, n, S, T, p);
+  GEN S3 = FpXQX_FpXQXQV_eval(Sphi, V, S, T, p);
+  GEN aS = FpXQX_FpXQXQV_eval(aphi, V, S, T, p);
+  GEN a3 = FpXQXQ_mul(aS, a2, S, T, p);
   return mkvec3(phi3, S3, a3);
 }
 
@@ -1479,7 +1627,7 @@ FpXQXQ_autsum_sqr(void * T, GEN x)
 { return FpXQXQ_autsum_mul(T,x,x); }
 
 GEN
-FpXQXQV_autsum(GEN aut, long n, GEN S, GEN T, GEN p)
+FpXQXQ_autsum(GEN aut, long n, GEN S, GEN T, GEN p)
 {
   struct _FpXQXQ D;
   T = FpX_get_red(T, p);
@@ -1487,3 +1635,120 @@ FpXQXQV_autsum(GEN aut, long n, GEN S, GEN T, GEN p)
   D.S=S; D.T=T; D.p=p;
   return gen_powu(aut,n,&D,FpXQXQ_autsum_sqr,FpXQXQ_autsum_mul);
 }
+
+GEN
+FpXQXn_mul(GEN x, GEN y, long n, GEN T, GEN p)
+{
+  pari_sp av = avma;
+  GEN z, kx, ky;
+  long d;
+  if (ZXX_is_ZX(y) && ZXX_is_ZX(x))
+    return FpXn_mul(x,y,n,p);
+  d = get_FpX_degree(T);
+  kx = ZXX_to_Kronecker(x, d);
+  ky = ZXX_to_Kronecker(y, d);
+  z = Kronecker_to_FpXQX(ZXn_mul(ky,kx,(2*d-1)*n), T, p);
+  return gerepileupto(av, z);
+}
+
+GEN
+FpXQXn_sqr(GEN x, long n, GEN T, GEN p)
+{
+  pari_sp av = avma;
+  GEN z, kx;
+  long d;
+  if (ZXX_is_ZX(x)) return ZXn_sqr(x, n);
+  d = get_FpX_degree(T);
+  kx= ZXX_to_Kronecker(x, d);
+  z = Kronecker_to_FpXQX(ZXn_sqr(kx, (2*d-1)*n), T, p);
+  return gerepileupto(av, z);
+}
+
+INLINE GEN
+FpXXn_red(GEN a, long n)
+{ return RgXn_red_shallow(a, n); }
+
+GEN
+FpXQXn_exp(GEN h, long e, GEN T, GEN p)
+{
+  pari_sp av = avma, av2;
+  long v = varn(h), n=1;
+  GEN f = pol_1(v), g = pol_1(v);
+  ulong mask = quadratic_prec_mask(e);
+  av2 = avma;
+  if (signe(h)==0 || degpol(h)<1 || !gequal0(gel(h,2)))
+    pari_err_DOMAIN("FpXQXn_exp","valuation", "<", gen_1, h);
+  for (;mask>1;)
+  {
+    GEN q, w;
+    long n2 = n;
+    n<<=1; if (mask & 1) n--;
+    mask >>= 1;
+    g = FpXX_sub(FpXX_mulu(g,2,p), FpXQXn_mul(f, FpXQXn_sqr(g, n2, T, p), n2, T, p), p);
+    q = FpXX_deriv(FpXXn_red(h,n2), p);
+    w = FpXX_add(q, FpXQXn_mul(g, FpXX_sub(FpXX_deriv(f, p), FpXQXn_mul(f,q,n-1, T, p), p),n-1, T, p), p);
+    f = FpXX_add(f, FpXQXn_mul(f, FpXX_sub(FpXXn_red(h, n), FpXX_integ(w, p), p), n, T, p), p);
+    if (gc_needed(av2,2))
+    {
+      if (DEBUGMEM>1) pari_warn(warnmem,"FpXQXn_exp, e = %ld", n);
+      gerepileall(av2, 2, &f, &g);
+    }
+  }
+  return gerepileupto(av, f);
+}
+
+/* (f*g) \/ x^n */
+static GEN
+FpXQX_mulhigh_i(GEN f, GEN g, long n, GEN T, GEN p)
+{
+  return RgX_shift_shallow(FpXQX_mul(f,g,T, p),-n);
+}
+
+static GEN
+FpXQXn_mulhigh(GEN f, GEN g, long n2, long n, GEN T, GEN p)
+{
+  GEN F = RgX_blocks(f, n2, 2), fl = gel(F,1), fh = gel(F,2);
+  return FpXX_add(FpXQX_mulhigh_i(fl, g, n2, T, p), FpXQXn_mul(fh, g, n - n2, T, p), p);
+}
+
+GEN
+FpXQXn_inv(GEN f, long e, GEN T, GEN p)
+{
+  pari_sp av = avma, av2;
+  ulong mask;
+  GEN W, a;
+  long v = varn(f), n = 1;
+
+  if (!signe(f)) pari_err_INV("FpXXn_inv",f);
+  a = Fq_inv(gel(f,2), T, p);
+  if (e == 1) return scalarpol(a, v);
+  else if (e == 2)
+  {
+    GEN b;
+    if (degpol(f) <= 0) return scalarpol(a, v);
+    b = Fq_neg(gel(f,3),T,p);
+    if (signe(b)==0) return scalarpol(a, v);
+    if (!is_pm1(a)) b = Fq_mul(b, Fq_sqr(a, T, p), T, p);
+    W = deg1pol_shallow(b, a, v);
+    return gerepilecopy(av, W);
+  }
+  W = scalarpol_shallow(Fq_inv(gel(f,2), T, p),v);
+  mask = quadratic_prec_mask(e);
+  av2 = avma;
+  for (;mask>1;)
+  {
+    GEN u, fr;
+    long n2 = n;
+    n<<=1; if (mask & 1) n--;
+    mask >>= 1;
+    fr = FpXXn_red(f, n);
+    u = FpXQXn_mul(W, FpXQXn_mulhigh(fr, W, n2, n, T, p), n-n2, T, p);
+    W = FpXX_sub(W, RgX_shift_shallow(u, n2), p);
+    if (gc_needed(av2,2))
+    {
+      if(DEBUGMEM>1) pari_warn(warnmem,"FpXQXn_inv, e = %ld", n);
+      W = gerepileupto(av2, W);
+    }
+  }
+  return gerepileupto(av, W);
+}
