@@ -83,6 +83,13 @@ GEN
 quadgen(GEN x)
 { retmkquad(quadpoly(x), gen_0, gen_1); }
 
+GEN
+quadgen0(GEN x, long v)
+{
+  if (v==-1) v = fetch_user_var("w");
+  retmkquad(quadpoly0(x, v), gen_0, gen_1);
+}
+
 /***********************************************************************/
 /**                                                                   **/
 /**                      BINARY QUADRATIC FORMS                       **/
@@ -128,7 +135,7 @@ qfb_sqr(GEN z, GEN x)
   d1 = bezout(gel(x,2),gel(x,1),&x2, NULL); /* usually 1 */
   c = gel(x,3);
   m = mulii(c,x2);
-  if (is_pm1(d1))
+  if (equali1(d1))
     v1 = v2 = gel(x,1);
   else
   {
@@ -156,13 +163,13 @@ qfb_comp(GEN z, GEN x, GEN y)
   v2 = gel(y,1);
   c  = gel(y,3);
   d = bezout(v2,v1,&y1,NULL);
-  if (is_pm1(d))
+  if (equali1(d))
     m = mulii(y1,n);
   else
   {
     GEN s = subii(gel(y,2), n);
     GEN x2, y2, d1 = bezout(s,d,&x2,&y2); /* x2 s + y2 (x1 v1 + y1 v2) = d1 */
-    if (!is_pm1(d1))
+    if (!equali1(d1))
     {
       v1 = diviiexact(v1,d1);
       v2 = diviiexact(v2,d1); /* gcd = 1 iff x or y primitive */
@@ -217,7 +224,7 @@ qfbcompraw(GEN x, GEN y)
     case t_QFR: return qfrcompraw(x,y);
   }
   pari_err_TYPE("composition",x);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 static GEN
@@ -261,7 +268,7 @@ qfr_1_by_disc(GEN D, long prec)
   check_quaddisc_real(D, &r, "qfr_1_by_disc");
   gel(y,1) = gen_1; isqrtD = sqrti(D);
   if ((r & 1) != mod2(isqrtD)) /* we know isqrtD > 0 */
-    isqrtD = gerepileuptoint(av, addsi(-1,isqrtD));
+    isqrtD = gerepileuptoint(av, subiu(isqrtD,1));
   gel(y,2) = isqrtD; av = avma;
   gel(y,3) = gerepileuptoint(av, shifti(subii(sqri(isqrtD), D),-2));
   gel(y,4) = real_0(prec); return y;
@@ -279,7 +286,7 @@ qfr_1_fill(GEN y, struct qfr_data *S)
   pari_sp av = avma;
   GEN y2 = S->isqrtD;
   gel(y,1) = gen_1;
-  if (mod2(S->D) != mod2(y2)) y2 = addsi(-1,y2);
+  if (mod2(S->D) != mod2(y2)) y2 = subiu(y,1);
   gel(y,2) = y2; av = avma;
   gel(y,3) = gerepileuptoint(av, shifti(subii(sqri(y2), S->D),-2));
 }
@@ -403,31 +410,30 @@ nucomp(GEN x, GEN y, GEN L)
   n = subii(gel(y,2), s);
   a1 = gel(x,1);
   a2 = gel(y,1); d = bezout(a2,a1,&u,&v);
-  if (is_pm1(d)) { a = negi(mulii(u,n)); d1 = d; }
+  if (equali1(d)) { a = negi(mulii(u,n)); d1 = d; }
+  else if (dvdii(s,d)) /* d | s */
+  {
+    a = negi(mulii(u,n)); d1 = d;
+    a1 = diviiexact(a1, d1);
+    a2 = diviiexact(a2, d1);
+    s = diviiexact(s, d1);
+  }
   else
-    if (remii(s,d) == gen_0) /* d | s */
+  {
+    GEN p2, l;
+    d1 = bezout(s,d,&u1,NULL);
+    if (!equali1(d1))
     {
-      a = negi(mulii(u,n)); d1 = d;
-      a1 = diviiexact(a1, d1);
-      a2 = diviiexact(a2, d1);
-      s = diviiexact(s, d1);
-    }
-    else
-    {
-      GEN p2, l;
-      d1 = bezout(s,d,&u1,NULL);
-      if (!is_pm1(d1))
-      {
-        a1 = diviiexact(a1,d1);
-        a2 = diviiexact(a2,d1);
-        s = diviiexact(s,d1);
-        d = diviiexact(d,d1);
-      }
-      p1 = remii(gel(x,3),d);
-      p2 = remii(gel(y,3),d);
-      l = modii(mulii(negi(u1), addii(mulii(u,p1),mulii(v,p2))), d);
-      a = subii(mulii(l,diviiexact(a1,d)), mulii(u,diviiexact(n,d)));
+      a1 = diviiexact(a1,d1);
+      a2 = diviiexact(a2,d1);
+      s = diviiexact(s,d1);
+      d = diviiexact(d,d1);
     }
+    p1 = remii(gel(x,3),d);
+    p2 = remii(gel(y,3),d);
+    l = modii(mulii(negi(u1), addii(mulii(u,p1),mulii(v,p2))), d);
+    a = subii(mulii(l,diviiexact(a1,d)), mulii(u,diviiexact(n,d)));
+  }
   a = modii(a,a1); p1 = subii(a,a1); if (abscmpii(a,p1) > 0) a = p1;
   d = a1; v3 = a; z = parteucl(L, &d,&v3, &v,&v2);
   Q = cgetg(4,t_QFI);
@@ -446,7 +452,7 @@ nucomp(GEN x, GEN y, GEN L)
     q4 = subii(q3,s);
     b2 = addii(q3,q4);
     g = diviiexact(q4,v);
-    if (!is_pm1(d1)) { v2 = mulii(d1,v2); v = mulii(d1,v); b2 = mulii(d1,b2); }
+    if (!equali1(d1)) { v2 = mulii(d1,v2); v = mulii(d1,v); b2 = mulii(d1,b2); }
     gel(Q,1) = addii(mulii(d,b), mulii(e,v));
   }
   q1 = mulii(b, v3);
@@ -467,7 +473,7 @@ nudupl(GEN x, GEN L)
   a = gel(x,1);
   b = gel(x,2);
   d1 = bezout(b,a, &u,NULL);
-  if (!is_pm1(d1))
+  if (!equali1(d1))
   {
     a = diviiexact(a, d1);
     b = diviiexact(b, d1);
@@ -489,7 +495,7 @@ nudupl(GEN x, GEN L)
     e = diviiexact(addii(mulii(gel(x,3),v), mulii(b,d)), a);
     g = diviiexact(subii(mulii(e,v2), b), v);
     b2 = addii(mulii(e,v2), mulii(v,g));
-    if (!is_pm1(d1)) { b2 = mulii(d1,b2); v = mulii(d1,v); v2 = mulii(d1,v2); }
+    if (!equali1(d1)) { b2 = mulii(d1,b2); v = mulii(d1,v); v2 = mulii(d1,v2); }
     gel(Q,1) = addii(a2, mulii(e,v));
   }
   gel(Q,2) = addii(b2, subii(sqri(addii(d,v3)), addii(a2,c2)));
@@ -514,7 +520,7 @@ nupow(GEN x, GEN n, GEN L)
   D = qfb_disc(x);
   y = qfi_1_by_disc(D);
   if (!signe(n)) return y;
-  if (!L) L = sqrtnint(absi(D), 4);
+  if (!L) L = sqrtnint(absi_shallow(D), 4);
   y = gen_pow(x, n, (void*)L, &mul_nudupl, &mul_nucomp);
   if (signe(n) < 0
   && !absequalii(gel(y,1),gel(y,2))
@@ -530,9 +536,9 @@ dvmdii_round(GEN b, GEN a, GEN *r)
 {
   GEN a2 = shifti(a, 1), q = dvmdii(b, a2, r);
   if (signe(b) >= 0) {
-    if (abscmpii(*r, a) > 0) { q = addis(q,  1); *r = subii(*r, a2); }
+    if (abscmpii(*r, a) > 0) { q = addiu(q, 1); *r = subii(*r, a2); }
   } else { /* r <= 0 */
-    if (abscmpii(*r, a) >= 0){ q = addis(q, -1); *r = addii(*r, a2); }
+    if (abscmpii(*r, a) >= 0){ q = subiu(q, 1); *r = addii(*r, a2); }
   }
   return q;
 }
@@ -797,8 +803,8 @@ static void
 fix_expo(GEN x)
 {
   if (expo(gel(x,5)) >= (1L << EMAX)) {
-    gel(x,4) = addsi(1, gel(x,4));
-    shiftr_inplace(gel(x, 5), - (1L << EMAX));
+    gel(x,4) = addiu(gel(x,4), 1);
+    shiftr_inplace(gel(x,5), - (1L << EMAX));
   }
 }
 
@@ -923,14 +929,34 @@ qfr_isreduced(GEN x, GEN isqrtD)
 
 /* Not stack-clean */
 GEN
-qfr5_red(GEN x, struct qfr_data *S) {
-  while (!qfr_isreduced(x,S->isqrtD)) x = qfr5_rho(x,S);
+qfr5_red(GEN x, struct qfr_data *S)
+{
+  pari_sp av = avma;
+  while (!qfr_isreduced(x, S->isqrtD))
+  {
+    x = qfr5_rho(x, S);
+    if (gc_needed(av,2))
+    {
+      if (DEBUGMEM>1) pari_warn(warnmem,"qfr5_red");
+      x = gerepilecopy(av, x);
+    }
+  }
   return x;
 }
 /* Not stack-clean */
 GEN
-qfr3_red(GEN x, struct qfr_data *S) {
-  while (!qfr_isreduced(x, S->isqrtD)) x = qfr3_rho(x, S);
+qfr3_red(GEN x, struct qfr_data *S)
+{
+  pari_sp av = avma;
+  while (!qfr_isreduced(x, S->isqrtD))
+  {
+    x = qfr3_rho(x, S);
+    if (gc_needed(av,2))
+    {
+      if (DEBUGMEM>1) pari_warn(warnmem,"qfr3_red");
+      x = gerepilecopy(av, x);
+    }
+  }
   return x;
 }
 
@@ -1210,7 +1236,7 @@ primeform(GEN x, GEN p, long prec)
   }
   /* 2 or 3 mod 4 */
   if (s & 2) pari_err_DOMAIN(f, "disc % 4", ">",gen_1, x);
-  absp = absi(p); av = avma;
+  absp = absi_shallow(p); av = avma;
   b = Fp_sqrt(x, absp); if (!b) pari_err_SQRTN(f, mkintmod(x,absp));
   s &= 1; /* s = x mod 2 */
   /* mod(b) != mod2(x) ? [Warning: we may have b == 0] */
@@ -1276,7 +1302,7 @@ qfisolvep(GEN Q, GEN p)
   }
   d = qfb_disc(Q); if (kronecker(d,p) < 0) return gen_0;
   a = redimagsl2(Q, &N);
-  if (is_pm1(gel(a,1))) /* principal form */
+  if (equali1(gel(a,1))) /* principal form */
   {
     long r;
     if (!signe(gel(a,2)))
@@ -1309,8 +1335,8 @@ redrealsl2step(GEN A, GEN d, GEN rd)
   pari_sp ltop = avma;
   GEN N, V = gel(A,1), M = gel(A,2);
   GEN a = gel(V,1), b = gel(V,2), c = gel(V,3);
-  GEN C = mpabs(c);
-  GEN t = addii(b, gmax(rd, C));
+  GEN C = mpabs_shallow(c);
+  GEN t = addii(b, gmax_shallow(rd, C));
   GEN r, q = truedvmdii(t, shifti(C,1), &r);
   b = subii(t, addii(r,b));
   a = c;
@@ -1331,8 +1357,8 @@ redrealsl2(GEN V, GEN d, GEN rd)
   u1 = v2 = gen_1; v1 = u2 = gen_0;
   while (!ab_isreduced(a,b,rd))
   {
-    GEN C = mpabs(c);
-    GEN t = addii(b, gmax(rd,C));
+    GEN C = mpabs_shallow(c);
+    GEN t = addii(b, gmax_shallow(rd,C));
     GEN r, q = truedvmdii(t, shifti(C,1), &r);
     b = subii(t, addii(r,b));
     a = c;
@@ -1420,7 +1446,7 @@ qfbsolve(GEN Q,GEN n)
   case t_QFR: return qfrsolvep(Q,n);
   default:
     pari_err_TYPE("qfbsolve",Q);
-    return NULL; /* NOT REACHED */
+    return NULL; /* LCOV_EXCL_LINE */
   }
 }
 
@@ -1458,34 +1484,13 @@ cornacchia(GEN d, GEN p, GEN *px, GEN *py)
   *px = icopy(b);
   *py = icopy(c); return 1;
 }
-/* 1 if there exists x,y such that x^2 + dy^2 = 4p [p prime], 0 otherwise */
-long
-cornacchia2(GEN d, GEN p, GEN *px, GEN *py)
-{
-  pari_sp av = avma, av2;
-  GEN a, b, c, L, r, px4;
-  long k;
 
-  if (typ(d) != t_INT) pari_err_TYPE("cornacchia2", d);
-  if (typ(p) != t_INT) pari_err_TYPE("cornacchia2", p);
-  if (signe(d) <= 0) pari_err_DOMAIN("cornacchia2", "d","<=",gen_0,d);
-  *px = *py = gen_0;
-  k = mod4(d);
-  if (k == 1 || k == 2) pari_err_DOMAIN("cornacchia2","-d mod 4", ">",gen_1,d);
-  px4 = shifti(p,2);
-  if (abscmpii(px4, d) < 0) { avma = av; return 0; }
-  if (absequaliu(p, 2))
-  {
-    avma = av;
-    switch (itou_or_0(d)) {
-      case 4: *px = gen_2; break;
-      case 7: *px = gen_1; break;
-      default: return 0;
-    }
-    *py = gen_1; return 1;
-  }
-  b = Fp_sqrt(negi(d), p);
-  if (!b) { avma = av; return 0; }
+static long
+cornacchia2_helper(long av, GEN d, GEN p, GEN b, GEN px4, GEN *px, GEN *py)
+{
+  pari_sp av2 = avma;
+  GEN a, c, r, L;
+  long k = mod4(d);
   if (!signe(b)) { /* d = p,2p,3p,4p */
     avma = av;
     if (absequalii(d, px4)){ *py = gen_1; return 1; }
@@ -1510,3 +1515,46 @@ cornacchia2(GEN d, GEN p, GEN *px, GEN *py)
   *px = icopy(b);
   *py = icopy(c); return 1;
 }
+
+/* 1 if there exists x,y such that x^2 + dy^2 = 4p [p prime], 0 otherwise */
+long
+cornacchia2(GEN d, GEN p, GEN *px, GEN *py)
+{
+  pari_sp av = avma;
+  GEN b, px4;
+  long k;
+
+  if (typ(d) != t_INT) pari_err_TYPE("cornacchia2", d);
+  if (typ(p) != t_INT) pari_err_TYPE("cornacchia2", p);
+  if (signe(d) <= 0) pari_err_DOMAIN("cornacchia2", "d","<=",gen_0,d);
+  *px = *py = gen_0;
+  k = mod4(d);
+  if (k == 1 || k == 2) pari_err_DOMAIN("cornacchia2","-d mod 4", ">",gen_1,d);
+  px4 = shifti(p,2);
+  if (abscmpii(px4, d) < 0) { avma = av; return 0; }
+  if (absequaliu(p, 2))
+  {
+    avma = av;
+    switch (itou_or_0(d)) {
+      case 4: *px = gen_2; break;
+      case 7: *px = gen_1; break;
+      default: return 0;
+    }
+    *py = gen_1; return 1;
+  }
+  b = Fp_sqrt(negi(d), p);
+  if (!b) { avma = av; return 0; }
+  return cornacchia2_helper(av, d, p, b, px4, px, py);
+}
+
+/* 1 if there exists x,y such that x^2 + dy^2 = 4p [p prime], 0 otherwise */
+long
+cornacchia2_sqrt(GEN d, GEN p, GEN b, GEN *px, GEN *py)
+{
+  pari_sp av = avma;
+  GEN px4;
+  *px = *py = gen_0;
+  px4 = shifti(p,2);
+  if (abscmpii(px4, d) < 0) { avma = av; return 0; }
+  return cornacchia2_helper(av, d, p, b, px4, px, py);
+}
