@@ -40,29 +40,42 @@ RgM_check_ZM(GEN A, const char *s)
   }
 }
 
-long
-ZV_max_lg(GEN x)
+static long
+ZV_max_lg_i(GEN x, long m)
 {
-  long i, prec = 2, m = lg(x);
+  long i, prec = 2;
   for (i=1; i<m; i++) { long l = lgefint(gel(x,i)); if (l > prec) prec = l; }
   return prec;
 }
+
 long
-ZM_max_lg(GEN x)
+ZV_max_lg(GEN x)
+{ return ZV_max_lg_i(x, lg(x)); }
+
+static long
+ZM_max_lg_i(GEN x, long n, long m)
 {
-  long i, prec = 2, n = lg(x);
+  long prec = 2;
   if (n != 1)
   {
-    long j, m = lgcols(x);
+    long j;
     for (j=1; j<n; j++)
     {
-      GEN c = gel(x,j);
-      for (i=1; i<m; i++) { long l = lgefint(gel(c,i)); if (l > prec) prec = l; }
+      long l = ZV_max_lg_i(gel(x,j), m);
+      if (l > prec) prec = l;
     }
   }
   return prec;
 }
 
+long
+ZM_max_lg(GEN x)
+{
+  long n = lg(x);
+  if (n==1) return 2;
+  return ZM_max_lg_i(x, n, lgcols(x));
+}
+
 GEN
 ZM_supnorm(GEN x)
 {
@@ -388,11 +401,11 @@ ZM_mul_classical(GEN x, GEN y, long l, long lx, long ly)
 }
 
 /* Strassen-Winograd used for dim >= ZM_sw_bound */
-static const long ZM_sw_bound = 36;
-
 static GEN
 ZM_mul_i(GEN x, GEN y, long l, long lx, long ly)
 {
+  long s = maxss(ZM_max_lg_i(x,lx,l), ZM_max_lg_i(y,ly,lx));
+  long ZM_sw_bound = s > 60 ? 2: s > 25 ? 4: s>15 ? 8 : s > 8 ? 16 : 32;
   if (l <= ZM_sw_bound || lx <= ZM_sw_bound || ly <= ZM_sw_bound)
     return ZM_mul_classical(x, y, l, lx, ly);
   else
@@ -407,6 +420,35 @@ ZM_mul(GEN x, GEN y)
   if (lx==1) return zeromat(0, ly-1);
   return ZM_mul_i(x, y, lgcols(x), lx, ly);
 }
+
+GEN
+QM_mul(GEN x, GEN y)
+{
+  GEN dx, nx = Q_primitive_part(x, &dx);
+  GEN dy, ny = Q_primitive_part(y, &dy);
+  GEN z = ZM_mul(nx, ny);
+  if (dx || dy)
+  {
+    GEN d = dx ? dy ? gmul(dx, dy): dx : dy;
+    if (!gequal1(d)) z = ZM_Q_mul(z, d);
+  }
+  return z;
+}
+
+GEN
+QM_QC_mul(GEN x, GEN y)
+{
+  GEN dx, nx = Q_primitive_part(x, &dx);
+  GEN dy, ny = Q_primitive_part(y, &dy);
+  GEN z = ZM_ZC_mul(nx, ny);
+  if (dx || dy)
+  {
+    GEN d = dx ? dy ? gmul(dx, dy): dx : dy;
+    if (!gequal1(d)) z = ZC_Q_mul(z, d);
+  }
+  return z;
+}
+
 /* assume result is symmetric */
 GEN
 ZM_multosym(GEN x, GEN y)
@@ -518,6 +560,8 @@ ZM_transmul(GEN x, GEN y)
 static GEN
 ZM_sqr_i(GEN x, long l)
 {
+  long s = ZM_max_lg_i(x,l,l);
+  long ZM_sw_bound = s > 60 ? 2: s > 25 ? 4: s>15 ? 8 : s > 8 ? 16 : 32;
   if (l <= ZM_sw_bound)
     return ZM_mul_classical(x, x, l, l, l);
   else
@@ -540,19 +584,46 @@ ZM_ZC_mul(GEN x, GEN y)
 
 GEN
 ZC_Z_div(GEN x, GEN c)
-{
-  long i, l;
-  GEN a = cgetg_copy(x, &l);
-  for (i = 1; i < l; i++) gel(a,i) = Qdivii(gel(x,i), c);
-  return a;
-}
+{ pari_APPLY_type(t_COL, Qdivii(gel(x,i), c)) }
+
 GEN
-ZM_Z_div(GEN X, GEN c)
+ZM_Z_div(GEN x, GEN c)
+{ pari_APPLY_same(ZC_Z_div(gel(x, i), c)) }
+
+GEN
+ZC_Q_mul(GEN A, GEN z)
 {
-  long j, l = lg(X);
-  GEN A = cgetg(l, t_MAT);
-  for (j = 1; j < l; j++) gel(A,j) = ZC_Z_div(gel(X,j), c);
-  return A;
+  pari_sp av = avma;
+  long i, l = lg(A);
+  GEN d, n, Ad, B, u;
+  if (typ(z)==t_INT) return ZC_Z_mul(A,z);
+  n = gel(z, 1); d = gel(z, 2);
+  Ad = FpC_red(A, d);
+  u = gcdii(d, FpV_factorback(Ad, NULL, d));
+  B = cgetg(l, t_COL);
+  if (equali1(u))
+  {
+    for(i=1; i<l; i++)
+      gel(B, i) = mkfrac(mulii(n, gel(A,i)), d);
+  } else
+  {
+    for(i=1; i<l; i++)
+    {
+      GEN di = gcdii(gel(Ad, i), u), ni = mulii(n, diviiexact(gel(A,i), di));
+      if (equalii(d, di))
+        gel(B, i) = ni;
+      else
+        gel(B, i) = mkfrac(ni, diviiexact(d, di));
+    }
+  }
+  return gerepilecopy(av, B);
+}
+
+GEN
+ZM_Q_mul(GEN x, GEN z)
+{
+  if (typ(z)==t_INT) return ZM_Z_mul(x,z);
+  pari_APPLY_same(ZC_Q_mul(gel(x, i), z));
 }
 
 long
@@ -720,55 +791,34 @@ ZM_sub(GEN x, GEN y)
 /********************************************************************/
 /* return X/c assuming division is exact */
 GEN
-ZC_Z_divexact(GEN X, GEN c)
-{
-  long i, l = lg(X);
-  GEN A = cgetg(l, t_COL);
-  for (i=1; i<l; i++) gel(A,i) = diviiexact(gel(X,i), c);
-  return A;
-}
+ZC_Z_divexact(GEN x, GEN c)
+{ pari_APPLY_type(t_COL, diviiexact(gel(x,i), c)) }
+
 GEN
-ZM_Z_divexact(GEN X, GEN c)
-{
-  long i, l = lg(X);
-  GEN A = cgetg(l, t_MAT);
-  for (i = 1; i < l; i++) gel(A,i) = ZC_Z_divexact(gel(X,i), c);
-  return A;
-}
-/* Return c * X */
+ZM_Z_divexact(GEN x, GEN c)
+{ pari_APPLY_same(ZC_Z_divexact(gel(x,i), c)) }
+
 GEN
-ZC_Z_mul(GEN X, GEN c)
+ZC_Z_mul(GEN x, GEN c)
 {
-  long i, l;
-  GEN A;
-  if (!signe(c)) return zerocol(lg(X)-1);
-  if (is_pm1(c)) return (signe(c) > 0)? ZC_copy(X): ZC_neg(X);
-  l = lg(X); A = cgetg(l, t_COL);
-  for (i=1; i<l; i++) gel(A,i) = mulii(c,gel(X,i));
-  return A;
-}
-GEN
-ZC_z_mul(GEN X, long c)
-{
-  long i, l;
-  GEN A;
-  if (!c) return zerocol(lg(X)-1);
-  if (c == 1) return ZC_copy(X);
-  if (c ==-1) return ZC_neg(X);
-  l = lg(X); A = cgetg(l, t_COL);
-  for (i=1; i<l; i++) gel(A,i) = mulsi(c,gel(X,i));
-  return A;
+  if (!signe(c)) return zerocol(lg(x)-1);
+  if (is_pm1(c)) return (signe(c) > 0)? ZC_copy(x): ZC_neg(x);
+  pari_APPLY_type(t_COL, mulii(gel(x,i), c))
 }
 
 GEN
-zv_z_mul(GEN M, long n)
+ZC_z_mul(GEN x, long c)
 {
-  long l;
-  GEN N = cgetg_copy(M, &l);
-  while (--l > 0) N[l] = M[l]*n;
-  return N;
+  if (!c) return zerocol(lg(x)-1);
+  if (c == 1) return ZC_copy(x);
+  if (c ==-1) return ZC_neg(x);
+  pari_APPLY_type(t_COL, mulsi(c, gel(x,i)))
 }
 
+GEN
+zv_z_mul(GEN x, long n)
+{ pari_APPLY_long(x[i]*n) }
+
 /* return a ZM */
 GEN
 nm_Z_mul(GEN X, GEN c)
@@ -809,14 +859,17 @@ ZM_Z_mul(GEN X, GEN c)
   }
   return A;
 }
-
+void
+ZC_lincomb1_inplace_i(GEN X, GEN Y, GEN v, long n)
+{
+  long i;
+  for (i = n; i; i--) gel(X,i) = addmulii_inplace(gel(X,i), gel(Y,i), v);
+}
 /* X <- X + v Y (elementary col operation) */
 void
 ZC_lincomb1_inplace(GEN X, GEN Y, GEN v)
 {
-  long i, m = lgefint(v);
-  if (m == 2) return; /* v = 0 */
-  for (i = lg(X)-1; i; i--) gel(X,i) = addmulii_inplace(gel(X,i), gel(Y,i), v);
+  if (lgefint(v) != 2) return ZC_lincomb1_inplace_i(X, Y, v, lg(X)-1);
 }
 void
 Flc_lincomb1_inplace(GEN X, GEN Y, ulong v, ulong q)
@@ -828,22 +881,14 @@ Flc_lincomb1_inplace(GEN X, GEN Y, ulong v, ulong q)
 
 /* X + v Y, wasteful if (v = 0) */
 static GEN
-ZC_lincomb1(GEN v, GEN X, GEN Y)
-{
-  long i, lx = lg(X);
-  GEN A = cgetg(lx,t_COL);
-  for (i=1; i<lx; i++) gel(A,i) = addmulii(gel(X,i), gel(Y,i), v);
-  return A;
-}
+ZC_lincomb1(GEN v, GEN x, GEN y)
+{ pari_APPLY_type(t_COL, addmulii(gel(x,i), gel(y,i), v)) }
+
 /* -X + vY */
 static GEN
-ZC_lincomb_1(GEN v, GEN X, GEN Y)
-{
-  long i, lx = lg(X);
-  GEN A = cgetg(lx,t_COL);
-  for (i=1; i<lx; i++) gel(A,i) = mulsubii(gel(Y,i), v, gel(X,i));
-  return A;
-}
+ZC_lincomb_1(GEN v, GEN x, GEN y)
+{ pari_APPLY_type(t_COL, mulsubii(gel(y,i), v, gel(x,i))) }
+
 /* X,Y compatible ZV; u,v in Z. Returns A = u*X + v*Y */
 GEN
 ZC_lincomb(GEN u, GEN v, GEN X, GEN Y)
@@ -887,68 +932,33 @@ ZC_lincomb(GEN u, GEN v, GEN X, GEN Y)
 /**                                                                **/
 /********************************************************************/
 GEN
-ZV_to_nv(GEN z)
-{
-  long i, l = lg(z);
-  GEN x = cgetg(l, t_VECSMALL);
-  for (i=1; i<l; i++) x[i] = itou(gel(z,i));
-  return x;
-}
+ZV_to_nv(GEN x)
+{ pari_APPLY_ulong(itou(gel(x,i))) }
 
 GEN
-zm_to_ZM(GEN z)
-{
-  long i, l = lg(z);
-  GEN x = cgetg(l,t_MAT);
-  for (i=1; i<l; i++) gel(x,i) = zc_to_ZC(gel(z,i));
-  return x;
-}
+zm_to_ZM(GEN x)
+{ pari_APPLY_type(t_MAT, zc_to_ZC(gel(x,i))) }
 
 GEN
-zmV_to_ZMV(GEN z)
-{
-  long i, l = lg(z);
-  GEN x = cgetg(l,t_VEC);
-  for (i=1; i<l; i++) gel(x,i) = zm_to_ZM(gel(z,i));
-  return x;
-}
+zmV_to_ZMV(GEN x)
+{ pari_APPLY_type(t_VEC, zm_to_ZM(gel(x,i))) }
 
 /* same as Flm_to_ZM but do not assume positivity */
 GEN
-ZM_to_zm(GEN z)
-{
-  long i, l = lg(z);
-  GEN x = cgetg(l,t_MAT);
-  for (i=1; i<l; i++) gel(x,i) = ZV_to_zv(gel(z,i));
-  return x;
-}
+ZM_to_zm(GEN x)
+{ pari_APPLY_same(ZV_to_zv(gel(x,i))) }
 
 GEN
-zv_to_Flv(GEN z, ulong p)
-{
-  long i, l = lg(z);
-  GEN x = cgetg(l,t_VECSMALL);
-  for (i=1; i<l; i++) x[i] = umodsu(z[i], p);
-  return x;
-}
+zv_to_Flv(GEN x, ulong p)
+{ pari_APPLY_ulong(umodsu(x[i], p)) }
 
 GEN
-zm_to_Flm(GEN z, ulong p)
-{
-  long i, l = lg(z);
-  GEN x = cgetg(l,t_MAT);
-  for (i=1; i<l; i++) gel(x,i) = zv_to_Flv(gel(z,i),p);
-  return x;
-}
+zm_to_Flm(GEN x, ulong p)
+{ pari_APPLY_same(zv_to_Flv(gel(x,i),p)) }
 
 GEN
-ZMV_to_zmV(GEN z)
-{
-  long i,l = lg(z);
-  GEN x = cgetg(l, t_VEC);
-  for (i=1; i<l; i++) gel(x,i) = ZM_to_zm(gel(z,i));
-  return x;
-}
+ZMV_to_zmV(GEN x)
+{ pari_APPLY_type(t_VEC, ZM_to_zm(gel(x,i))) }
 
 /********************************************************************/
 /**                                                                **/
@@ -970,12 +980,7 @@ ZC_copy(GEN x)
 
 GEN
 ZM_copy(GEN x)
-{
-  long l;
-  GEN y = cgetg_copy(x, &l);
-  while (--l > 0) gel(y,l) = ZC_copy(gel(x,l));
-  return y;
-}
+{ pari_APPLY_same(ZC_copy(gel(x,i))) }
 
 void
 ZV_neg_inplace(GEN M)
@@ -984,21 +989,11 @@ ZV_neg_inplace(GEN M)
   while (--l > 0) gel(M,l) = negi(gel(M,l));
 }
 GEN
-ZC_neg(GEN M)
-{
-  long l = lg(M);
-  GEN N = cgetg(l, t_COL);
-  while (--l > 0) gel(N,l) = negi(gel(M,l));
-  return N;
-}
+ZC_neg(GEN x)
+{ pari_APPLY_type(t_COL, negi(gel(x,i))) }
 GEN
-zv_neg(GEN M)
-{
-  long l;
-  GEN N = cgetg_copy(M, &l);
-  while (--l > 0) N[l] = -M[l];
-  return N;
-}
+zv_neg(GEN x)
+{ pari_APPLY_long(-x[i]) }
 GEN
 zv_neg_inplace(GEN M)
 {
@@ -1008,12 +1003,7 @@ zv_neg_inplace(GEN M)
 }
 GEN
 ZM_neg(GEN x)
-{
-  long l;
-  GEN y = cgetg_copy(x, &l);
-  while (--l > 0) gel(y,l) = ZC_neg(gel(x,l));
-  return y;
-}
+{ pari_APPLY_same(ZC_neg(gel(x,i))) }
 
 void
 ZV_togglesign(GEN M)
@@ -1126,6 +1116,17 @@ ZM_equal(GEN A, GEN B)
   return 1;
 }
 int
+ZM_equal0(GEN A)
+{
+  long i, j, m, l = lg(A);
+  if (l == 1) return 1;
+  m = lgcols(A);
+  for (j = 1; j < l; j++)
+    for (i = 1; i < m; i++)
+      if (signe(gcoeff(A,i,j))) return 0;
+  return 1;
+}
+int
 zv_equal(GEN V, GEN W)
 {
   long l = lg(V);
@@ -1183,13 +1184,31 @@ ZM_isidentity(GEN x)
   return 1;
 }
 int
+ZM_isdiagonal(GEN x)
+{
+  long i,j, lx = lg(x);
+  if (lx == 1) return 1;
+  if (lx != lgcols(x)) return 0;
+
+  for (j=1; j<lx; j++)
+  {
+    GEN c = gel(x,j);
+    for (i=1; i<j; i++)
+      if (signe(gel(c,i))) return 0;
+    for (i++; i<lx; i++)
+      if (signe(gel(c,i))) return 0;
+  }
+  return 1;
+}
+int
 ZM_isscalar(GEN x, GEN s)
 {
   long i, j, lx = lg(x);
 
   if (lx == 1) return 1;
-  if (lx != lgcols(x)) return 0;
   if (!s) s = gcoeff(x,1,1);
+  if (equali1(s)) return ZM_isidentity(x);
+  if (lx != lgcols(x)) return 0;
   for (j=1; j<lx; j++)
   {
     GEN c = gel(x,j);
@@ -1203,7 +1222,6 @@ ZM_isscalar(GEN x, GEN s)
   return 1;
 }
 
-
 long
 ZC_is_ei(GEN x)
 {
@@ -1249,7 +1267,7 @@ zv_content(GEN x)
   long i, s, l = lg(x);
   if (l == 1) return 0;
   s = labs(x[1]);
-  for (i=2; i<l && s!=1; i++) s = cgcd(x[i],s);
+  for (i = 2; i < l && s != 1; i++) s = ugcd(s, labs(x[i]));
   return s;
 }
 GEN
@@ -1301,16 +1319,35 @@ GEN
 zv_prod_Z(GEN v)
 {
   pari_sp av = avma;
-  long k, n = lg(v)-1, m;
-  GEN x;
-  if (n == 0) return gen_1;
-  if (n == 1) return utoi(v[1]);
-  if (n == 2) return muluu(v[1], v[2]);
+  long k, m, n = lg(v)-1;
+  GEN V;
+  switch(n) {
+    case 0: return gen_1;
+    case 1: return utoi(v[1]);
+    case 2: return muluu(v[1], v[2]);
+  }
   m = n >> 1;
-  x = cgetg(m + (odd(n)? 2: 1), t_VEC);
-  for (k = 1; k <= m; k++) gel(x,k) = muluu(v[k<<1], v[(k<<1)-1]);
-  if (odd(n)) gel(x,k) = utoipos(v[n]);
-  return gerepileuptoint(av, gen_product(x, NULL, _mulii));
+  V = cgetg(m + (odd(n)? 2: 1), t_VEC);
+  for (k = 1; k <= m; k++) gel(V,k) = muluu(v[k<<1], v[(k<<1)-1]);
+  if (odd(n)) gel(V,k) = utoipos(v[n]);
+  return gerepileuptoint(av, gen_product(V, NULL, &_mulii));
+}
+GEN
+vecsmall_prod(GEN v)
+{
+  pari_sp av = avma;
+  long k, m, n = lg(v)-1;
+  GEN V;
+  switch (n) {
+    case 0: return gen_1;
+    case 1: return stoi(v[1]);
+    case 2: return mulss(v[1], v[2]);
+  }
+  m = n >> 1;
+  V = cgetg(m + (odd(n)? 2: 1), t_VEC);
+  for (k = 1; k <= m; k++) gel(V,k) = mulss(v[k<<1], v[(k<<1)-1]);
+  if (odd(n)) gel(V,k) = stoi(v[n]);
+  return gerepileuptoint(av, gen_product(V, NULL, &_mulii));
 }
 
 GEN
