@@ -625,6 +625,7 @@ RgM_minpoly(GEN M, long v)
 {
   pari_sp av = avma;
   GEN V, W;
+  if (lg(M) == 1) return pol_1(v);
   M = RgM_Frobenius(M, 1, NULL, &V);
   W = minpoly_listpolslice(M, V, v);
   if (varncmp(v,gvar2(W)) >= 0)
@@ -690,18 +691,6 @@ matfrobenius(GEN M, long flag, long v)
 /*                                                                 */
 /*******************************************************************/
 
-static GEN
-easymin(GEN x, long v)
-{
-  GEN G, R, dR;
-  if (typ(x)==t_POLMOD && !issquarefree(gel(x,1))) return NULL;
-  R = easychar(x, v);
-  if (!R) return NULL;
-  dR = RgX_deriv(R);
-  if (!lgpol(dR)) return NULL;
-  G = RgX_normalize(RgX_gcd(R,dR));
-  return RgX_div(R,G);
-}
 static GEN
 RgXQ_minpoly_naive(GEN y, GEN P)
 {
@@ -709,17 +698,41 @@ RgXQ_minpoly_naive(GEN y, GEN P)
   GEN M = ker(RgXQ_matrix_pow(y,n,n,P));
   return content(RgM_to_RgXV(M,varn(P)));
 }
+static GEN
+easymin(GEN x, long v)
+{
+  GEN G, R, dR;
+  long tx = typ(x);
+  if (tx == t_FFELT)
+  {
+    R = FpX_to_mod(FF_minpoly(x), FF_p_i(x));
+    setvarn(R,v); return R;
+  }
+  if (tx == t_POLMOD)
+  {
+    GEN a = gel(x,2), b = gel(x,1);
+    if (typ(a) != t_POL || varn(a) != varn(b))
+    {
+      if (varncmp(gvar(a), v) <= 0) pari_err_PRIORITY("minpoly", x, "<", v);
+      return deg1pol(gen_1, gneg_i(a), v);
+    }
+    if (!issquarefree(b))
+    {
+      R = RgXQ_minpoly_naive(a, b);
+      setvarn(R,v); return R;
+    }
+  }
+  R = easychar(x, v); if (!R) return NULL;
+  dR = RgX_deriv(R);  if (!lgpol(dR)) return NULL;
+  G = RgX_normalize(RgX_gcd(R,dR));
+  return RgX_div(R,G);
+}
 GEN
 minpoly(GEN x, long v)
 {
   pari_sp av = avma;
   GEN P;
   if (v < 0) v = 0;
-  if (typ(x) == t_FFELT)
-  {
-    P = FpX_to_mod(FF_minpoly(x), FF_p_i(x));
-    setvarn(P,v); return gerepileupto(av,P);
-  }
   P = easymin(x,v);
   if (P) return gerepileupto(av,P);
   avma = av;
