@@ -20,9 +20,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include "pari.h"
 #include "paripriv.h"
 
-INLINE void
-clone_unlock_deep(GEN C) { if (isclone(C)) gunclone_deep(C); }
-
 /*******************************************************************/
 /*                                                                 */
 /*                               SUM                               */
@@ -198,6 +195,25 @@ extract_selector_ok(long lx, GEN L)
   return 0;
 }
 
+GEN
+shallowmatextract(GEN x, GEN l1, GEN l2)
+{
+  long i, j, n1 = lg(l1), n2 = lg(l2);
+  GEN M = cgetg(n2, t_MAT);
+  for(i=1; i < n2; i++)
+  {
+    long ii = l2[i];
+    GEN C = cgetg(n1, t_COL);
+    for (j=1; j < n1; j++)
+    {
+      long jj = l1[j];
+      gel(C, j) = gmael(x, ii, jj);
+    }
+    gel(M, i) = C;
+  }
+  return M;
+}
+
 GEN
 shallowextract(GEN x, GEN L)
 {
@@ -301,7 +317,7 @@ shallowextract(GEN x, GEN L)
     return y;
   }
   pari_err_TYPE("vecextract [mask]", L);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 /* does the component selector l select 0 component ? */
@@ -514,7 +530,7 @@ genindexselect(void *E, long (*f)(void* E, GEN x), GEN A)
       break;
     default:
       pari_err_TYPE("select",A);
-      return NULL;/*not reached*/
+      return NULL;/*LCOV_EXCL_LINE*/
   }
   v = cgetg(l, t_VECSMALL);
   av = avma;
@@ -568,7 +584,7 @@ genselect(void *E, long (*f)(void* E, GEN x), GEN A)
       break;
     default:
       pari_err_TYPE("select",A);
-      return NULL;/*not reached*/
+      return NULL;/*LCOV_EXCL_LINE*/
   }
   clone_unlock_deep(A); return y;
 }
@@ -589,36 +605,32 @@ select0(GEN f, GEN x, long flag)
     case 0: return genselect((void *) f, gp_callbool, x);
     case 1: return genindexselect((void *) f, gp_callbool, x);
     default: pari_err_FLAG("select");
-             return NULL;/*not reached*/
+             return NULL;/*LCOV_EXCL_LINE*/
   }
 }
 
+GEN
+parselect_worker(GEN d, GEN C)
+{
+  return gequal0(closure_callgen1(C, d))? gen_0: gen_1;
+}
+
 GEN
 parselect(GEN C, GEN D, long flag)
 {
-  pari_sp av, av2;
-  long lv, l = lg(D), i, pending = 0, workid;
-  GEN V, worker, done;
-  struct pari_mt pt;
+  pari_sp av;
+  long lv, l = lg(D), i;
+  GEN V, W, worker;
   check_callgen1(C, "parselect");
-  if (!is_vec_t(typ(D))) pari_err_TYPE("parapply",D);
-  V = cgetg(l, t_VECSMALL); av = avma;
-  worker = strtoclosure("_parapply_worker", 1, C);
-  av2 = avma;
-  mt_queue_start(&pt, worker);
-  for (i=1; i<l || pending; i++)
-  {
-    mt_queue_submit(&pt, i, i<l? mkvec(gel(D,i)): NULL);
-    done = mt_queue_get(&pt, &workid, &pending);
-    if (done) V[workid] = !gequal0(done);
-    avma = av2;
-  }
-  mt_queue_end(&pt);
-  avma = av;
+  if (!is_vec_t(typ(D))) pari_err_TYPE("parselect",D);
+  W = cgetg(l, t_VECSMALL); av = avma;
+  worker = strtoclosure("_parselect_worker", 1, C);
+  V = gen_parapply(worker, D);
   for (lv=1, i=1; i<l; i++)
-    if (V[i]) V[lv++]=i;
-  fixlg(V, lv);
-  return flag? V: extract_copy(D, V);
+    if (signe(gel(V,i))) W[lv++] = i;
+  fixlg(W, lv);
+  avma = av;
+  return flag? W: extract_copy(D, W);
 }
 
 GEN
@@ -707,7 +719,8 @@ genapply(void *E, GEN (*f)(void* E, GEN x), GEN x)
 
     case t_VEC: case t_COL: y = vecapply1(E,f,x); break;
     default:
-      pari_err_TYPE("apply",x); return NULL;/*not reached*/
+      pari_err_TYPE("apply",x);
+      return NULL;/*LCOV_EXCL_LINE*/
   }
   clone_unlock_deep(x); return y;
 }
@@ -741,31 +754,32 @@ veccatselapply(void *Epred, long (*pred)(void* E, GEN x), void *Efun,
 }
 
 GEN
-parapply_worker(GEN d, GEN C)
+gen_parapply(GEN worker, GEN D)
 {
-  return closure_callgen1(C, d);
+  long l, i, pending = 0, workid;
+  GEN V, W, done;
+  struct pari_mt pt;
+  W = cgetg(2, t_VEC);
+  V = cgetg_copy(D, &l);
+  mt_queue_start_lim(&pt, worker, l-1);
+  for (i=1; i<l || pending; i++)
+  {
+    if (i<l) gel(W,1) = gel(D,i);
+    mt_queue_submit(&pt, i, i<l? W: NULL);
+    done = mt_queue_get(&pt, &workid, &pending);
+    if (done) gel(V,workid) = done;
+  }
+  mt_queue_end(&pt);
+  return V;
 }
 
 GEN
 parapply(GEN C, GEN D)
 {
   pari_sp av = avma;
-  long l = lg(D), i, pending = 0, workid;
-  GEN V, worker, done;
-  struct pari_mt pt;
   check_callgen1(C, "parapply");
   if (!is_vec_t(typ(D))) pari_err_TYPE("parapply",D);
-  worker = strtoclosure("_parapply_worker", 1, C);
-  V = cgetg(l, typ(D));
-  mt_queue_start(&pt, worker);
-  for (i=1; i<l || pending; i++)
-  {
-    mt_queue_submit(&pt, i, i<l? mkvec(gel(D,i)): NULL);
-    done = mt_queue_get(&pt, &workid, &pending);
-    if (done) gel(V,workid) = done;
-  }
-  mt_queue_end(&pt);
-  return gerepilecopy(av, V);
+  return gerepileupto(av, gen_parapply(C, D));
 }
 
 GEN
