@@ -1027,6 +1027,7 @@ algtobasis(GEN nf, GEN x)
       return gerepileupto(av,poltobasis(nf,x));
 
     case t_COL:
+      if (!RgV_is_QV(x)) pari_err_TYPE("nfalgtobasis",x);
       if (lg(x)-1 != nf_get_degree(nf)) pari_err_DIM("nfalgtobasis");
       return gcopy(x);
 
@@ -2512,7 +2513,7 @@ Q_mod_bid(GEN bid, GEN a)
   GEN xZ = gcoeff(bid_get_ideal(bid),1,1);
   GEN b = Rg_to_Fp(a, xZ);
   if (gsigne(a) < 0) b = subii(b, xZ);
-  return b;
+  return signe(b)? b: xZ;
 }
 /* Return decomposition of a on the CRT generators blocks attached to the
  * S->sprk and sarch; sgn = sign(a, S->arch), NULL if unknown */
