@@ -273,6 +273,44 @@ gen_pow_fold(GEN x, GEN n, void *E, GEN (*sqr)(void*,GEN),
   return gerepilecopy(av, gen_pow_fold_i(x,n,E,sqr,msqr));
 }
 
+GEN
+gen_pow_init(GEN x, GEN n, long k, void *E, GEN (*sqr)(void*,GEN), GEN (*mul)(void*,GEN,GEN))
+{
+  long i, j, l = expi(n);
+  long m = 1UL<<(k-1);
+  GEN x2 = sqr(E, x), y = gcopy(x);
+  GEN R = cgetg(m+1, t_VEC);
+  for(i = 1; i <= m; i++)
+  {
+    GEN C = cgetg(l+1, t_VEC);
+    gel(C,1) = y;
+    for(j = 2; j <= l; j++)
+      gel(C,j) = sqr(E, gel(C,j-1));
+    gel(R,i) = C;
+    y = mul(E, y, x2);
+  }
+  return R;
+}
+
+GEN
+gen_pow_table(GEN R, GEN n, void *E, GEN (*one)(void*), GEN (*mul)(void*,GEN,GEN))
+{
+  long e = expu(lg(R)-1) + 1;
+  long l = expi(n);
+  long i, w;
+  GEN z = one(E), tw;
+  for(i=0; i<=l; )
+  {
+    if (int_bit(n, i)==0) { i++; continue; }
+    if (i+e-1>l) e = l+1-i;
+    w = int_block(n,i+e-1,e);
+    tw = gmael(R, 1+(w>>1), i+1);
+    z = mul(E, z, tw);
+    i += e;
+  }
+  return z;
+}
+
 GEN
 gen_powers(GEN x, long l, int use_sqr, void *E, GEN (*sqr)(void*,GEN),
                                       GEN (*mul)(void*,GEN,GEN), GEN (*one)(void*))
@@ -365,7 +403,7 @@ iter_rho(GEN x, GEN g, GEN q, GEN A, ulong h, void *E, const struct bb_group *gr
     case 1:
       return mkvec3(grp->mul(E,a,x),addis(gel(A,2),1),gel(A,3));
     case 2:
-      return mkvec3(grp->mul(E,a,g),gel(A,2),addis(gel(A,3),1));
+      return mkvec3(grp->mul(E,a,g),gel(A,2),addiu(gel(A,3),1));
   }
   return NULL;
 }
@@ -549,7 +587,7 @@ get_arith_ZZM(GEN o)
       break;
   }
   pari_err_TYPE("generic discrete logarithm (order factorization)",o);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 GEN
 get_arith_Z(GEN o)
@@ -571,7 +609,7 @@ get_arith_Z(GEN o)
       break;
   }
   pari_err_TYPE("generic discrete logarithm (order factorization)",o);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 /* grp->easylog() is an optional trapdoor function that catch easy logarithms*/
