@@ -537,7 +537,7 @@ choose_params(GEN P, GEN N, GEN X, GEN B, long *pdelta, long *pt)
 
 static int
 sol_OK(GEN x, GEN N, GEN B)
-{ return B? (cmpii(gcdii(x,N),B) >= 0): !signe(remii(x,N)); }
+{ return B? (cmpii(gcdii(x,N),B) >= 0): dvdii(x,N); }
 /* deg(P) > 0, x >= 0. Find all j such that gcd(P(j), N) >= B, |j| <= x */
 static GEN
 do_exhaustive(GEN P, GEN N, long x, GEN B)
@@ -656,7 +656,7 @@ zncoppersmith(GEN P0, GEN N, GEN X, GEN B)
     if (ZV_isscalar(short_pol)) short_pol = gel(sh, 2);
 
     nsp = gen_0;
-    for (j = 1; j <= dim; j++) nsp = addii(nsp, absi(gel(short_pol,j)));
+    for (j = 1; j <= dim; j++) nsp = addii(nsp, absi_shallow(gel(short_pol,j)));
 
     if (DEBUGLEVEL >= 2)
     {
@@ -703,75 +703,74 @@ zncoppersmith(GEN P0, GEN N, GEN X, GEN B)
 /********************************************************************/
 
 static int
-real_indep(GEN re, GEN im, long bitprec)
+real_indep(GEN re, GEN im, long bit)
 {
-  GEN p1 = gsub(gmul(gel(re,1),gel(im,2)),
-                gmul(gel(re,2),gel(im,1)));
-  return (!gequal0(p1) && gexpo(p1) > - bitprec);
+  GEN d = gsub(gmul(gel(re,1),gel(im,2)), gmul(gel(re,2),gel(im,1)));
+  return (!gequal0(d) && gexpo(d) > - bit);
 }
 
 GEN
-lindep2(GEN x, long bit)
+lindepfull_bit(GEN x, long bit)
 {
-  long tx=typ(x), lx=lg(x), ly, i, j;
-  pari_sp av = avma;
+  long lx = lg(x), ly, i, j;
   GEN re, im, M;
 
-  if (! is_vec_t(tx)) pari_err_TYPE("lindep2",x);
-  if (lx<=2)
+  if (! is_vec_t(typ(x))) pari_err_TYPE("lindep2",x);
+  if (lx <= 2)
   {
-    if (lx == 2 && gequal0(x)) return mkcol(gen_1);
-    return cgetg(1,t_COL);
+    if (lx == 2 && gequal0(x)) return matid(1);
+    return NULL;
   }
-  if (bit < 0) pari_err_DOMAIN("lindep2", "accuracy", "<", gen_0, stoi(bit));
-  if (!bit)
-  {
-    bit = gprecision(x);
-    if (!bit)
-    {
-      x = primpart(x);
-      bit = 32 + gexpo(x);
-    }
-    else
-      bit = (long)prec2nbits_mul(bit, 0.8);
-  }
-  else
-    bit = (long) (bit/LOG10_2);
   re = real_i(x);
   im = imag_i(x);
   /* independent over R ? */
-  if (lx == 3 && real_indep(re,im,bit)) { avma = av; return cgetg(1, t_COL); }
+  if (lx == 3 && real_indep(re,im,bit)) return NULL;
   if (gequal0(im)) im = NULL;
   ly = im? lx+2: lx+1;
   M = cgetg(lx,t_MAT);
   for (i=1; i<lx; i++)
   {
     GEN c = cgetg(ly,t_COL); gel(M,i) = c;
-    for (j=1; j<lx; j++) gel(c,j) = (i==j)? gen_1: gen_0;
+    for (j=1; j<lx; j++) gel(c,j) = gen_0;
+    gel(c,i) = gen_1;
     gel(c,lx)           = gtrunc2n(gel(re,i), bit);
     if (im) gel(c,lx+1) = gtrunc2n(gel(im,i), bit);
   }
-  M = ZM_lll(M, 0.99, LLL_INPLACE);
-  M = gel(M,1);
-  M[0] = evaltyp(t_COL) | evallg(lx);
-  return gerepilecopy(av, M);
+  return ZM_lll(M, 0.99, LLL_INPLACE);
 }
-
-void
-init_dalloc(void)
-{ /* correct alignment for dalloc */
-  (void)new_chunk((avma % sizeof(double)) / sizeof(long));
-}
-
-double *
-dalloc(size_t n)
+GEN
+lindep_bit(GEN x, long bit)
 {
-  return (double*)new_chunk(n / sizeof(long));
+  pari_sp av = avma;
+  GEN v, M = lindepfull_bit(x,bit);
+  if (!M) { avma = av; return cgetg(1, t_COL); }
+  v = gel(M,1); setlg(v, lg(M));
+  return gerepilecopy(av, v);
+}
+/* deprecated */
+GEN
+lindep2(GEN x, long dig)
+{
+  long bit;
+  if (dig < 0) pari_err_DOMAIN("lindep2", "accuracy", "<", gen_0, stoi(dig));
+  if (dig) bit = (long) (dig/LOG10_2);
+  else
+  {
+    bit = gprecision(x);
+    if (!bit)
+    {
+      x = Q_primpart(x); /* left on stack */
+      bit = 32 + gexpo(x);
+    }
+    else
+      bit = (long)prec2nbits_mul(bit, 0.8);
+  }
+  return lindep_bit(x, bit);
 }
 
 /* x is a vector of elts of a p-adic field */
 GEN
-padic_lindep(GEN x)
+lindep_padic(GEN x)
 {
   long i, j, prec = LONG_MAX, nx = lg(x)-1, v;
   pari_sp av = avma;
@@ -785,9 +784,9 @@ padic_lindep(GEN x)
 
     j = precp(c); if (j < prec) prec = j;
     q = gel(c,2);
-    if (!p) p = q; else if (!equalii(p, q)) pari_err_MODULUS("padic_lindep", p, q);
+    if (!p) p = q; else if (!equalii(p, q)) pari_err_MODULUS("lindep_padic", p, q);
   }
-  if (!p) pari_err_TYPE("padic_lindep [not a p-adic vector]",x);
+  if (!p) pari_err_TYPE("lindep_padic [not a p-adic vector]",x);
   v = gvaluation(x,p); pn = powiu(p,prec);
   if (v) x = gmul(x, powis(p, -v));
   x = RgV_to_FpV(x, pn);
@@ -806,7 +805,7 @@ padic_lindep(GEN x)
 }
 /* x is a vector of t_POL/t_SER */
 GEN
-Xadic_lindep(GEN x)
+lindep_Xadic(GEN x)
 {
   long i, prec = LONG_MAX, deg = 0, lx = lg(x), vx, v;
   pari_sp av = avma;
@@ -826,7 +825,7 @@ Xadic_lindep(GEN x)
     switch(typ(c))
     {
       case t_POL: deg = maxss(deg, degpol(c)); break;
-      case t_RFRAC: pari_err_TYPE("Xadic_lindep", c);
+      case t_RFRAC: pari_err_TYPE("lindep_Xadic", c);
       case t_SER:
         prec = minss(prec, valp(c)+lg(c)-2);
         gel(x,i) = ser2rfrac_i(c);
@@ -865,10 +864,10 @@ lindep0(GEN x,long bit)
   for (i = 1; i < lg(x); i++)
     switch(typ(gel(x,i)))
     {
-      case t_PADIC: return padic_lindep(x);
+      case t_PADIC: return lindep_padic(x);
       case t_POL:
       case t_RFRAC:
-      case t_SER: return Xadic_lindep(x);
+      case t_SER: return lindep_Xadic(x);
       case t_VEC:
       case t_COL: return vec_lindep(x);
     }
@@ -896,7 +895,7 @@ algdep0(GEN x, long n, long bit)
   gel(y,2) = x; /* n >= 1 */
   for (i=3; i<=n+1; i++) gel(y,i) = gmul(gel(y,i-1),x);
   if (typ(x) == t_PADIC)
-    y = padic_lindep(y);
+    y = lindep_padic(y);
   else
     y = lindep2(y, bit);
   if (lg(y) == 1) pari_err(e_DOMAIN,"algdep", "degree(x)",">", stoi(n), x);
@@ -943,7 +942,7 @@ seralgdep(GEN s, long p, long r)
       }
       gel(v, r*n + m + 1) = c;
     }
-  D = Xadic_lindep(v);
+  D = lindep_Xadic(v);
   if (lg(D) == 1) { avma = av; return gen_0; }
   v = cgetg(p+1, t_VEC);
   for (n = 0; n < p; n++)
@@ -951,6 +950,96 @@ seralgdep(GEN s, long p, long r)
   return gerepilecopy(av, RgV_to_RgX(v, 0));
 }
 
+/* FIXME: could precompute ZM_lll attached to V[2..] */
+static GEN
+lindepcx(GEN V, long bit)
+{
+  GEN Vr = real_i(V), Vi = imag_i(V);
+  if (gexpo(Vr) < -bit) V = Vi;
+  else if (gexpo(Vi) < -bit) V = Vr;
+  return lindepfull_bit(V, bit);
+}
+/* c floating point t_REAL or t_COMPLEX, T ZX, recognize in Q[x]/(T).
+ * V helper vector (containing complex roots of T), MODIFIED */
+static GEN
+cx_bestapprnf(GEN c, GEN T, GEN V, long bit)
+{
+  GEN M, a, v = NULL;
+  long i, l;
+  gel(V,1) = gneg(c); M = lindepcx(V, bit);
+  if (!M) pari_err(e_MISC, "cannot rationalize coeff in bestapprnf");
+  l = lg(M); a = NULL;
+  for (i = 1; i < l; i ++) { v = gel(M,i); a = gel(v,1); if (signe(a)) break; }
+  v = RgC_Rg_div(vecslice(v, 2, lg(M)-1), a);
+  if (!T) return gel(v,1);
+  v = RgV_to_RgX(v, varn(T)); l = lg(v);
+  if (l == 2) return gen_0;
+  if (l == 3) return gel(v,2);
+  return mkpolmod(v, T);
+}
+static GEN
+bestapprnf_i(GEN x, GEN T, GEN V, long bit)
+{
+  long i, l, tx = typ(x);
+  GEN z;
+  switch (tx)
+  {
+    case t_INT: case t_FRAC: return x;
+    case t_REAL: case t_COMPLEX: return cx_bestapprnf(x, T, V, bit);
+    case t_POLMOD: if (RgX_equal(gel(x,1),T)) return x;
+                   break;
+    case t_POL: case t_SER: case t_VEC: case t_COL: case t_MAT:
+      l = lg(x); z = cgetg(l, tx);
+      for (i = 1; i < lontyp[tx]; i++) z[i] = x[i];
+      for (; i < l; i++) gel(z,i) = bestapprnf_i(gel(x,i), T, V, bit);
+      return z;
+  }
+  pari_err_TYPE("mfcxtoQ", x);
+  return NULL;/*LCOV_EXCL_LINE*/
+}
+
+GEN
+bestapprnf(GEN x, GEN T, GEN roT, long prec)
+{
+  pari_sp av = avma;
+  long tx = typ(x), dT = 1, bit;
+  GEN V;
+
+  if (T)
+  {
+    if (typ(T) != t_POL) T = nf_get_pol(checknf(T));
+    else if (!RgX_is_ZX(T)) pari_err_TYPE("bestapprnf", T);
+    dT = degpol(T);
+  }
+  if (is_rational_t(tx)) return gcopy(x);
+  if (tx == t_POLMOD)
+  {
+    if (!T || !RgX_equal(T, gel(x,1))) pari_err_TYPE("bestapprnf",x);
+    return gcopy(x);
+  }
+
+  if (roT)
+  {
+    long l = gprecision(roT);
+    switch(typ(roT))
+    {
+      case t_INT: case t_FRAC: case t_REAL: case t_COMPLEX: break;
+      default: pari_err_TYPE("bestapprnf", roT);
+    }
+    if (prec < l) prec = l;
+  }
+  else if (!T)
+    roT = gen_1;
+  else
+  {
+    long n = poliscyclo(T); /* cyclotomic is an important special case */
+    roT = n? rootsof1u_cx(n,prec): gel(QX_complex_roots(T,prec), 1);
+  }
+  V = vec_prepend(gpowers(roT, dT-1), NULL);
+  bit = prec2nbits_mul(prec, 0.8);
+  return gerepilecopy(av, bestapprnf_i(x, T, V, bit));
+}
+
 /********************************************************************/
 /**                                                                **/
 /**                              MINIM                             **/
@@ -964,11 +1053,10 @@ minim_alloc(long n, double ***q, GEN *x, double **y,  double **z, double **v)
   *x = cgetg(n, t_VECSMALL);
   *q = (double**) new_chunk(n);
   s = n * sizeof(double);
-  init_dalloc();
-  *y = dalloc(s);
-  *z = dalloc(s);
-  *v = dalloc(s);
-  for (i=1; i<n; i++) (*q)[i] = dalloc(s);
+  *y = (double*) stack_malloc_align(s, sizeof(double));
+  *z = (double*) stack_malloc_align(s, sizeof(double));
+  *v = (double*) stack_malloc_align(s, sizeof(double));
+  for (i=1; i<n; i++) (*q)[i] = (double*) stack_malloc_align(s, sizeof(double));
 }
 
 static GEN
@@ -1352,7 +1440,7 @@ qfminim0(GEN a, GEN borne, GEN stockmax, long flag, long prec)
     }
     default: pari_err_FLAG("qfminim");
   }
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 GEN
@@ -1840,7 +1928,7 @@ fincke_pohst(GEN a, GEN B0, long stockmax, long PREC, FP_chk_fun *CHECK)
   if (lg(v) != lg(rinvtrans)) return NULL;
 
   rinvtrans = RgM_mul(rinvtrans, v);
-  v = ZM_inv(shallowtrans(v),gen_1);
+  v = ZM_inv(shallowtrans(v),NULL);
   r = RgM_mul(r,v);
   u = u? ZM_mul(u,v): v;
 
