@@ -205,7 +205,7 @@ binaire(GEN x)
       return F2v_to_ZV_inplace( binary_zv(x) );
     case t_REAL:
       ex = expo(x);
-      if (!signe(x)) return const_vec(maxss(-ex,0), gen_0);
+      if (!signe(x)) return zerovec(maxss(-ex,0));
 
       lx=lg(x); y=cgetg(3,t_VEC);
       if (ex > bit_prec(x)) pari_err_PREC("binary");
@@ -245,7 +245,7 @@ binaire(GEN x)
       for (i=1; i<lx; i++) gel(y,i) = binaire(gel(x,i));
       break;
     default: pari_err_TYPE("binary",x);
-      return NULL; /* not reached */
+      return NULL; /* LCOV_EXCL_LINE */
   }
   return y;
 }
@@ -676,8 +676,8 @@ gbitnegimply(GEN x, GEN y)
   return gerepileuptoint(ltop,z);
 }
 
-INLINE long
-hamming_word(ulong w)
+long
+hammingl(ulong w)
 {
 #if 0
   return __builtin_popcountl(w);
@@ -726,8 +726,8 @@ hamming_int(GEN n)
 {
   long lx = lgefint(n), i, sum;
   if (lx == 2) return 0;
-  sum = hamming_word(n[2]);
-  for (i = 3; i < lx; i++) sum += hamming_word(n[i]);
+  sum = hammingl(n[2]);
+  for (i = 3; i < lx; i++) sum += hammingl(n[i]);
   return sum;
 }
 
@@ -744,5 +744,5 @@ hammingweight(GEN n)
     case t_MAT: return hamming_mat(n);
   }
   pari_err_TYPE("hammingweight", n);
-  return 0;/*notreached*/
+  return 0;/*LCOV_EXCL_LINE*/
 }
