@@ -85,7 +85,7 @@ vconcat(GEN A, GEN B)
 
   if (!A) return B;
   if (!B) return A;
-  la = lg(A); if (la==1) return A;
+  la = lg(A); if (la==1) return B;
   T = typ(gel(A,1)); /* t_COL or t_VECSMALL */
   ha = lgcols(A); M = cgetg(la,t_MAT);
   hb = lgcols(B); hc = ha+hb-1;
@@ -210,7 +210,7 @@ shallowconcat(GEN x, GEN y)
       break;
   }
   err_cat(x,y);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 /* see catmany() */
@@ -254,7 +254,7 @@ catmanySTR(GEN y1, GEN y2)
   {
     char *c = GSTR( gel(y,0) );
     long nc = strlen(c);
-    if (nc) { (void)strncpy(s, c, nc); s += nc; }
+    if (nc) { (void)memcpy(s, c, nc); s += nc; }
   }
   *s = 0; return z;
 }
@@ -303,7 +303,7 @@ shallowconcat1(GEN x)
       break;
     default:
       pari_err_TYPE("concat",x);
-      return NULL; /* not reached */
+      return NULL; /* LCOV_EXCL_LINE */
   }
   if (lx==2) return gel(x,1);
   z = gel(x,1); t = typ(z); i = 2;
@@ -577,5 +577,5 @@ gconcat(GEN x, GEN y)
       break;
   }
   err_cat(x,y);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
