@@ -244,7 +244,8 @@ Flxq_elldivpol34(long n, GEN a4, GEN a6, GEN S, GEN T, ulong p)
     }
     break;
     default:
-      pari_err_BUG("Flxq_elldivpol34"); return NULL;
+      pari_err_BUG("Flxq_elldivpol34");
+      return NULL;/*LCOV_EXCL_LINE*/
   }
   setvarn(res, get_FlxqX_var(S));
   return FlxqX_rem(res, S, T, p);
@@ -272,7 +273,8 @@ Fq_elldivpol34(long n, GEN a4, GEN a6, GEN S, GEN T, GEN p)
     }
     break;
     default:
-      pari_err_BUG("Fq_elldivpol34"); return NULL;
+      pari_err_BUG("Fq_elldivpol34");
+      return NULL;/*LCOV_EXCL_LINE*/
   }
   if (S)
   {
@@ -290,6 +292,14 @@ rhs(GEN a4, GEN a6, long v)
   return RHS;
 }
 
+static GEN
+Flxq_rhs(GEN a4, GEN a6, long v, long vs)
+{
+  GEN RHS = mkpoln(4, pol1_Flx(vs),  pol0_Flx(vs), a4, a6);
+  setvarn(RHS, v);
+  return RHS;
+}
+
 struct divpolmod_red
 {
   const struct bb_algebra *ff;
@@ -332,9 +342,10 @@ Flxq_elldivpolmod_init(struct divpolmod_red *d, GEN a4, GEN a6, long n, GEN h, G
   void *E;
   const struct bb_algebra *ff;
   GEN RHS, D3 = NULL, D4 = NULL;
+  long v = get_FlxqX_var(h), vT = get_Flx_var(T);
   D3 = n>=0 ? Flxq_elldivpol34(3, a4, a6, h, T, p): NULL;
   D4 = n>=1 ? Flxq_elldivpol34(4, a4, a6, h, T, p): NULL;
-  RHS = FlxX_Fl_mul(FlxqX_rem(rhs(a4, a6, get_FlxqX_var(h)), h, T, p), 4, p);
+  RHS = FlxX_Fl_mul(FlxqX_rem(Flxq_rhs(a4, a6, v, vT), h, T, p), 4, p);
   ff = get_FlxqXQ_algebra(&E, h, T, p);
   divpolmod_init(d, D3, D4, RHS, n, E, ff);
 }
@@ -547,8 +558,8 @@ Fq_to_Flx(GEN a4, GEN T, ulong p)
 static GEN
 Flxq_find_eigen_Frobenius(GEN a4, GEN a6, GEN h, GEN T, ulong p)
 {
-  long v = get_FlxqX_var(h);
-  GEN RHS = FlxqX_rem(rhs(a4, a6, v), h, T, p);
+  long v = get_FlxqX_var(h), vT = get_Flx_var(T);
+  GEN RHS = FlxqX_rem(Flxq_rhs(a4, a6, v, vT), h, T, p);
   return FlxqXQ_halfFrobenius(RHS, h, T, p);
 }
 
@@ -807,7 +818,7 @@ find_isogenous_from_Atkin(GEN a4, GEN a6, ulong ell, struct meqn *MEQN, GEN g, G
     }
   }
   pari_err_BUG("find_isogenous_from_Atkin, kernel not found");
-  return NULL;
+  return NULL;/*LCOV_EXCL_LINE*/
 }
 
 /* Finds E' ell-isogenous to E and the trace term p1 from canonical modular
@@ -1065,11 +1076,12 @@ find_isogenous(GEN a4,GEN a6, ulong ell, struct meqn *MEQN, GEN g, GEN T,GEN p)
     GEN meqnj = meqn_j(MEQN, Zq_ellj(a4, a6, T, pe, p, e), ell, T, pe);
     g = ZqX_liftroot(meqnj, g, T, p, e);
   }
-  return (MEQN->type == 'C')
-    ? find_isogenous_from_canonical(a4, a6, ell, MEQN, g, T, p, e)
-    : (MEQN->type == 'A')
-    ? find_isogenous_from_Atkin(a4, a6, ell, MEQN, g, T, p, e)
-    : find_isogenous_from_J(a4, a6, ell, MEQN, g, T, p, e);
+  switch(MEQN->type)
+  {
+    case 'C': return find_isogenous_from_canonical(a4,a6,ell, MEQN, g, T,p,e);
+    case 'A': return find_isogenous_from_Atkin(a4,a6,ell, MEQN, g, T,p,e);
+    default:  return find_isogenous_from_J(a4,a6,ell, MEQN, g, T,p,e);
+  }
 }
 
 static GEN
@@ -1123,14 +1135,13 @@ find_kernel_power(GEN Eba4, GEN Eba6, GEN Eca4, GEN Eca6, ulong ell, struct meqn
     h = FqX_homogenous_eval(gtmp, num_iso, kpoly2, T, p);
     if (signe(Fq_elldivpolmod(Eba4, Eba6, ell, h, T, p)))
     {
-      GEN Ic = FqX_homogenous_div(num_iso, kpoly2, numer(Ib), denom(Ib), T, p);
-      GEN kpoly_new = FqX_homogenous_eval(gtmp, numer(Ic), denom(Ic), T, p);
+      GEN Ic = FqX_homogenous_div(num_iso,kpoly2, numer_i(Ib),denom_i(Ib), T,p);
+      GEN kpoly_new = FqX_homogenous_eval(gtmp,   numer_i(Ic),denom_i(Ic), T,p);
       return gerepilecopy(ltop, mkvecn(5, a4t, a6t, kpoly_new, gtmp, Ic));
     }
     avma = btop;
   }
-  pari_err_BUG("failed to find kernel polynomial");
-  return NULL; /*NOT REACHED*/
+  avma = ltop; return NULL;
 }
 
 /****************************************************************************/
@@ -1139,48 +1150,37 @@ find_kernel_power(GEN Eba4, GEN Eba6, GEN Eca4, GEN Eca6, ulong ell, struct meqn
 enum mod_type {MTpathological, MTAtkin, MTElkies, MTone_root, MTroots};
 
 static GEN
-Flxq_study_eqn(long ell, GEN mpoly, GEN T, ulong p, long *pt_dG, long *pt_r)
+Flxq_study_eqn(GEN mpoly, GEN T, ulong p, long *pt_dG, long *pt_r)
 {
   GEN Xq = FlxqX_Frobenius(mpoly, T, p);
   GEN G  = FlxqX_gcd(FlxX_sub(Xq, pol_x(0), p), mpoly, T, p);
   *pt_dG = degpol(G);
-  if (!*pt_dG)
-  {
-    GEN L = FlxqXQ_matrix_pow(Xq, ell+1, ell+1, mpoly, T, p);
-    long vT = get_Flx_var(T);
-    long s = ell + 1 - FlxqM_rank(FlxM_Flx_add_shallow(L, Fl_to_Flx(p-1, vT), p), T, p);
-    *pt_r = (ell + 1)/s;
-    return NULL;
-  }
+  if (!*pt_dG) { *pt_r = FlxqX_ddf_degree(mpoly, Xq, T, p); return NULL; }
   return gel(FlxqX_roots(G, T, p), 1);
 }
 
 static GEN
-Fp_study_eqn(long ell, GEN mpoly, GEN p, long *pt_dG, long *pt_r)
+Fp_study_eqn(GEN mpoly, GEN p, long *pt_dG, long *pt_r)
 {
   GEN T  = FpX_get_red(mpoly, p);
   GEN XP = FpX_Frobenius(T, p);
   GEN G  = FpX_gcd(FpX_sub(XP, pol_x(0), p), mpoly, p);
   *pt_dG = degpol(G);
-  if (!*pt_dG)
-  {
-    long s = FpX_nbfact_Frobenius(T, XP, p);
-    *pt_r = (ell + 1)/s;
-    return NULL;
-  }
+  if (!*pt_dG) { *pt_r = FpX_ddf_degree(T, XP, p); return NULL; }
   return FpX_oneroot(G, p);
 }
 
 static GEN
-FpXQ_study_eqn(long ell, GEN mpoly, GEN T, GEN p, long *pt_dG, long *pt_r)
+Fq_study_eqn(GEN mpoly, GEN T, GEN p, long *pt_dG, long *pt_r)
 {
   GEN G;
+  if (!T) return Fp_study_eqn(mpoly, p, pt_dG, pt_r);
   if (lgefint(p)==3)
   {
     ulong pp = p[2];
     GEN Tp = ZXT_to_FlxT(T,pp);
     GEN mpolyp = ZXX_to_FlxX(mpoly,pp,get_FpX_var(T));
-    G = Flxq_study_eqn(ell, mpolyp, Tp, pp, pt_dG, pt_r);
+    G = Flxq_study_eqn(mpolyp, Tp, pp, pt_dG, pt_r);
     return G ? Flx_to_ZX(G): NULL;
   }
   else
@@ -1188,13 +1188,7 @@ FpXQ_study_eqn(long ell, GEN mpoly, GEN T, GEN p, long *pt_dG, long *pt_r)
     GEN Xq = FpXQX_Frobenius(mpoly, T, p);
     G  = FpXQX_gcd(FpXX_sub(Xq, pol_x(0), p), mpoly, T, p);
     *pt_dG = degpol(G);
-    if (!*pt_dG)
-    {
-      GEN L = FpXQXQ_matrix_pow(Xq, ell+1, ell+1, mpoly, T, p);
-      long s = ell + 1 - FqM_rank(RgM_Rg_add(L, gen_m1), T, p);
-      *pt_r = (ell + 1)/s;
-      return NULL;
-    }
+    if (!*pt_dG) { *pt_r = FpXQX_ddf_degree(mpoly, Xq, T, p); return NULL; }
     return gel(FpXQX_roots(G, T, p), 1);
   }
 }
@@ -1206,13 +1200,11 @@ study_modular_eqn(long ell, GEN mpoly, GEN T, GEN p, enum mod_type *mt, long *pt
   pari_sp ltop = avma;
   GEN g = gen_0;
   *ptr_r = 0; /*gcc -Wall*/
-  if (degpol(FqX_gcd(mpoly, FqX_deriv(mpoly, T, p), T, p)) > 0)
-    *mt = MTpathological;
+  if (!FqX_is_squarefree(mpoly, T, p)) *mt = MTpathological;
   else
   {
     long dG;
-    g = T ? FpXQ_study_eqn(ell, mpoly, T, p, &dG, ptr_r)
-            : Fp_study_eqn(ell, mpoly, p, &dG, ptr_r);
+    g = Fq_study_eqn(mpoly, T, p, &dG, ptr_r);
     switch(dG)
     {
       case 0:  *mt = MTAtkin; break;
@@ -1234,10 +1226,11 @@ study_modular_eqn(long ell, GEN mpoly, GEN T, GEN p, enum mod_type *mt, long *pt
 
 /*Returns the trace modulo ell^k when ell is an Elkies prime */
 static GEN
-find_trace_Elkies_power(GEN a4, GEN a6, ulong ell, long k, struct meqn *MEQN, GEN g, GEN tr, GEN q, GEN T, GEN p, ulong smallfact, pari_timer *ti)
+find_trace_Elkies_power(GEN a4, GEN a6, ulong ell, long *pt_k, struct meqn *MEQN, GEN g, GEN tr, GEN q, GEN T, GEN p, long smallfact, pari_timer *ti)
 {
   pari_sp ltop = avma, btop;
   GEN tmp, Eba4, Eba6, Eca4, Eca6, Ib, kpoly;
+  long k = *pt_k;
   ulong lambda, ellk = upowuu(ell, k), pellk = umodiu(q, ellk);
   long cnt;
 
@@ -1253,19 +1246,19 @@ find_trace_Elkies_power(GEN a4, GEN a6, ulong ell, long k, struct meqn *MEQN, GE
   lambda = tr ? find_eigen_value_oneroot(a4, a6, ell, tr, kpoly, T, p):
                 find_eigen_value_power(a4, a6, ell, 1, 1, kpoly, T, p);
   if (DEBUGLEVEL>1) err_printf(" [%ld ms]", timer_delay(ti));
-  if (smallfact && smallfact%ell!=0)
+  if (smallfact && smallfact%(long)ell!=0)
   {
     ulong pell = pellk%ell;
     ulong ap = Fl_add(lambda, Fl_div(pell, lambda, ell), ell);
     if (Fl_sub(pell, ap, ell)==ell-1) { avma = ltop; return mkvecsmall(ap); }
+    if (smallfact < 0 && Fl_add(pell, ap, ell)==ell-1) { avma = ltop; return mkvecsmall(ap); }
   }
   btop = avma;
   for (cnt = 2; cnt <= k; cnt++)
   {
-    GEN tmp;
+    GEN tmp = find_kernel_power(Eba4, Eba6, Eca4, Eca6, ell, MEQN, kpoly, Ib, T, p);
+    if (!tmp) { k = cnt-1; break; }
     if (DEBUGLEVEL) err_printf(", %Ps", powuu(ell, cnt));
-    tmp = find_kernel_power(Eba4, Eba6, Eca4, Eca6, ell, MEQN, kpoly, Ib, T, p);
-    if (!tmp) { avma = ltop; return NULL; }
     lambda = find_eigen_value_power(a4, a6, ell, cnt, lambda, gel(tmp,3), T, p);
     Eba4 = Eca4;
     Eba6 = Eca6;
@@ -1281,6 +1274,9 @@ find_trace_Elkies_power(GEN a4, GEN a6, ulong ell, long k, struct meqn *MEQN, GE
     if (DEBUGLEVEL>1) err_printf(" [%ld ms]", timer_delay(ti));
   }
   avma = ltop;
+  ellk = upowuu(ell, k);
+  pellk = umodiu(q, ellk);
+  *pt_k = k;
   return mkvecsmall(Fl_add(lambda, Fl_div(pellk, lambda, ellk), ellk));
 }
 
@@ -1338,7 +1334,7 @@ find_trace_lp1_roots(long ell, GEN q)
 /*trace modulo ell^k: [], [t] or [t1,...,td] */
 static GEN
 find_trace(GEN a4, GEN a6, GEN j, ulong ell, GEN q, GEN T, GEN p, long *ptr_kt,
-  ulong smallfact, long vx, long vy)
+  long smallfact, long vx, long vy)
 {
   pari_sp ltop = avma;
   GEN g, meqnj, tr, tr2;
@@ -1347,7 +1343,7 @@ find_trace(GEN a4, GEN a6, GEN j, ulong ell, GEN q, GEN T, GEN p, long *ptr_kt,
   struct meqn MEQN;
   pari_timer ti;
 
-  kt = maxss((long)(log(expi(q)*LOG2)/log((double)ell)), 1);
+  kt = maxss((long)(log(expi(q)*M_LN2)/log((double)ell)), 1);
   if (DEBUGLEVEL)
   { err_printf("SEA: Prime %5ld ", ell); timer_start(&ti); }
   get_modular_eqn(&MEQN, ell, vx, vy);
@@ -1360,17 +1356,14 @@ find_trace(GEN a4, GEN a6, GEN j, ulong ell, GEN q, GEN T, GEN p, long *ptr_kt,
   {
   case MTone_root:
     tr2 = find_trace_one_root(ell, q);
-    kt = 1;
-    /* Must take k = 1 because we can't apply Hensel: no guarantee that a
-     * root mod ell^2 exists */
-    tr = find_trace_Elkies_power(a4,a6,ell, kt, &MEQN, g, tr2, q, T, p, smallfact, &ti);
-    if (!tr) tr = tr2;
+    tr = find_trace_Elkies_power(a4,a6,ell, &kt, &MEQN, g, tr2, q, T, p, smallfact, &ti);
+    if (!tr) { tr = tr2; kt = 1; }
     break;
   case MTElkies:
     /* Contrary to MTone_root, may look mod higher powers of ell */
     if (abscmpiu(p, 2*ell+3) <= 0)
       kt = 1; /* Not implemented in this case */
-    tr = find_trace_Elkies_power(a4,a6,ell, kt, &MEQN, g, NULL, q, T, p, smallfact, &ti);
+    tr = find_trace_Elkies_power(a4,a6,ell, &kt, &MEQN, g, NULL, q, T, p, smallfact, &ti);
     if (!tr)
     {
       if (DEBUGLEVEL) err_printf("[fail]\n");
@@ -1657,7 +1650,7 @@ static GEN
 match_and_sort(GEN compile_atkin, GEN Mu, GEN u, GEN q, void *E, const struct bb_group *grp)
 {
   pari_sp av1, av2;
-  GEN baby, giant, SgMb, Mb, Mg, den, Sg, dec_inf, div, pp1 = addis(q,1);
+  GEN baby, giant, SgMb, Mb, Mg, den, Sg, dec_inf, div, pp1 = addiu(q,1);
   GEN P, Pb, Pg, point, diff, pre, table, table_ind;
   long best_i, i, lbaby, lgiant, k = lg(compile_atkin)-1;
   GEN bound = sqrti(shifti(q, 2)), card;
@@ -1675,9 +1668,8 @@ match_and_sort(GEN compile_atkin, GEN Mu, GEN u, GEN q, void *E, const struct bb
     for (j = 1, i = 1; i < l; i++)
     {
       GEN t = Z_chinese_post(u, stoi(r2[i]), C, U, NULL);
-      t = Fp_center(t, C, Cs2);
-      if (abscmpii(t, bound) <= 0)
-        gel(card, j++) = subii(pp1, t);
+      t = Fp_center_i(t, C, Cs2);
+      if (abscmpii(t, bound) <= 0) gel(card, j++) = subii(pp1, t);
     }
     setlg(card, j);
     return gen_select_order(card, E, grp);
@@ -1687,15 +1679,15 @@ match_and_sort(GEN compile_atkin, GEN Mu, GEN u, GEN q, void *E, const struct bb
   best_i = separation( get_lgatkin(compile_atkin, k) );
   avma = av1;
 
-  baby  = possible_traces(compile_atkin, stoi(best_i), &Mb, 1);
-  giant = possible_traces(compile_atkin, subis(int2n(k), best_i+1), &Mg, 0);
+  baby  = possible_traces(compile_atkin, utoi(best_i), &Mb, 1);
+  giant = possible_traces(compile_atkin, subiu(int2n(k), best_i+1), &Mg, 0);
   lbaby = lg(baby);
   lgiant = lg(giant);
   den = Fp_inv(Fp_mul(Mu, Mb, Mg), Mg);
   av2 = avma;
   for (i = 1; i < lgiant; i++, avma = av2)
     affii(Fp_mul(gel(giant,i), den, Mg), gel(giant,i));
-  gen_sort_inplace(giant, (void*)&cmpii, &cmp_nodata, NULL);
+  ZV_sort_inplace(giant);
   Sg = Fp_mul(negi(u), den, Mg);
   den = Fp_inv(Fp_mul(Mu, Mg, Mb), Mb);
   dec_inf = divii(mulii(Mb,addii(Mg,shifti(Sg,1))), shifti(Mg,1));
@@ -1708,7 +1700,7 @@ match_and_sort(GEN compile_atkin, GEN Mu, GEN u, GEN q, void *E, const struct bb
     if (cmpii(b, dec_inf) < 0) b = addii(b, Mb);
     affii(b, gel(baby,i));
   }
-  gen_sort_inplace(baby, (void*)&cmpii, &cmp_nodata, NULL);
+  ZV_sort_inplace(baby);
 
   SgMb = mulii(Sg, Mb);
   card = cgetg(lcard+1,t_VEC);
@@ -1771,7 +1763,6 @@ MATCH_RESTART:
         {
           GEN card1 = subii(Be, mulii(Mu, GMb));
           GEN card2 = addii(card1, mulii(mulsi(2,Mu), GMb));
-          if (DEBUGLEVEL>=2) timer_printf(&ti,"match_and_sort");
           if (abscmpii(subii(pp1, card1), bound) <= 0)
             affii(card1, gel(card, ++nbcard));
           if (nbcard >= lcard) goto MATCH_RESTART;
@@ -1791,6 +1782,7 @@ MATCH_RESTART:
     }
   }
   setlg(card, nbcard+1);
+  if (DEBUGLEVEL>=2) timer_printf(&ti,"match_and_sort");
   return gen_select_order(card, E, grp);
 }
 
@@ -1829,7 +1821,7 @@ get_FqE_group(void ** pt_E, GEN a4, GEN a6, GEN T, GEN p)
  * is detected. Useful when searching for a good curve for cryptographic
  * applications */
 GEN
-Fq_ellcard_SEA(GEN a4, GEN a6, GEN q, GEN T, GEN p, ulong smallfact)
+Fq_ellcard_SEA(GEN a4, GEN a6, GEN q, GEN T, GEN p, long smallfact)
 {
   const long MAX_ATKIN = 21;
   pari_sp ltop = avma, btop;
@@ -1889,11 +1881,16 @@ Fq_ellcard_SEA(GEN a4, GEN a6, GEN q, GEN T, GEN p, ulong smallfact)
       long t_mod_ellkt = trace_mod[1];
       if (smallfact && smallfact%ell!=0)
       { /* does ell divide q + 1 - t ? */
-        long card_mod_ell = umodsu(umodiu(q,ell) + 1 - t_mod_ellkt, ell) ;
-        if (!card_mod_ell)
+        long q_mod_ell_plus_one = umodiu(q,ell) + 1;
+        ulong  card_mod_ell = umodsu(q_mod_ell_plus_one - t_mod_ellkt, ell);
+        ulong tcard_mod_ell = 1;
+        if (card_mod_ell && smallfact < 0)
+          tcard_mod_ell = umodsu(q_mod_ell_plus_one + t_mod_ellkt, ell);
+        if (!card_mod_ell || !tcard_mod_ell)
         {
           if (DEBUGLEVEL)
-            err_printf("\nAborting: #E(Fq) divisible by %ld\n",ell);
+            err_printf("\nAborting: #E%s(Fq) divisible by %ld\n",
+                       tcard_mod_ell ? "" : "_twist", ell);
           delete_var();
           delete_var();
           avma = ltop; return gen_0;
@@ -1915,7 +1912,7 @@ Fq_ellcard_SEA(GEN a4, GEN a6, GEN q, GEN T, GEN p, ulong smallfact)
       {
         delete_var();
         delete_var();
-        return gerepileuptoint(ltop, subii(addis(q, 1), TR));
+        return gerepileuptoint(ltop, subii(addiu(q, 1), TR));
       }
       bound_tr = mulrr(bound_bsgs, dbltor(bound_gr));
       bound_gr *= growth_factor;
@@ -1950,11 +1947,11 @@ Fq_ellcard_SEA(GEN a4, GEN a6, GEN q, GEN T, GEN p, ulong smallfact)
     if (gc_needed(btop, 1))
       gerepileall(btop,5, &TR,&TR_mod, &compile_atkin, &max_traces, &prod_atkin);
   }
-  return NULL;/*not reached*/
+  return NULL;/*LCOV_EXCL_LINE*/
 }
 
 GEN
-Fp_ellcard_SEA(GEN a4, GEN a6, GEN p, ulong smallfact)
+Fp_ellcard_SEA(GEN a4, GEN a6, GEN p, long smallfact)
 {
   return Fq_ellcard_SEA(a4, a6, p, NULL, p, smallfact);
 }
