@@ -285,10 +285,11 @@ static GEN
 galoisborne(GEN T, GEN dn, struct galois_borne *gb, long d)
 {
   pari_sp ltop, av2;
-  GEN borne, borneroots, borneabs;
+  GEN borne, borneroots, bornetrace, borneabs;
   long prec;
   GEN L, M, prep, den;
   pari_timer ti;
+  const long step=3;
 
   prec = nbits2prec(bit_accuracy(ZX_max_lg(T)));
   den = initgaloisborne(T,dn,prec, &L,&prep,NULL);
@@ -299,8 +300,11 @@ galoisborne(GEN T, GEN dn, struct galois_borne *gb, long d)
   if (DEBUGLEVEL>=4) timer_printf(&ti,"vandermondeinverse");
   borne = matrixnorm(M, prec);
   borneroots = gsupnorm(L, prec); /*t_REAL*/
-  borneabs = ceil_safe(gmul(borne,gmulsg(d, powru(borneroots, d))));
+  bornetrace = gmulsg((2*step)*degpol(T)/d,
+                      powru(borneroots, minss(degpol(T), step)));
   borneroots = ceil_safe(gmul(borne, borneroots));
+  borneabs = ceil_safe(gmax_shallow(gmul(borne, bornetrace),
+                                    powru(bornetrace, d)));
   av2 = avma;
   /*We use d-1 test, so we must overlift to 2^BITS_IN_LONG*/
   gb->valsol = logint(shifti(borneroots,2+BITS_IN_LONG), gb->l) + 1;
@@ -390,7 +394,7 @@ monoratlift(void *E, GEN S, GEN q)
       err_printf("MonomorphismLift: trying early solution %Ps\n",tlift);
     if (gl->den != gen_1) {
       GEN N = gl->gb->ladicsol, N2 = shifti(N,-1);
-      tlift = FpX_center(FpX_red(Q_muli_to_int(tlift, gl->den), N), N,N2);
+      tlift = FpX_center_i(FpX_red(Q_muli_to_int(tlift, gl->den), N), N,N2);
     }
     if (poltopermtest(tlift, gl, d->frob))
     {
@@ -559,7 +563,7 @@ frobeniusliftall(GEN sg, long el, GEN *psi, struct galois_lift *gl,
       {
         u = v;
         for (j = 1; j < m; j++) u = ZX_add(u, gmael(C,SG[pf[j]],j));
-        u = FpX_center(FpX_red(u, gl->Q), gl->Q, shifti(gl->Q,-1));
+        u = FpX_center_i(FpX_red(u, gl->Q), gl->Q, shifti(gl->Q,-1));
         if (poltopermtest(u, gl, frob))
         {
           if (DEBUGLEVEL >= 4)
@@ -751,22 +755,23 @@ testpermutation(GEN F, GEN B, GEN x, long s, long e, long cut,
       /* intheadlong test: overflow in + is OK, we compute mod 2^BIL */
       for (p1 = i+1, p5 = p1%d - 1 ; p1 >= 1; p1--, p5--) /* p5 = (p1%d) - 1 */
       {
+        GEN G1, G6;
         ulong V = 0;
         if (p5 == - 1) { p5 = d - 1; p6 = p1 + 1 - d; } else p6 = p1 + 1;
+        G1 = gel(G,p1); G6 = gel(G,p6);
         p4 = p5 ? x[p1-1] : 0;
-        V = 0;
         for (p2 = 1+p4, p3 = 1 + x[p1]; p2 <= b; p2++)
         {
-          V += umael(W,mael(G,p6,p3),mael(G,p1,p2));
+          V += umael(W,uel(G6,p3),uel(G1,p2));
           p3 += s; if (p3 > b) p3 -= b;
         }
         p3 = 1 + x[p1] - s; if (p3 <= 0) p3 += b;
         for (p2 = p4; p2 >= 1; p2--)
         {
-          V += umael(W,mael(G,p6,p3),mael(G,p1,p2));
+          V += umael(W,uel(G6,p3),uel(G1,p2));
           p3 -= s; if (p3 <= 0) p3 += b;
         }
-        ar[p1] = ar[p1+1] + V;
+        uel(ar,p1) = uel(ar,p1+1) + V;
       }
       if (!headlongisint(uel(ar,1),n)) continue;
 
@@ -919,8 +924,8 @@ fixedfieldsurmer(GEN mod, GEN l, GEN p, long v, GEN NS, GEN W)
     sym[j]++;
     if (DEBUGLEVEL>=6) err_printf("FixedField: Sym: %Ps\n",sym);
     L = sympol_eval(sym,NS);
-    if (!vec_is1to1(FpC_red(L,l))) continue;
-    P = FpX_center(FpV_roots_to_pol(L,mod,v),mod,mod2);
+    if (!vec_is1to1(FpC_red(L,l))) { avma = av; continue; }
+    P = FpX_center_i(FpV_roots_to_pol(L,mod,v),mod,mod2);
     if (!p || FpX_is_squarefree(P,p)) return mkvec3(mkvec2(sym,W),L,P);
     avma = av;
   }
@@ -1153,8 +1158,7 @@ galoisanalysis(GEN T, struct galois_analysis *ga, long calcul_l)
     pari_warn(warner,"Galois group almost certainly not weakly super solvable");
     return 0;
   }
-  /*linf=(n*(n-1))>>2;*/
-  linf = n;
+  linf = 2*n*usqrt(n);
   if (calcul_l && O[1] <= linf)
   {
     pari_sp av2;
@@ -1425,7 +1429,7 @@ s4test(GEN u, GEN liftpow, struct galois_lift *gl, GEN phi)
       res = ZX_add(res, ZX_Z_mul(gel(liftpow,i), gel(u,i+2)));
   res = FpX_red(res, Q);
   if (gl->den != gen_1) res = FpX_Fp_mul(res, gl->den, Q);
-  res = FpX_center(res, Q, shifti(Q,-1));
+  res = FpX_center_i(res, Q, shifti(Q,-1));
   bl = poltopermtest(res, gl, phi);
   if (DEBUGLEVEL >= 6) timer_printf(&ti, "s4test()");
   avma = av; return bl;
@@ -1652,7 +1656,7 @@ galoisfrobeniustest(GEN aut, struct galois_lift *gl, GEN frob)
   GEN tlift = aut;
   long res;
   if (gl->den != gen_1) tlift = FpX_Fp_mul(tlift, gl->den, gl->Q);
-  tlift = FpX_center(tlift, gl->Q, shifti(gl->Q,-1));
+  tlift = FpX_center_i(tlift, gl->Q, shifti(gl->Q,-1));
   res = poltopermtest(tlift, gl, frob);
   avma = av; return res;
 }
@@ -1893,20 +1897,14 @@ galoisgenfixedfield(GEN Tp, GEN Pmod, GEN V, GEN ip, struct galois_borne *gb)
   return mkvec2(PG,Pg);
 }
 
-/* Let sigma^m=1,  tau*sigma*tau^-1=sigma^s.
- * Compute n so that (sigma*tau)^e = sigma^n*tau^e
- * We have n = sum_{k=0}^{e-1} s^k mod m.
- * so n*(1-s) = 1-s^e mod m
- * Unfortunately (1-s) might not invertible mod m.
- */
-
+/* Let sigma^m=1, tau*sigma*tau^-1=sigma^s. Return n = sum_{0<=k<e,0} s^k mod m
+ * so that (sigma*tau)^e = sigma^n*tau^e. N.B. n*(1-s) = 1-s^e mod m,
+ * unfortunately (1-s) may not invertible mod m */
 static long
 stpow(long s, long e, long m)
 {
-  long i;
-  long n = 1;
-  for (i = 1; i < e; i++)
-    n = (1 + n * s) % m;
+  long i, n = 1;
+  for (i = 1; i < e; i++) n = (1 + n * s) % m;
   return n;
 }
 
@@ -1962,7 +1960,7 @@ galoisgenliftauto(GEN O, GEN gj, long s, long n, struct galois_test *td)
       dg *= p; el /= p;
       sel = Fl_powu(s,el,deg);
       if (DEBUGLEVEL >= 6) err_printf("GaloisConj: B=%Ps\n", Bel);
-      sr  = cgcd(stpow(sel,p,deg),deg);
+      sr  = ugcd(stpow(sel,p,deg),deg);
       if (DEBUGLEVEL >= 6)
         err_printf("GaloisConj: exp %d: s=%ld [%ld] a=%ld w=%ld wg=%ld sr=%ld\n",
             dg, sel, deg, a, w[f], wg[f+1], sr);
@@ -2099,7 +2097,7 @@ galoisconj4_main(GEN T, GEN den, long flag)
   if (n) return flag? galoiscyclo(n, varn(T)): conjcyclo(T, n);
   n = degpol(T);
   if (nf)
-  { if (!den) den = Q_denom(nf_get_zk(nf)); }
+  { if (!den) den = nf_get_zkden(nf); }
   else
   {
     if (n <= 0) pari_err_IRREDPOL("galoisinit",T);
@@ -2120,7 +2118,7 @@ galoisconj4_main(GEN T, GEN den, long flag)
   if (den)
   {
     if (typ(den) != t_INT) pari_err_TYPE("galoisconj4 [2nd arg integer]", den);
-    den = absi(den);
+    den = absi_shallow(den);
   }
   gb.l = utoipos(ga.l);
   if (DEBUGLEVEL >= 1) timer_start(&ti);
@@ -2248,7 +2246,7 @@ galoisconj0(GEN nf, long flag, GEN d, long prec)
     case 4: return galoisconj4(nf, d);
   }
   pari_err_FLAG("nfgaloisconj");
-  return NULL; /*not reached*/
+  return NULL; /*LCOV_EXCL_LINE*/
 }
 
 /******************************************************************************/
@@ -2293,7 +2291,7 @@ galoispermtopol_i(GEN gal, GEN perm, GEN mod, GEN mod2)
     }
   }
   pari_err_TYPE("galoispermtopol", perm);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 GEN
@@ -2420,6 +2418,7 @@ galoisfixedfield(GEN gal, GEN perm, long flag, long y)
     if (y < 0) y = 1;
     if (varncmp(y, vT) <= 0)
       pari_err_PRIORITY("galoisfixedfield", T, "<=", y);
+    setvarn(P, y);
     res = cgetg(4, t_VEC);
     gel(res,3) = fixedfieldfactor(L,O,gal_get_group(gal), PM,Pden,mod,mod2,vT,y);
   }
@@ -2440,6 +2439,31 @@ checkgroup(GEN g, GEN *S)
   *S = gal_get_group(g); return galois_group(g);
 }
 
+GEN
+checkgroupelts(GEN G)
+{
+  long i, n;
+  if (typ(G)!=t_VEC) pari_err_TYPE("checkgroupelts", G);
+  if (is_group(G))
+  { /* subgroup of S_n */
+    if (lg(gel(G,1))==1) return mkvec(mkvecsmall(1));
+    return group_elts(G, group_domain(G));
+  }
+  if (lg(G)==9 && typ(gel(G,1))==t_POL)
+    return gal_get_group(G); /* galoisinit */
+  /* vector of permutations ? */
+  n = lg(G)-1;
+  if (n==0) pari_err_DIM("checkgroupelts");
+  for (i = 1; i <= n; i++)
+  {
+    if (typ(gel(G,i)) != t_VECSMALL)
+      pari_err_TYPE("checkgroupelts (element)", gel(G,i));
+    if (lg(gel(G,i)) != lg(gel(G,1)))
+      pari_err_DIM("checkgroupelts [length of permutations]");
+  }
+  return G;
+}
+
 GEN
 galoisisabelian(GEN gal, long flag)
 {
@@ -2453,7 +2477,7 @@ galoisisabelian(GEN gal, long flag)
     case 2: return gerepileupto(av, group_abelianSNF(G,S));
     default: pari_err_FLAG("galoisisabelian");
   }
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 long
@@ -2466,6 +2490,33 @@ galoisisnormal(GEN gal, GEN sub)
   return res;
 }
 
+static GEN
+conjclasses_count(GEN conj, long nb)
+{
+  long i, l = lg(conj);
+  GEN c = zero_zv(nb);
+  for (i = 1; i < l; i++) c[conj[i]]++;
+  return c;
+}
+GEN
+galoisconjclasses(GEN G)
+{
+  pari_sp av = avma;
+  GEN c, e, cc = group_to_cc(G);
+  GEN elts = gel(cc,1), conj = gel(cc,2), repr = gel(cc,3);
+  long i, l = lg(conj), lc = lg(repr);
+  c = conjclasses_count(conj, lc-1);
+  e = cgetg(lc, t_VEC);
+  for (i = 1; i < lc; i++) gel(e,i) = cgetg(c[i]+1, t_VEC);
+  for (i = 1; i < l; i++)
+  {
+    long ci = conj[i];
+    gmael(e, ci, c[ci]) = gel(elts, i);
+    c[ci]--;
+  }
+  return gerepilecopy(av, e);
+}
+
 GEN
 galoissubgroups(GEN gal)
 {
@@ -2501,3 +2552,254 @@ galoisidentify(GEN gal)
   long idx = group_ident(G,S), card = group_order(G);
   avma = av; return mkvec2s(card, idx);
 }
+
+/* index of conjugacy class containing g */
+static long
+cc_id(GEN cc, GEN g)
+{
+  GEN conj = gel(cc,2);
+  long k = signe(gel(cc,4))? g[1]: vecvecsmall_search(gel(cc,1),g,0);
+  return conj[k];
+}
+
+static GEN
+Qevproj_RgX(GEN c, long d, GEN pro)
+{ return RgV_to_RgX(Qevproj_down(RgX_to_RgC(c,d), pro), varn(c)); }
+/* c in Z[X] / (X^o-1), To = polcyclo(o), T = polcyclo(expo), e = expo/o
+ * return c(X^e) mod T as an element of Z[X] / (To) */
+static GEN
+chival(GEN c, GEN T, GEN To, long e, GEN pro, long phie)
+{
+  c = ZX_rem(c, To);
+  if (e != 1) c = ZX_rem(RgX_inflate(c,e), T);
+  if (pro) c = Qevproj_RgX(c, phie, pro);
+  return c;
+}
+/* chi(g^l) = sum_{k=0}^{o-1} a_k zeta_o^{l*k} for all l;
+* => a_k = 1/o sum_{l=0}^{o-1} chi(g^l) zeta_o^{-k*l}. Assume o > 1 */
+static GEN
+chiFT(GEN cp, GEN jg, GEN vze, long e, long o, ulong p, ulong pov2)
+{
+  const long var = 1;
+  ulong oinv = Fl_inv(o,p);
+  long k, l;
+  GEN c = cgetg(o+2, t_POL);
+  for (k = 0; k < o; k++)
+  {
+    ulong a = 0;
+    for (l=0; l<o; l++)
+    {
+      ulong z = vze[Fl_mul(k,l,o)*e + 1];/* zeta_o^{-k*l} */
+      a = Fl_add(a, Fl_mul(uel(cp,jg[l+1]), z, p), p);
+    }
+    gel(c,k+2) = stoi(Fl_center(Fl_mul(a,oinv,p), p, pov2)); /* a_k */
+  }
+  c[1] = evalvarn(var) | evalsigne(1); return ZX_renormalize(c,o+2);
+}
+static GEN
+cc_chartable(GEN cc)
+{
+  GEN al, elts, rep, ctp, ct, dec, id, vjg, H, vord, operm;
+  long i, j, k, f, l, expo, lcl, n;
+  ulong p, pov2;
+
+  elts = gel(cc,1); n = lg(elts)-1;
+  if (n == 1) return mkvec2(mkmat(mkcol(gen_1)), gen_1);
+  rep = gel(cc,3);
+  lcl = lg(rep);
+  vjg = cgetg(lcl, t_VEC);
+  vord = cgetg(lcl,t_VECSMALL);
+  id = identity_perm(lg(gel(elts,1))-1);
+  expo = 1;
+  for(j=1;j<lcl;j++)
+  {
+    GEN jg, h = id, g = gel(elts,rep[j]);
+    long o;
+    vord[j] = o = perm_order(g);
+    expo = ulcm(expo, o);
+    gel(vjg,j) = jg = cgetg(o+1,t_VECSMALL);
+    for (l=1; l<=o; l++)
+    {
+      jg[l] = cc_id(cc, h); /* index of conjugacy class of g^(l-1) */
+      if (l < o) h = perm_mul(h, g);
+    }
+  }
+  /* would sort conjugacy classes by inc. order */
+  operm = vecsmall_indexsort(vord);
+
+  /* expo > 1, exponent of G */
+  p = unextprime(2*n+1);
+  while (p%expo != 1) p = unextprime(p+1);
+  /* compute character table modulo p: idempotents of Z(KG) */
+  al = conjclasses_algcenter(cc, utoipos(p));
+  dec = algsimpledec_ss(al,1);
+  ctp = cgetg(lcl,t_VEC);
+  for(i=1; i<lcl; i++)
+  {
+    GEN e = ZV_to_Flv(gmael3(dec,i,3,1), p); /*(1/n)[(dim chi)chi(g): g in G]*/
+    ulong d = usqrt(Fl_mul(e[1], n, p)); /* = chi(1) <= sqrt(n) < sqrt(p) */
+    gel(ctp,i) = Flv_Fl_mul(e,Fl_div(n,d,p), p); /*[chi(g): g in G]*/
+  }
+  /* Find minimal f such that table is defined over Q(zeta(f)): the conductor
+   * of the class field Q(\zeta_e)^H defined by subgroup
+   * H = { k in (Z/e)^*: g^k ~ g, for all g } */
+  H = coprimes_zv(expo);
+  for (k = 2; k < expo; k++)
+  {
+    if (!H[k]) continue;
+    for (j = 2; j < lcl; j++) /* skip g ~ 1 */
+      if (umael(vjg,j,(k % vord[j])+1) != umael(vjg,j,2)) { H[k] = 0; break; }
+  }
+  f = znstar_conductor_bits(Flv_to_F2v(H));
+  /* lift character table to Z[zeta_f] */
+  pov2 = p>>1;
+  ct = cgetg(lcl, t_MAT);
+  if (f == 1)
+  { /* rational representation */
+    for (j=1; j<lcl; j++) gel(ct,j) = cgetg(lcl,t_COL);
+    for(j=1; j<lcl; j++)
+    {
+      GEN jg = gel(vjg,j); /* jg[l+1] = class of g^l */
+      long t = lg(jg) > 2? jg[2]: jg[1];
+      for(i=1; i<lcl; i++)
+      {
+        GEN cp = gel(ctp,i); /* cp[i] = chi(g_i) mod \P */
+        gcoeff(ct,j,i) = stoi(Fl_center(cp[t], p, pov2));
+      }
+    }
+  }
+  else
+  {
+    const long var = 1;
+    ulong ze = Fl_powu(pgener_Fl(p),(p-1)/expo, p); /* seen as zeta_e^(-1) */
+    GEN vze = Fl_powers(ze, expo-1, p); /* vze[i] = ze^(i-1) */
+    GEN vzeZX = const_vec(p, gen_0);
+    GEN T = polcyclo(expo, var), vT = const_vec(expo,NULL), pro = NULL;
+    long phie = degpol(T), id1 = gel(vjg,1)[1]; /* index of 1_G, always 1 ? */
+    gel(vT, expo) = T;
+    if (f != expo)
+    {
+      long phif = eulerphiu(f);
+      GEN zf = ZX_rem(pol_xn(expo/f,var), T), zfj = zf;
+      GEN M = cgetg(phif+1, t_MAT);
+      gel(M,1) = col_ei(phie,1);
+      for (j = 2; j <= phif; j++)
+      {
+        gel(M,j) = RgX_to_RgC(zfj, phie);
+        if (j < phif) zfj = ZX_rem(ZX_mul(zfj, zf), T);
+      }
+      pro = Qevproj_init(M);
+    }
+    gel(vzeZX,1) = pol_1(var);
+    for (i = 2; i <= expo; i++)
+    {
+      GEN t = ZX_rem(pol_xn(expo-(i-1), var), T);
+      if (pro) t = Qevproj_RgX(t, phie, pro);
+      gel(vzeZX, vze[i]) = t;
+    }
+    for(i=1; i<lcl; i++)
+    { /* loop over characters */
+      GEN cp = gel(ctp,i), C, cj; /* cp[j] = chi(g_j) mod \P */
+      long dim = cp[id1];
+      gel(ct, i) = C = const_col(lcl-1, NULL);
+      gel(C,operm[1]) = utoi(dim); /* chi(1_G) */
+      for (j=lcl-1; j > 1; j--)
+      { /* loop over conjugacy classes, decreasing order: skip 1_G */
+        long e, jperm = operm[j], o = vord[jperm];
+        GEN To, jg = gel(vjg,jperm); /* jg[l+1] = class of g^l */
+
+        if (gel(C, jperm)) continue; /* done already */
+        if (dim == 1) { gel(C, jperm) = gel(vzeZX, cp[jg[2]]); continue; }
+        e = expo / o;
+        cj = chiFT(cp, jg, vze, e, o, p, pov2);
+        To = gel(vT, o); if (!To) To = gel(vT,o) = polcyclo(o, var);
+        gel(C, jperm) = chival(cj, T, To, e, pro, phie);
+        for (k = 2; k < o; k++)
+        {
+          GEN ck = RgX_inflate(cj, k); /* chi(g^k) */
+          gel(C, jg[k+1]) = chival(ck, T, To, e, pro, phie);
+        }
+      }
+    }
+  }
+  ct = gen_sort(ct,(void*)cmp_universal,cmp_nodata);
+  i = 1; while (!vec_isconst(gel(ct,i))) i++;
+  if (i > 1) swap(gel(ct,1), gel(ct,i));
+  return mkvec2(ct, utoipos(f));
+}
+GEN
+galoischartable(GEN gal)
+{
+  pari_sp av = avma;
+  GEN cc = group_to_cc(gal);
+  return gerepilecopy(av, cc_chartable(cc));
+}
+
+static void
+checkgaloischar(GEN ch, GEN repr)
+{
+  if (gvar(ch) == 0) pari_err_PRIORITY("galoischarpoly",ch,"=",0);
+  if (!is_vec_t(typ(ch))) pari_err_TYPE("galoischarpoly", ch);
+  if (lg(repr) != lg(ch)) pari_err_DIM("galoischarpoly");
+}
+
+static long
+galoischar_dim(GEN ch)
+{
+  pari_sp av = avma;
+  long d = gtos(simplify_shallow(lift_shallow(gel(ch,1))));
+  avma = av; return d;
+}
+
+static GEN
+galoischar_aut_charpoly(GEN cc, GEN ch, GEN p, long d)
+{
+  GEN q = p, V = cgetg(d+3, t_POL);
+  long i;
+  V[1] = evalsigne(1)|evalvarn(0);
+  gel(V,2) = gen_0;
+  for (i = 1; i <= d; i++)
+  {
+    gel(V,i+2) = gdivgs(gel(ch, cc_id(cc,q)), -i);
+    if (i < d) q = perm_mul(q, p);
+  }
+  return liftpol_shallow(RgXn_exp(V,d+1));
+}
+
+static GEN
+galoischar_charpoly(GEN cc, GEN ch, long o)
+{
+  GEN chm, V, elts = gel(cc,1), repr = gel(cc,3);
+  long i, d, l = lg(ch), v = gvar(ch);
+  checkgaloischar(ch, repr);
+  chm = v < 0 ? ch: gmodulo(ch, polcyclo(o, v));
+  V = cgetg(l, t_COL); d = galoischar_dim(ch);
+  for (i = 1; i < l; i++)
+    gel(V,i) = galoischar_aut_charpoly(cc, chm, gel(elts,repr[i]), d);
+  return V;
+}
+
+GEN
+galoischarpoly(GEN gal, GEN ch, long o)
+{
+  pari_sp av = avma;
+  GEN cc = group_to_cc(gal);
+  return gerepilecopy(av, galoischar_charpoly(cc, ch, o));
+}
+
+static GEN
+cc_char_det(GEN cc, GEN ch, long o)
+{
+  long i, l = lg(ch), d = galoischar_dim(ch);
+  GEN V = galoischar_charpoly(cc, ch, o);
+  for (i = 1; i < l; i++) gel(V,i) = leading_coeff(gel(V,i));
+  return odd(d)? gneg(V): V;
+}
+
+GEN
+galoischardet(GEN gal, GEN ch, long o)
+{
+  pari_sp av = avma;
+  GEN cc = group_to_cc(gal);
+  return gerepilecopy(av, cc_char_det(cc, ch, o));
+}
