@@ -49,15 +49,13 @@ mathnf0(GEN x, long flag)
 
   switch(flag)
   {
-    case 0: return RgM_is_ZM(x)? ZM_hnf(x): RgM_hnfall(x,NULL,1);
-    case 1: return RgM_is_ZM(x)? hnfall(x): hnfallgen(x);
-    case 2: return RgM_hnfall(x, NULL, 1);
-    case 3: return hnfallgen(x);
+    case 0: case 2: return RgM_is_ZM(x)? ZM_hnf(x): RgM_hnfall(x,NULL,1);
+    case 1: case 3: return RgM_is_ZM(x)? hnfall(x): hnfallgen(x);
     case 4: RgM_check_ZM(x, "mathnf0"); return hnflll(x);
     case 5: RgM_check_ZM(x, "mathnf0"); return hnfperm(x);
     default: pari_err_FLAG("mathnf");
   }
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 /*******************************************************************/
@@ -734,7 +732,7 @@ gbezout_step(GEN *pa, GEN *pb, GEN *pu, GEN *pv, long vx)
   { /* possible accuracy problem */
     GEN D = RgX_gcd_simple(a,b);
     if (degpol(D)) {
-      D = RgX_Rg_div(D, leading_coeff(D));
+      D = RgX_normalize(D);
       a = RgX_div(a, D);
       b = RgX_div(b, D);
       d = RgX_extgcd(a,b, pu,pv); /* retry now */
@@ -877,18 +875,14 @@ GEN
 hnfmerge_get_1(GEN A, GEN B)
 {
   pari_sp av = avma;
-  long j, k, c, l = lg(A), lb;
-  GEN b, t, U = cgetg(l + 1, t_MAT), C = cgetg(l + 1, t_VEC);
+  long j, k, l = lg(A), lb;
+  GEN b, U = cgetg(l + 1, t_MAT), C = cgetg(l + 1, t_VEC);
 
-  t = NULL; /* -Wall */
   b = gcoeff(B,1,1); lb = lgefint(b);
-  if (!signe(b)) {
-    if (!is_pm1(gcoeff(A,1,1))) return NULL;
-    return gen_1;
-  }
   for (j = 1; j < l; j++)
   {
-    c = j+1;
+    GEN t;
+    long c = j+1;
     gel(U,j) = col_ei(l-1, j);
     gel(U,c) = zerocol(l-1); /* dummy */
     gel(C,j) = vecslice(gel(A,j), 1,j);
@@ -920,7 +914,9 @@ hnfmerge_get_1(GEN A, GEN B)
     if (equali1(t)) break;
   }
   if (j >= l) return NULL;
-  return gerepileupto(av, ZM_ZC_mul(A,gel(U,1)));
+  b = lcmii(gcoeff(A,1,1),b);
+  A = FpC_red(ZM_ZC_mul(A,gel(U,1)), b);
+  return gerepileupto(av, FpC_center(A, b, shifti(b,-1)));
 }
 
 /* remove the first r columns */
@@ -1141,41 +1137,80 @@ zlm_echelon(GEN x, long early_abort, ulong p, ulong pm)
   return gerepilecopy(av0, x);
 }
 
-/* dm = multiple of diag element (usually detint(x))
- * flag & hnf_MODID: reduce mod dm * matid [ otherwise as above ].
- * flag & hnf_PART: don't reduce once diagonal is known; */
+static int
+ZV_allequal(GEN v)
+{
+  long i, l = lg(v);
+  if (l > 1)
+  {
+    GEN x = gel(v,1);
+    for (i = 2; i < l; i++) if (!equalii(x,gel(v,i))) return 0;
+  }
+  return 1;
+}
+/* compute optimal D for hnfmod: x upper triangular */
+static GEN
+optimal_D(GEN x, GEN D)
+{
+  long i, n = nbrows(x);
+  GEN C = shallowcopy(D);
+  gel(C,1) = gcoeff(x,1,1);
+  for (i = 2; i < n; i++)
+  {
+    GEN c = mulii(gel(C,i-1), gcoeff(x,i,i));
+    if (signe(c) < 0) togglesign(c);
+    if (cmpii(c, gel(D,i)) >= 0) break;
+    gel(C,i) = c;
+  }
+  return C;
+}
 
-/* x a ZM, dm a t_INT */
+/* D = multiple of det x (usually detint(x)) or vector of positive moduli
+ * (compute hnf(x | D))
+ * flag & hnf_MODID: reduce mod D * matid [ otherwise as above ].
+ * flag & hnf_PART: don't reduce once diagonal is known
+ * flag & hnf_CENTER: centermod HNF (2|x[i,j]| <] x[i,i]) */
 GEN
-ZM_hnfmodall_i(GEN x, GEN dm, long flag)
+ZM_hnfmodall_i(GEN x, GEN D, long flag)
 {
   pari_sp av;
   const long center = (flag & hnf_CENTER);
-  long moddiag = (flag & hnf_MODID);
-  long li, co, i, j, k, def, ldef;
-  GEN a, b, p1, p2, u, dm2, LDM;
+  long moddiag, modsame, nli, li, co, i, j, k, def, ldef;
+  GEN u, LDM;
 
-  co = lg(x); if (co == 1) return cgetg(1,t_MAT);
-  li = lgcols(x); if (li == 1) return cgetg(1,t_MAT);
-  if (typ(dm) == t_INT)
+  co = lg(x);
+  if (co == 1)
   {
-    long ldm = lgefint(dm);
-    dm2 = shifti(dm, -1);
-    dm = const_vec(li-1,dm);
-    dm2= const_vec(li-1,dm2);
-    LDM= const_vecsmall(li-1,ldm);
+    if (typ(D) == t_INT || lg(D) == 1) return cgetg(1,t_MAT);
+    x = diagonal_shallow(D); /* handle flags properly */
+    co = lg(x);
+  }
+  li = lgcols(x);
+  if (li == 1)
+  {
+    if (typ(D) != t_INT && lg(D) != li) pari_err_DIM("ZM_hnfmod");
+    return cgetg(1,t_MAT);
+  }
+  nli = li - 1;
+  modsame = typ(D)==t_INT;
+  if (!modsame)
+  {
+    if (lg(D) != li) pari_err_DIM("ZM_hnfmod");
+    if (ZV_allequal(D)) { modsame = 1; D = gel(D,1); }
+  }
+  moddiag = (flag & hnf_MODID) || !modsame;
+  /* modsame: triangularize mod fixed d*Id;
+   * moddiag: modulo diagonal matrix, else modulo multiple of determinant */
+
+  if (modsame)
+  {
+    LDM = const_vecsmall(nli, 2*lgefint(D)-2);
+    D = const_vec(nli,D);
   }
   else
   {
-    if (lg(dm) != li) pari_err_DIM("ZM_hnfmod");
-    moddiag = 1;
-    dm2 = cgetg(li, t_VEC);
     LDM = cgetg(li, t_VECSMALL);
-    for (i=1; i<li; i++)
-    {
-      gel(dm2,i) = shifti(gel(dm,i),-1);
-      LDM[i] = lgefint(gel(dm,i));
-    }
+    for (i=1; i<li; i++) LDM[i] = lgefint(gel(D,i));
   }
   av = avma;
   x = RgM_shallowcopy(x);
@@ -1187,31 +1222,50 @@ ZM_hnfmodall_i(GEN x, GEN dm, long flag)
     if (!moddiag)
       pari_err_DOMAIN("ZM_hnfmod","nb lines",">", strtoGENstr("nb columns"), x);
   }
-  for (def = co-1,i = li-1; i > ldef; i--,def--)
+  for (def = co-1,i = nli; i > ldef; i--,def--)
   {
-    GEN d = gel(dm,i), d2 = gel(dm2,i);
-    gcoeff(x,i,def) = centermodii(gcoeff(x,i,def), d,d2);
-    for (j = def-1; j; j--)
+    GEN d = gel(D,i);
+    long add_N = modsame;
+    for (j = 1; j < def; j++)
     {
-      gcoeff(x,i,j) = centermodii(gcoeff(x,i,j), d,d2);
-      a = gcoeff(x,i,j);
+      GEN p1, p2, b, a = gcoeff(x,i,j) = remii(gcoeff(x,i,j), d);
       if (!signe(a)) continue;
 
-      k = (j==1)? def: j-1;
-      gcoeff(x,i,k) = centermodii(gcoeff(x,i,k), d,d2);
-      ZC_elem(a,gcoeff(x,i,k), x,NULL, j,k);
+      k = j+1;
+      b = gcoeff(x,i,k) = remii(gcoeff(x,i,k), d);
+      if (!signe(b)) { swap(gel(x,j), gel(x,k)); continue; }
+      if (add_N)
+      { /* ensure the moving pivot on row i divides d from now on */
+        add_N = 0;
+        if (!equali1(a))
+        { /* x[j] *= u; after this, a = x[i,j] | d */
+          GEN u = Fp_invgen(a, d, &a);
+          long t;
+          p1 = gel(x,j);
+          for (t = 1; t < i; t++) gel(p1,t) = mulii(gel(p1,t), u);
+          FpV_red_part_ipvec(p1, D, i-1);
+          gel(p1,i) = a;
+          if (2*lg(a) < lg(b))
+          { /* reduce x[i,k] mod x[i,j]: helps ZC_elem */
+            GEN r, q = dvmdii(b, a, &r);
+            togglesign(q);
+            ZC_lincomb1_inplace_i(gel(x,k), gel(x,j), q, i-1);
+            FpV_red_part_ipvec(gel(x,k), D, i-1);
+            gcoeff(x,i,k) = b = r;
+          }
+        }
+      }
+      ZC_elem(a,b, x, NULL, j,k);
       p1 = gel(x,j);
       p2 = gel(x,k);
-      /* prevent coeffs explosion: reduce mod dm when lg() > ldm */
+      /* prevent coeffs explosion */
       for (k = 1; k < i; k++)
       {
-        if (lgefint(gel(p1,k)) > LDM[k])
-          gel(p1,k) = centermodii(gel(p1,k), gel(dm,k),gel(dm2,k));
-        if (lgefint(gel(p2,k)) > LDM[k])
-          gel(p2,k) = centermodii(gel(p2,k), gel(dm,k),gel(dm2,k));
+        if (lgefint(gel(p1,k)) > LDM[k]) gel(p1,k) = remii(gel(p1,k),gel(D,k));
+        if (lgefint(gel(p2,k)) > LDM[k]) gel(p2,k) = remii(gel(p2,k),gel(D,k));
       }
     }
-    if (gc_needed(av,1))
+    if (gc_needed(av,2))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"ZM_hnfmod[1]. i=%ld",i);
       x = gerepilecopy(av, x);
@@ -1220,86 +1274,87 @@ ZM_hnfmodall_i(GEN x, GEN dm, long flag)
     { /* missing pivot on line i, insert column */
       GEN a = cgetg(co + 1, t_MAT);
       for (k = 1; k <= def; k++) gel(a,k) = gel(x,k);
-      gel(a,k++) = Rg_col_ei(gel(dm,i), li-1, i);
+      gel(a,k++) = Rg_col_ei(gel(D,i), nli, i);
       for (     ; k <= co;  k++) gel(a,k) = gel(x,k-1);
       ldef--; if (ldef < 0) ldef = 0;
       co++; def++; x = a;
     }
   }
   if (co < li)
-  { /* implies moddiag, add missing diag(dm) components */
+  { /* implies moddiag, add missing diag(D) components */
     GEN a = cgetg(li+1, t_MAT);
-    for (k = 1; k <= li-co; k++) gel(a,k) = Rg_col_ei(gel(dm,k), li-1, k);
+    for (k = 1; k <= li-co; k++) gel(a,k) = Rg_col_ei(gel(D,k), nli, k);
     for (i = 1; i < co; i++) gel(a,k-1+i) = gel(x,i);
-    gel(a,li) = zerocol(li-1); x = a;
+    gel(a,li) = zerocol(nli); x = a;
   }
   else
   {
     x += co - li;
     x[0] = evaltyp(t_MAT) | evallg(li); /* kill 0 columns */
+    if (moddiag) x = shallowconcat(x, zerocol(nli));
   }
   if (moddiag)
-  { /* one column extra: an accumulator, discarded at the end */
-    if (lg(x) == li) x = shallowconcat(x, zerocol(li-1));
-    /* add up missing diag(dm) components */
-    for (i = li-1; i > 0; i--)
+  { /* x[li]: extra column, an accumulator discarded at the end */
+    GEN D2;
+    gcoeff(x,1,1) = gcdii(gcoeff(x,1,1), gel(D,1));
+    D2 = optimal_D(x,D);
+    /* add up missing diag(D) components */
+    for (i = nli; i > 0; i--)
     {
-      gcoeff(x, i, li) = gel(dm,i);
+      gcoeff(x, i, li) = gel(D,i);
       for (j = i; j > 0; j--)
       {
         GEN a = gcoeff(x, j, li);
         if (!signe(a)) continue;
         ZC_elem(a, gcoeff(x,j,j), x, NULL, li,j);
-        FpV_red_part_ipvec(gel(x,li), dm, j-1);
-        FpV_red_part_ipvec(gel(x,j),  dm, j-1);
+        FpV_red_part_ipvec(gel(x,li), D, j-1);
+        FpV_red_part_ipvec(gel(x,j),  D, j-1);
       }
       if (gc_needed(av,1))
       {
         if (DEBUGMEM>1) pari_warn(warnmem,"ZM_hnfmod[2]. i=%ld", i);
-        x = gerepilecopy(av, x);
+        gerepileall(av, 2, &x, &D2);
       }
     }
+    D = D2;
   }
   else
   {
-    b = gel(dm,1);
-    for (i = li-1; i > 0; i--)
+    GEN b = gel(D,1);
+    for (i = nli; i > 0; i--)
     {
       GEN d = bezout(gcoeff(x,i,i),b, &u,NULL);
       gcoeff(x,i,i) = d;
       FpV_Fp_mul_part_ip(gel(x,i), u, b, i-1);
       if (i > 1) b = diviiexact(b,d);
     }
+    D = optimal_D(x,D);
   }
   x[0] = evaltyp(t_MAT) | evallg(li); /* kill 0 columns / discard accumulator */
   if (flag & hnf_PART) return x;
 
-  if (!moddiag)
-  { /* compute optimal value for dm */
-    b = cgetg(li, t_VEC); gel(b,1) = gcoeff(x,1,1);
-    for (i = 2; i < li; i++) gel(b,i) = mulii(gel(b,i-1), gcoeff(x,i,i));
-    dm = b;
-  }
-
-  for (i = li-1; i > 0; i--)
+  for (i = nli; i > 0; i--)
   {
     GEN diag = gcoeff(x,i,i);
     if (signe(diag) < 0) { gel(x,i) = ZC_neg(gel(x,i)); diag = gcoeff(x,i,i); }
+    if (i != nli)
+      for (j = 1; j < i; j++) gcoeff(x,j,i) = remii(gcoeff(x,j,i), gel(D,j));
     for (j = i+1; j < li; j++)
     {
-      b = gcoeff(x,i,j);
-      b = center? diviiround(b,diag): truedivii(b, diag);
-      if (!signe(b)) continue;
-      togglesign(b);
-      ZC_lincomb1_inplace(gel(x,j), gel(x,i),b);
-      p1 = gel(x,j);
-      for (k=1; k<i; k++)
-        if (lgefint(gel(p1,k)) > LDM[k]) gel(p1,k) = remii(gel(p1,k), gel(dm,i));
+      GEN b = gcoeff(x,i,j) = remii(gcoeff(x,i,j), gel(D,i));
+      GEN r, q = truedvmdii(b, diag, &r);
+      /* ensure -diag/2 <= r < diag/2 */
+      if (center && signe(r) && abscmpii(shifti(r,1),diag) >= 0)
+      { r = subii(r,diag); q = addiu(q,1); }
+      if (!signe(q)) continue;
+      togglesign(q);
+      ZC_lincomb1_inplace_i(gel(x,j), gel(x,i), q, i-1);
+      gcoeff(x,i,j) = r;
     }
     if (gc_needed(av,1))
     {
       if (DEBUGMEM>1) pari_warn(warnmem,"ZM_hnfmod[3]. i=%ld", i);
-      gerepileall(av, 2, &x, &dm); diag = gcoeff(x,i,i);
+      gerepileall(av, 2, &x, &D);
     }
   }
   return x;
@@ -1313,7 +1368,109 @@ ZM_hnfmodall(GEN x, GEN dm, long flag)
 GEN
 ZM_hnfmod(GEN x, GEN d) { return ZM_hnfmodall(x,d,0); }
 GEN
-ZM_hnfmodid(GEN x, GEN d) { return ZM_hnfmodall(x,d,hnf_MODID); }
+ZM_hnfmodid(GEN x, GEN d)
+{ return ZM_hnfmodall(x,d,hnf_MODID); }
+/* return the column echelon form of x with 1's as pivots,
+ * P contains the row indices containing the pivots in increasing order */
+static GEN
+FpM_echelon(GEN x, GEN *pP, GEN p)
+{
+  pari_sp av;
+  long iP, li, co, i, j, k, def, ldef;
+  GEN P;
+
+  co = lg(x); if (co == 1) { *pP = cgetg(1,t_VECSMALL); return cgetg(1,t_MAT); }
+  li = lgcols(x);
+  iP = 1;
+  *pP = P = cgetg(li, t_VECSMALL);
+  av = avma;
+  x = FpM_red(x, p);
+
+  ldef = (li > co)? li - co: 0;
+  for (def = co-1,i = li-1; i > ldef; i--)
+  {
+    GEN u = NULL;
+    for (k = def; k; k--)
+    {
+      u = gcoeff(x,i,k);
+      if (signe(u)) break;
+    }
+    if (!k)
+    {
+      if (--ldef < 0) ldef = 0;
+      continue;
+    }
+    P[iP++] = i;
+    if (k != def) swap(gel(x,def), gel(x,k));
+    if (!equali1(u))
+      FpV_Fp_mul_part_ip(gel(x,def), Fp_inv(u,p), p, i-1);
+    gcoeff(x, i, def) = gen_1;
+    for (j = def-1; j; j--)
+    { /* zero x[i, 1..def-1] using x[i,def] = 1*/
+      GEN xj = gel(x,j), u = gel(xj,i);
+      if (!signe(u)) continue;
+
+      ZC_lincomb1_inplace(xj, gel(x,def), negi(u));
+      for (k = 1; k < i; k++) gel(xj,k) = modii(gel(xj,k), p);
+    }
+    if (gc_needed(av,2))
+    {
+      if (DEBUGMEM>1) pari_warn(warnmem,"FpM_echelon. i=%ld",i);
+      x = gerepilecopy(av, x);
+    }
+    def--;
+  }
+  /* rank = iP - 1 */
+  setlg(P, iP); vecsmall_sort(P);
+  if (co > iP) x += co - iP;
+  x[0] = evaltyp(t_MAT) | evallg(iP);
+  return x;
+}
+/* given x square of maximal rank with 1 or p on diagonal from hnfmodid
+ * (=> a column containing p has its other entries at 0 ), return the HNF */
+static GEN
+FpM_hnfend(pari_sp av, GEN x, GEN p)
+{
+  long i, l = lgcols(x);
+  for (i = l-1; i > 0; i--)
+  {
+    GEN diag = gcoeff(x,i,i);
+    long j;
+    if (is_pm1(diag))
+      for (j = i+1; j < l; j++)
+      {
+        GEN xj = gel(x,j), b = gel(xj,i);
+        long k;
+        if (!signe(b)) continue;
+        ZC_lincomb1_inplace(xj, gel(x,i), negi(b));
+        for (k=1; k<i; k++)
+          if (lgefint(gel(xj,k)) > 3) gel(xj,k) = remii(gel(xj,k), p);
+      }
+    else
+      for (j = i+1; j < l; j++) gcoeff(x,i,j) = modii(gcoeff(x,i,j), p);
+    if (gc_needed(av,2))
+    {
+      if (DEBUGMEM>1) pari_warn(warnmem,"FpM_hnfend. i=%ld",i);
+      x = gerepilecopy(av, x);
+    }
+  }
+  return x;
+}
+GEN
+ZM_hnfmodprime(GEN x, GEN p)
+{
+  pari_sp av = avma;
+  GEN P, y;
+  long l, lP, i;
+  if (lg(x) == 1) return cgetg(1, t_MAT);
+  l = lgcols(x);
+  x = FpM_echelon(x, &P, p);
+  lP = lg(P); /* rank = lP-1 */
+  if (lP == l) { avma = av; return matid(l-1); }
+  y = scalarmat_shallow(p, l-1);
+  for (i = 1; i < lP; i++) gel(y,P[i]) = gel(x,i);
+  return gerepilecopy(av, FpM_hnfend(av,y,p));
+}
 
 static GEN
 allhnfmod(GEN x, GEN dm, int flag)
@@ -1434,7 +1591,7 @@ reduce2(GEN A, GEN B, long k, long j, long *row0, long *row1, GEN lambda, GEN D)
     togglesign_safe(&q);
     if (*row0) ZC_lincomb1_inplace(gel(A,k),gel(A,j),q);
     if (B) ZC_lincomb1_inplace(gel(B,k),gel(B,j),q);
-    gel(Lk,j) = addii(gel(Lk,j), mulii(q,gel(D,j)));
+    gel(Lk,j) = addmulii(gel(Lk,j), q, gel(D,j));
     if (is_pm1(q))
     {
       if (signe(q) > 0)
@@ -1451,7 +1608,7 @@ reduce2(GEN A, GEN B, long k, long j, long *row0, long *row1, GEN lambda, GEN D)
     else
     {
       for (i=1; i<j; i++)
-        if (signe(gel(Lj,i))) gel(Lk,i) = addii(gel(Lk,i), mulii(q,gel(Lj,i)));
+        if (signe(gel(Lj,i))) gel(Lk,i) = addmulii(gel(Lk,i), q, gel(Lj,i));
     }
   }
 }
@@ -1497,14 +1654,20 @@ reverse_rows(GEN A)
   }
   return A;
 }
+/* decide whether to swap */
+static int
+must_swap(long k, GEN lambda, GEN D)
+{
+  pari_sp av = avma;
+  GEN z = addii(mulii(gel(D,k-2),gel(D,k)), sqri(gcoeff(lambda,k-1,k)));
+  long s = cmpii(z, sqri(gel(D,k-1)));
+  avma = av; return s < 0;
+}
 
 GEN
 ZM_hnflll(GEN A, GEN *ptB, int remove)
 {
   pari_sp av = avma;
-#ifdef HNFLLL_QUALITY
-  const long m1 = 1, n1 = 1; /* alpha = m1/n1. Maybe 3/4 here ? */
-#endif
   long n, k, kmax;
   GEN B, lambda, D;
 
@@ -1519,21 +1682,9 @@ ZM_hnflll(GEN A, GEN *ptB, int remove)
     long row0, row1;
     int do_swap;
     reduce2(A,B,k,k-1,&row0,&row1,lambda,D);
-    if (row0)
-      do_swap = (!row1 || row0 <= row1);
-    else if (!row1)
-    { /* row0 == row1 == 0 */
-      pari_sp av1 = avma;
-      GEN z = addii(mulii(gel(D,k-2),gel(D,k)), sqri(gcoeff(lambda,k-1,k)));
-#ifdef HNFLLL_QUALITY
-      do_swap = (cmpii(mului(n1,z), mului(m1,sqri(gel(D,k-1)))) < 0);
-#else /* assume m1 = n1 = 1 */
-      do_swap = (cmpii(z, sqri(gel(D,k-1))) < 0);
-#endif
-      avma = av1;
-    }
-    else
-      do_swap = 0;
+    if      (row0) do_swap = (!row1 || row0 <= row1);
+    else if (row1) do_swap = 0;
+    else do_swap = must_swap(k,lambda,D);
     if (do_swap)
     {
       hnfswap(A,B,k,lambda,D);
@@ -1606,20 +1757,17 @@ reduce1(GEN A, GEN B, long k, long j, GEN lambda, GEN D)
   {
     GEN Lk = gel(lambda,k), Lj = gel(lambda,j);
     togglesign_safe(&q);
-    gel(A,k) = addii(gel(A,k), mulii(q,gel(A,j)));
+    gel(A,k) = addmulii(gel(A,k), q, gel(A,j));
     ZC_lincomb1_inplace(gel(B,k),gel(B,j),q);
-    gel(Lk,j) = addii(gel(Lk,j), mulii(q,gel(D,j)));
+    gel(Lk,j) = addmulii(gel(Lk,j), q, gel(D,j));
     for (i=1; i<j; i++)
-      if (signe(gel(Lj,i))) gel(Lk,i) = addii(gel(Lk,i), mulii(q,gel(Lj,i)));
+      if (signe(gel(Lj,i))) gel(Lk,i) = addmulii(gel(Lk,i), q, gel(Lj,i));
   }
 }
 
 static GEN
 ZV_gcdext_i(GEN A)
 {
-#ifdef HNFLLL_QUALITY
-  const long m1 = 1, n1 = 1; /* alpha = m1/n1. Maybe 3/4 here ? */
-#endif
   long k, n = lg(A);
   GEN B, lambda, D;
 
@@ -1634,20 +1782,9 @@ ZV_gcdext_i(GEN A)
     int do_swap;
 
     reduce1(A,B,k,k-1,lambda,D);
-    if (signe(gel(A,k-1))) do_swap = 1;
-    else if (!signe(gel(A,k)))
-    {
-      pari_sp av1 = avma;
-      GEN z = addii(mulii(gel(D,k-2),gel(D,k)), sqri(gcoeff(lambda,k-1,k)));
-#ifdef HNFLLL_QUALITY
-      do_swap = (cmpii(mului(n1,z), mului(m1,sqri(gel(D,k-1)))) < 0);
-#else /* assume m1 = n1 = 1 */
-      do_swap = (cmpii(z, sqri(gel(D,k-1))) < 0);
-#endif
-      avma = av1;
-    }
-    else do_swap = 0;
-
+    if    (signe(gel(A,k-1))) do_swap = 1;
+    else if (signe(gel(A,k))) do_swap = 0;
+    else do_swap = must_swap(k,lambda,D);
     if (do_swap)
     {
       hnfswap(A,B,k,lambda,D);
@@ -1866,10 +2003,11 @@ ZM_hnfall_i(GEN A, GEN *ptB, long remove)
         k = c[i];
         /* zero a = Aij  using  Aik */
         if (signe(a)) ZC_elem(a,gcoeff(A,i,k), A,B,j,k);
-        ZM_reduce(A,B, i,k); /* ensure reduced entries */
+        ZM_reduce(A,B, i,k); /* ensure reduced entries even if a = 0 */
         if (gc_needed(av,1))
         {
-          if (DEBUGMEM>1) pari_warn(warnmem,"hnfall[1], li = %ld", li);
+          if (DEBUGMEM>1)
+            pari_warn(warnmem,"ZM_hnfall[1], li = %ld, j = %ld", li, j);
           gerepileall(av, B? 2: 1, &A, &B);
         }
       }
@@ -1892,7 +2030,7 @@ ZM_hnfall_i(GEN A, GEN *ptB, long remove)
     ZM_reduce(A,B, li,r);
     if (gc_needed(av,1))
     {
-      if (DEBUGMEM>1) pari_warn(warnmem,"hnfall[2], li = %ld", li);
+      if (DEBUGMEM>1) pari_warn(warnmem,"ZM_hnfall[2], li = %ld", li);
       gerepileall(av, B? 2: 1, &A, &B);
     }
   }
@@ -1908,7 +2046,7 @@ ZM_hnfall_i(GEN A, GEN *ptB, long remove)
       ZM_reduce(A,B, i,k); /* ensure reduced entries, even if a = 0 */
       if (gc_needed(av,1))
       {
-        if (DEBUGMEM>1) pari_warn(warnmem,"hnfall[3], j = %ld", j);
+        if (DEBUGMEM>1) pari_warn(warnmem,"ZM_hnfall[3], j = %ld", j);
         gerepileall(av, B? 2: 1, &A, &B);
       }
     }
@@ -2141,11 +2279,9 @@ ZM_snf_no_divide(GEN x, long i)
     return 0;
   }
   if (is_pm1(b)) return 0;
-  for (k=1; k<i; k++)
-  {
-    for (j=1; j<i; j++)
-      if (signe(remii(gcoeff(x,k,j),b))) return k;
-  }
+  for (k = 1; k < i; k++)
+    for (j = 1; j < i; j++)
+      if (!dvdii(gcoeff(x,k,j),b)) return k;
   return 0;
 }
 
@@ -2638,19 +2774,23 @@ snf_group(GEN H, GEN D, GEN *newU, GEN *newUi)
     }
     *newU = U;
   }
-  if (newUi) { /* UHV=D -> U^-1 = (HV)D^-1 -> U^-1 = H(VD^-1 mod 1) mod H */
-    if (l > 1)
-    { /* Ui = ZM_inv(U, gen_1); setlg(Ui, l); */
-      GEN V = *newUi, Ui;
-      for (i = 1; i < l; i++) gel(V,i) = FpC_red(gel(V,i), gel(D,i));
-      Ui = typ(H) == t_VEC? ZM_diag_mul(H, V): ZM_mul(H, V);
-      for (i = 1; i < l; i++) gel(Ui,i) = ZC_Z_divexact(gel(Ui,i), gel(D,i));
-      if (typ(H) == t_VEC)
-      { for (i = 1; i < l; i++) gel(Ui,i) = vecmodii(gel(Ui,i), H); }
-      else
-        Ui = ZM_hnfrem(Ui, H);
-      *newUi = Ui;
+  if (newUi && l > 1)
+  { /* UHV=D -> U^-1 = (HV)D^-1 -> U^-1 = H(VD^-1 mod 1) mod H */
+    /* Ui = ZM_inv(U, NULL); setlg(Ui, l); */
+    GEN V = *newUi, Ui;
+    int Hvec = (typ(H) == t_VEC);
+    for (i = 1; i < l; i++) gel(V,i) = FpC_red(gel(V,i), gel(D,i));
+    if (!Hvec)
+    {
+      if (ZM_isdiagonal(H)) { H = RgM_diagonal_shallow(H); Hvec = 1; }
     }
+    Ui = Hvec? ZM_diag_mul(H, V): ZM_mul(H, V);
+    for (i = 1; i < l; i++) gel(Ui,i) = ZC_Z_divexact(gel(Ui,i), gel(D,i));
+    if (Hvec)
+    { for (i = 1; i < l; i++) gel(Ui,i) = vecmodii(gel(Ui,i), H); }
+    else
+      Ui = ZM_hnfrem(Ui, H);
+    *newUi = Ui;
   }
   return D;
 }
@@ -2683,7 +2823,7 @@ ZV_snfall(GEN D, GEN *pU, GEN *pV)
     GEN b = gel(D,j);
     if (signe(b) < 0)
     {
-      gel(D,j) = absi(b);
+      gel(D,j) = negi(b);
       if (V) ZV_togglesign(gel(V,j));
     }
   }
