@@ -921,7 +921,7 @@ lfunparams(GEN ldata, long der, long bitprec, struct lfunp *S)
   S->logN2 = log(gtodouble(N)) / 2;
   maxs = S->dc + S->dw;
   mins = S->dc - S->dw;
-  S->MAXs = maxss(maxs, k-mins);
+  S->MAXs = maxdd(maxs, k-mins);
 
   /* we compute Lambda^(der)(s) / der!; need to compensate for L^(der)(s)
    * ln |gamma(s)| ~ (pi/4) d |t|; max with 1: fudge factor */
@@ -1032,18 +1032,18 @@ lfuninit_vecc2_sum(GEN an, GEN qk, GEN a, struct lfunp *Q, GEN poqk)
 
 /* theta(exp(mh)) ~ sum_{n <= L[m]} a(n) k[m,n] */
 static GEN
-lfuninit_vecc_sum(GEN L, long M, GEN vecan, GEN vK, GEN pokq, long prec)
+lfuninit_vecc_sum(GEN L, long M, GEN an, GEN vK, GEN pokq, long prec)
 {
-  long m;
+  long m, L0 = lg(an)-1;
   GEN vecc = cgetg(M+2, t_VEC);
   for (m = 0; m <= M; ++m)
   {
     pari_sp av = avma;
     GEN s = gen_0, vKm = gel(vK,m+1);
-    long n;
-    for (n = 1; n <= L[m+1]; n++)
+    long n, N = minss(L0, L[m+1]);
+    for (n = 1; n <= N; n++)
     {
-      GEN c = mul_an(vecan, n, gel(vKm,n));
+      GEN c = mul_an(an, n, gel(vKm,n));
       if (c)
       {
         s = gadd(s, c);
@@ -2156,7 +2156,7 @@ lfunzeros(GEN ldata, GEN lim, long divz, long bitprec)
   if (typ(lim) == t_VEC)
   {
     if (lg(lim) != 3 || gcmp(gel(lim,1),gel(lim,2)) >= 0
-                     || gcmp(gel(lim,1),gen_0) <= 0)
+                     || gcmp(gel(lim,1),gen_0) < 0)
       pari_err_TYPE("lfunzeros",lim);
     h1 = gel(lim,1); h2 = gel(lim,2);
   }
