@@ -746,12 +746,10 @@ dirzetak0(GEN nf, ulong N)
   long court[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3),0};
   forprime_t S;
 
-  (void)evallg(N+1);
   c  = cgetalloc(t_VECSMALL, N+1);
   c2 = cgetalloc(t_VECSMALL, N+1);
   c2[1] = c[1] = 1; for (i=2; i<=N; i++) c[i] = 0;
-  u_forprime_init(&S, 2, N);
-  av2 = avma;
+  u_forprime_init(&S, 2, N); av2 = avma;
   while ( (p = u_forprime_next(&S)) )
   {
     avma = av2;
@@ -2332,16 +2330,14 @@ lfunartin(GEN nf, GEN gal, GEN ch, long o, long bitprec)
   repr = gel(cc,3);
   mod = mkpolmod(gen_1, polcyclo(o, var));
   if (lg(ch)>1 && typ(gel(ch,1))==t_MAT)
-  {
     chx = artin_charfromgens(gal, gmul(ch,mod));
-    ch = shallowextract(chx, repr);
-  }
   else
   {
     if (lg(repr) != lg(ch)) pari_err_DIM("lfunartin");
     chx = char_expand(conj, gmul(ch,mod));
   }
   chx = handle_zeta(nf_get_degree(nf), chx, &tmult);
+  ch = shallowextract(chx, repr);
   if (!gequal0(chx))
   {
     GEN real = char_is_real(chx, gel(mod,1))? gen_0: gen_1;
