@@ -225,6 +225,19 @@ vecvecsmall_search(GEN x, GEN y, long flag)
   return gen_search(x,y,flag,(void*)vecsmall_prefixcmp, cmp_nodata);
 }
 
+/* assume x non empty */
+long
+vecvecsmall_max(GEN x)
+{
+  long i, l = lg(x), m = vecsmall_max(gel(x,1));
+  for (i = 2; i < l; i++)
+  {
+    long t = vecsmall_max(gel(x,i));
+    if (t > m) m = t;
+  }
+  return m;
+}
+
 /*************************************************************************/
 /**                                                                     **/
 /**                  Routines for handling permutations                 **/
@@ -247,26 +260,27 @@ perm_mul_inplace2(GEN s, GEN t)
 static GEN
 vecperm_orbits_i(GEN v, long n)
 {
-  long mj = 1, k, l, m;
-  GEN cy, cycle = cgetg(n+1, t_VEC), bit = zero_F2v(n);
+  long mj = 1, lv = lg(v), k, l;
+  GEN cycle = cgetg(n+1, t_VEC), bit = const_vecsmall(n, 0);
   for (k = 1, l = 1; k <= n;)
   {
-    for (  ; F2v_coeff(bit,mj); mj++) /*empty*/;
-    cy = cgetg(n+1, t_VECSMALL);
-    m = 1; k++; cy[m++] = mj;
-    F2v_set(bit, mj++);
+    long m = 1;
+    GEN cy = cgetg(n+1, t_VECSMALL);
+    for (  ; bit[mj]; mj++) /*empty*/;
+    k++; cy[m++] = mj;
+    bit[mj++] = 1;
     for(;;)
     {
       long o, mold = m;
-      for (o = 1; o < lg(v); o++)
+      for (o = 1; o < lv; o++)
       {
         GEN vo = gel(v,o);
         long p;
         for (p = 1; p < m; p++) /* m increases! */
         {
           long j = vo[ cy[p] ];
-          if (!F2v_coeff(bit,j)) cy[m++] = j;
-          F2v_set(bit,j);
+          if (!bit[j]) cy[m++] = j;
+          bit[j] = 1;
         }
       }
       if (m == mold) break;
@@ -292,6 +306,23 @@ perm_cycles(GEN v)
   return gerepilecopy(av, vecperm_orbits_i(mkvec(v), lg(v)-1));
 }
 
+static long
+isperm(GEN v)
+{
+  pari_sp av = avma;
+  long i, n = lg(v)-1;
+  GEN w;
+  if (typ(v) != t_VECSMALL) return 0;
+  w = zero_zv(n);
+  for (i=1; i<=n; i++)
+  {
+    long d = v[i];
+    if (d < 1 || d > n || w[d]) { avma = av; return 0; }
+    w[d] = 1;
+  }
+  avma = av; return 1;
+}
+
 /* Output the order of p */
 long
 perm_order(GEN v)
@@ -299,10 +330,106 @@ perm_order(GEN v)
   pari_sp ltop = avma;
   GEN c = vecperm_orbits_i(mkvec(v), lg(v)-1);
   long i, d;
-  for(i=1, d=1; i<lg(c); i++) d = clcm(d, lg(gel(c,i))-1);
+  for(i=1, d=1; i<lg(c); i++) d = ulcm(d, lg(gel(c,i))-1);
   avma = ltop; return d;
 }
 
+long
+permorder(GEN v)
+{
+  if (!isperm(v)) pari_err_TYPE("permorder",v);
+  return perm_order(v);
+}
+
+/* sign of a permutation */
+long
+perm_sign(GEN v)
+{
+  pari_sp av = avma;
+  GEN c = vecperm_orbits_i(mkvec(v), lg(v)-1);
+  long i, l = lg(c), s = 1;
+  for (i = 1; i < l; i++)
+    if (odd(lg(gel(c, i)))) s = -s;
+  avma = av; return s;
+}
+
+long
+permsign(GEN v)
+{
+  if (!isperm(v)) pari_err_TYPE("permsign",v);
+  return perm_sign(v);
+}
+
+GEN
+Z_to_perm(long n, GEN x)
+{
+  pari_sp av;
+  ulong i, r;
+  GEN v = cgetg(n+1, t_VECSMALL);
+  if (n==0) return v;
+  uel(v,n) = 1; av = avma;
+  if (signe(x) <= 0) x = modii(x, mpfact(n));
+  for (r=n-1; r>=1; r--)
+  {
+    ulong a;
+    x = absdiviu_rem(x, n+1-r, &a);
+    for (i=r+1; i<=(ulong)n; i++)
+      if (uel(v,i) > a) uel(v,i)++;
+    uel(v,r) = a+1;
+  }
+  avma = av; return v;
+}
+GEN
+numtoperm(long n, GEN x)
+{
+  if (n < 0) pari_err_DOMAIN("numtoperm", "n", "<", gen_0, stoi(n));
+  if (typ(x) != t_INT) pari_err_TYPE("numtoperm",x);
+  return Z_to_perm(n, x);
+}
+
+/* destroys v */
+static GEN
+perm_to_Z_inplace(GEN v)
+{
+  long l = lg(v), i, r;
+  GEN x = gen_0;
+  if (!isperm(v)) pari_err_TYPE("permsign",v);
+  for (i = 1; i < l; i++)
+  {
+    long vi = v[i];
+    if (vi <= 0) return NULL;
+    x = i==1 ? utoi(vi-1): addiu(muliu(x,l-i), vi-1);
+    for (r = i+1; r < l; r++)
+      if (v[r] > vi) v[r]--;
+  }
+  return x;
+}
+GEN
+perm_to_Z(GEN v)
+{
+  pari_sp av = avma;
+  GEN x = perm_to_Z_inplace(leafcopy(v));
+  if (!x) pari_err_TYPE("permtonum",v);
+  return gerepileuptoint(av, x);
+}
+GEN
+permtonum(GEN p)
+{
+  pari_sp av = avma;
+  GEN v, x;
+  switch(typ(p))
+  {
+    case t_VECSMALL: return perm_to_Z(p);
+    case t_VEC: case t_COL:
+      if (RgV_is_ZV(p)) { v = ZV_to_zv(p); break; }
+    default: pari_err_TYPE("permtonum",p); return NULL;
+  }
+  x = perm_to_Z_inplace(v);
+  if (!x) pari_err_TYPE("permtonum",p);
+  return gerepileuptoint(av, x);
+}
+
+
 GEN
 cyc_pow(GEN cyc, long exp)
 {
@@ -354,15 +481,31 @@ cyc_pow_perm(GEN cyc, long exp)
   return p;
 }
 
-/* Compute the power of a permutation.
- * TODO: make it more clever for small exp */
 GEN
 perm_pow(GEN perm, long exp)
 {
-  return cyc_pow_perm(perm_cycles(perm), exp);
+  long i, r = lg(perm)-1;
+  GEN p = zero_zv(r);
+  pari_sp av = avma;
+  GEN v = cgetg(r+1, t_VECSMALL);
+  for (i=1; i<=r; i++)
+  {
+    long e, n, k, l;
+    if (p[i]) continue;
+    v[1] = i;
+    for (n=1, k=perm[i]; k!=i; k=perm[k], n++)
+      v[n+1] = k;
+    e = smodss(exp, n);
+    for (k = 1, l = e; k <= n; k++)
+    {
+      p[v[k]] = v[l+1];
+      if (++l == n) l = 0;
+    }
+  }
+  avma = av; return p;
 }
 
-GEN
+static GEN
 perm_to_GAP(GEN p)
 {
   pari_sp ltop=avma;
@@ -957,6 +1100,14 @@ group_subgroup_isnormal(GEN G, GEN H)
   return 1;
 }
 
+long
+groupelts_exponent(GEN elts)
+{
+  long i, n = lg(elts)-1, expo = 1;
+  for(i=1; i<=n; i++) expo = ulcm(expo, perm_order(gel(elts,i)));
+  return expo;
+}
+
 GEN
 groupelts_center(GEN S)
 {
@@ -979,6 +1130,73 @@ groupelts_center(GEN S)
   return gerepileupto(ltop,V);
 }
 
+GEN
+groupelts_conjclasses(GEN elts, long *pnbcl)
+{
+  long i, j, cl = 0, n = lg(elts)-1;
+  GEN c = const_vecsmall(n,0);
+  pari_sp av = avma;
+  for (i=1; i<=n; i++)
+  {
+    GEN g = gel(elts,i);
+    if (c[i]) continue;
+    c[i] = ++cl;
+    for(j=1; j<=n; j++)
+      if (j != i)
+      {
+        GEN h = perm_conj(gel(elts,j), g);
+        long i2 = gen_search(elts,h,0,(void*)&vecsmall_lexcmp,&cmp_nodata);
+        c[i2] = cl;
+        avma = av;
+      }
+  }
+  if (pnbcl) *pnbcl = cl;
+  return c;
+}
+
+GEN
+conjclasses_repr(GEN conj, long nb)
+{
+  long i, l = lg(conj);
+  GEN e = const_vecsmall(nb, 0);
+  for(i=1; i<l; i++)
+  {
+    long ci = conj[i];
+    if (!e[ci]) e[ci] = i;
+  }
+  return e;
+}
+
+/* elts of G sorted wrt vecsmall_lexcmp order: g in G is determined by g[1]
+ * so sort by increasing g[1] */
+static GEN
+galois_elts_sorted(GEN gal)
+{
+  long i, l;
+  GEN elts = gal_get_group(gal), v = cgetg_copy(elts, &l);
+  for (i = 1; i < l; i++) { GEN g = gel(elts,i); gel(v, g[1]) = g; }
+  return v;
+}
+GEN
+group_to_cc(GEN G)
+{
+  GEN elts = checkgroupelts(G), z = cgetg(5,t_VEC);
+  long n, flag = 1;
+  if (typ(gel(G,1)) == t_POL)
+    elts = galois_elts_sorted(G); /* galoisinit */
+  else
+  {
+    long i, l = lg(elts);
+    elts = gen_sort(elts,(void*)vecsmall_lexcmp,cmp_nodata); /* general case */
+    for (i = 1; i < l; i++)
+      if (gel(elts,i)[1] != i) { flag = 0; break; }
+  }
+  gel(z,1) = elts;
+  gel(z,2) = groupelts_conjclasses(elts,&n);
+  gel(z,3) = conjclasses_repr(gel(z,2),n);
+  gel(z,4) = utoi(flag); return z;
+}
+
 /* S a list of generators */
 GEN
 groupelts_abelian_group(GEN S)
