@@ -2877,25 +2877,36 @@ resultant2(GEN x, GEN y)
   return r? r: gerepileupto(av, det(RgX_sylvestermatrix(x,y)));
 }
 
-/* If x a t_POL, let vx = main variable of x; return a t_POL in variable v0:
+/* let vx = main variable of x, v0 a variable of highest priority;
+ * return a t_POL in variable v0:
  * if vx <= v, return subst(x, v, pol_x(v0))
  * if vx >  v, return scalarpol(x, v0) */
 static GEN
 fix_pol(GEN x, long v, long v0)
 {
-  long vx;
-  if (typ(x) != t_POL) return x;
-  vx = varn(x);
-  if (v == vx)
-  {
-    if (v0 != v) { x = leafcopy(x); setvarn(x, v0); }
-    return x;
+  long vx, tx = typ(x);
+  if (tx != t_POL)
+    vx = gvar(x);
+  else
+  { /* shortcut: almost nothing to do */
+    vx = varn(x);
+    if (v == vx)
+    {
+      if (v0 != v) { x = leafcopy(x); setvarn(x, v0); }
+      return x;
+    }
   }
   if (varncmp(v, vx) > 0)
   {
-    x = gsubst(x,v,pol_x(v0));
-    if (typ(x) == t_POL && varn(x) == v0) return x;
+    x = gsubst(x, v, pol_x(v0));
+    if (typ(x) != t_POL) vx = gvar(x);
+    else
+    {
+      vx = varn(x);
+      if (vx == v0) return x;
+    }
   }
+  if (varncmp(vx, v0) <= 0) pari_err_TYPE("polresultant", x);
   return scalarpol_shallow(x, v0);
 }
 
@@ -3172,7 +3183,7 @@ static GEN
 RgX_disc_aux(GEN P)
 {
   long n = degpol(P), TP, dd;
-  GEN D, L, y, p;
+  GEN N, D, L, y, p;
   if (!signe(P) || !n) return Rg_get_0(P);
   if (n == 1) return Rg_get_1(P);
   if (n == 2) {
@@ -3187,6 +3198,8 @@ RgX_disc_aux(GEN P)
     return Fp_to_mod(FpX_disc(RgX_to_FpX(P,p), p), p);
 
   y = RgX_deriv(P);
+  N = characteristic(P);
+  if (signe(N)) y = gmul(y, mkintmod(gen_1,N));
   if (!signe(y)) return Rg_get_0(y);
   dd = degpol(P)-2 - degpol(y);
   if (TP == t_REAL)
@@ -3207,35 +3220,21 @@ RgX_disc(GEN x) { pari_sp av = avma; return gerepileupto(av, RgX_disc_aux(x)); }
 GEN
 poldisc0(GEN x, long v)
 {
+  long v0, tx = typ(x);
   pari_sp av;
-  switch(typ(x))
+  GEN D;
+  if (tx == t_POL && (v < 0 || v == varn(x))) return RgX_disc(x);
+  switch(tx)
   {
-    case t_POL:
-    {
-      GEN D;
-      long v0 = -1;
-      av = avma;
-      if (v >= 0 && v != varn(x))
-      {
-        v0 = fetch_var_higher();
-        x = fix_pol(x,v, v0);
-      }
-      D = RgX_disc_aux(x);
-      if (v0 >= 0) (void)delete_var();
-      return gerepileupto(av, D);
-    }
-
     case t_COMPLEX:
       return utoineg(4);
-
     case t_QUAD:
       return quad_disc(x);
     case t_POLMOD:
-      return poldisc0(gel(x,1), v);
-
+      if (v >= 0 && varn(gel(x,1)) != v) break;
+      return RgX_disc(gel(x,1));
     case t_QFR: case t_QFI:
       av = avma; return gerepileuptoint(av, qfb_disc(x));
-
     case t_VEC: case t_COL: case t_MAT:
     {
       long i;
@@ -3244,8 +3243,11 @@ poldisc0(GEN x, long v)
       return z;
     }
   }
-  pari_err_TYPE("poldisc",x);
-  return NULL; /* LCOV_EXCL_LINE */
+  if (v < 0) pari_err_TYPE("poldisc",x);
+  av = avma; v0 = fetch_var_higher();
+  x = fix_pol(x,v, v0);
+  D = RgX_disc(x); (void)delete_var();
+  return gerepileupto(av, D);
 }
 
 GEN
