@@ -1227,11 +1227,10 @@ primepi(GEN x)
   GEN pp, nn, N = typ(x) == t_INT? x: gfloor(x);
   forprime_t S;
   ulong n, p;
-  long i, l;
+  long i;
   if (typ(N) != t_INT) pari_err_TYPE("primepi",N);
   if (signe(N) <= 0) return gen_0;
-  avma = av; l = lgefint(N);
-  if (l == 3) return utoi(uprimepi(N[2]));
+  if (lgefint(N) == 3) { n = N[2]; avma = av; return utoi(uprimepi(n)); }
   i = prime_table_len-1;
   p = prime_table[i].p;
   n = prime_table[i].n;
@@ -1464,7 +1463,7 @@ rmprime(GEN T, GEN p)
   if (typ(p) != t_INT) pari_err_TYPE("removeprimes",p);
   i = ZV_search(T, p);
   if (!i)
-    pari_err_DOMAIN("removeprime","prime","not in",
+    pari_err_DOMAIN("removeprimes","prime","not in",
                     strtoGENstr("primetable"), p);
   gunclone(gel(T,i)); gel(T,i) = NULL;
   cleanprimetab(T);
