@@ -1317,6 +1317,7 @@ scpvecs(GEN *pt_vec, long I, GEN b, long dep, struct qfauto *qf)
     }
   }
   settyp(list,t_MAT);
+  settyp(vec,t_MAT);
   *pt_vec = vec;
   return list;
 }
@@ -1367,7 +1368,7 @@ gen_comb(long cdep, GEN A, GEN e, struct qfauto *qf, long lim)
     GEN T = lllgramint(qf_apply_ZM(A,M));
     if (lim && lg(T)-1>=lim) return NULL;
     B = ZM_mul(M,T);
-    BI = RgM_solve(B,NULL);
+    BI = RgM_inv(B);
     sumvecbase = ZM_trunc_to_zm(B);
     trans = ZM_trunc_to_zm(T);
     ccoef = ZM_trunc_to_zm(RgM_mul(BI,M));
@@ -1475,7 +1476,7 @@ qfisom_bestmat(GEN A, long *pt_max)
   max2 = zm_maxdiag(A2);
   if (max2 < max)
   {
-    *pt_max = max2; return mkvec2(ZM_to_zm(U),ZM_to_zm(ZM_inv(U, gen_1)));
+    *pt_max = max2; return mkvec2(ZM_to_zm(U),ZM_to_zm(ZM_inv(U,NULL)));
   } else
   {
     *pt_max = max; return NULL;
@@ -1755,11 +1756,11 @@ qfisominit0(GEN x, GEN flags, GEN minvec)
 }
 
 GEN
-qfisom(GEN F, GEN FF, GEN flags)
+qfisom(GEN F, GEN FF, GEN flags, GEN G)
 {
   pari_sp av = avma;
   struct fingerprint fp;
-  GEN G, res;
+  GEN res;
   struct qfauto qf, qff;
   struct qfcand cand;
   long max;
@@ -1768,14 +1769,22 @@ qfisom(GEN F, GEN FF, GEN flags)
   if (lg(qf.W)!=lg(qff.W)
       || !zvV_equal(vecvecsmall_sort(qf.W), vecvecsmall_sort(qff.W)))
     { avma=av; return gen_0; }
-  G = mkvec(scalar_Flm(-1, qff.dim));
+  if (!G) G = mkvec(scalar_Flm(-1, qff.dim));
   res = isometry(&qf, &qff, &fp, G, &cand);
   if (!res) { avma=av; return gen_0; }
   return gerepilecopy(av, zm_to_ZM(qf.U? zm_mul(res,gel(qf.U, 2)):res));
 }
 
+static GEN
+check_qfauto(GEN G)
+{
+  if (typ(G)==t_VEC && lg(G)==3 && typ(gel(G,1))==t_INT)
+    G = gel(G,2);
+  return qf_to_zmV(G);
+}
+
 GEN
-qfisom0(GEN x, GEN y, GEN flags)
+qfisom0(GEN x, GEN y, GEN flags, GEN G)
 {
   pari_sp av = avma;
   GEN F, FF;
@@ -1788,7 +1797,8 @@ qfisom0(GEN x, GEN y, GEN flags)
   }
   FF = qf_to_zmV(y);
   if (!FF) pari_err_TYPE("qfisom",y);
-  return gerepileupto(av, qfisom(F, FF, flags));
+  if (G) G = check_qfauto(G);
+  return gerepileupto(av, qfisom(F, FF, flags, G));
 }
 
 static GEN
@@ -1854,9 +1864,7 @@ qforbits(GEN G, GEN V)
   GEN gen, w, W, p, v, orb, o;
   long i, j, n, ng;
   long nborbits = 0;
-  if (typ(G)==t_VEC && lg(G)==3 && typ(gel(G,1))==t_INT)
-    G = gel(G,2);
-  gen = qf_to_zmV(G);
+  gen = check_qfauto(G);
   if (!gen) pari_err_TYPE("qforbits", G);
   if (typ(V)==t_VEC && lg(V)==4
    && typ(gel(V,1))==t_INT && typ(gel(V,2))==t_INT)
