@@ -46,7 +46,7 @@ completebasis(GEN Mv, long redflag)
   m = nbrows(Mv); /* m x n */
   if (m == n) return Mv;
   (void)ZM_hnfall_i(shallowtrans(Mv), &U, 0);
-  U = ZM_inv(shallowtrans(U), gen_1);
+  U = ZM_inv(shallowtrans(U),NULL);
   if (m==1 || !redflag) return U;
   /* LLL-reduce the m-n first columns */
   return shallowconcat(ZM_lll(vecslice(U,1,m-n), 0.99, LLL_INPLACE),
@@ -177,8 +177,10 @@ qfbreduce(GEN D, GEN Q)
   {
     GEN r, q, nexta, nextc;
     q = dvmdii(b,a, &r); /* FIXME: export as dvmdiiround ? */
-    if (signe(r) > 0 && abscmpii(shifti(r,1), a) > 0) {
-      r = subii(r, absi(a)); q = addis(q, signe(a));
+    if (signe(r) > 0 && abscmpii(shifti(r,1), a) > 0)
+    {
+      r = subii(r, absi_shallow(a));
+      q = addis(q, signe(a));
     }
     nextc = a; nexta = subii(c, mulii(q, addii(r,b)));
     if (abscmpii(nexta, a) >= 0) break;
@@ -355,7 +357,7 @@ qflllgram_indefgoon2(GEN G)
   a = gcoeff(G2,3,1);
   b = gcoeff(G2,3,2);
   d = bezout(a,b, &u,&v);
-  if (!is_pm1(d))
+  if (!equali1(d))
   {
     a = diviiexact(a,d);
     b = diviiexact(b,d);
@@ -757,7 +759,7 @@ qftriv(GEN G, GEN R, long base)
     }
   if (!R) return G; /* fail */
   /* case 3: a principal minor is 0 */
-  s = keri(principal_minor(G, itos(R)));
+  s = ZM_ker(principal_minor(G, itos(R)));
   s = vecextend(Q_primpart(gel(s,1)), n);
   if (!base) return s;
   H = completebasis(s, 0);
@@ -849,7 +851,7 @@ qfsolve_i(GEN G)
   if (!signe(d))
   {
     if (n == 1) return mkcol(gen_1);
-    sol = keri(G);
+    sol = ZM_ker(G);
     if (lg(sol) == 2) sol = gel(sol,1);
     return sol;
   }
@@ -880,7 +882,7 @@ qfsolve_i(GEN G)
   }
 
   /* factorization of the determinant */
-  fam2detG = Z_factor( absi(d) );
+  fam2detG = absZ_factor(d);
   P = gel(fam2detG,1);
   E = ZV_to_zv(gel(fam2detG,2));
   /* P,E = factor(|det(G)|) */
