@@ -28,19 +28,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 
 #ifdef LONG_IS_64BIT
   typedef ulong u64;
-  static THREAD ulong state[64];
-  #define u64state(i)      ((u64)state[(i)])
-  #define u64stateset(i,x) state[(i)] = (ulong) (x);
 #else
   typedef unsigned long long u64;
-  static THREAD ulong state[128];
-  #define u64state(i)      _32to64(state[2*(i)],state[2*(i)+1])
-  #define u64stateset(i,x) _64to32(x,state+2*(i),state+2*(i)+1)
 static u64
 _32to64(ulong a, ulong b) { u64 v = a; return (v<<32)|b; }
 static void
 _64to32(u64 v, ulong *a, ulong *b) { *a = v>>32; *b = v&0xFFFFFFFF; }
 #endif
+static THREAD u64 state[64];
 static THREAD u64 xorgen_w;
 static THREAD int xorgen_i;
 /* weyl = odd approximation to 2^64*(sqrt(5)-1)/2. */
@@ -53,20 +48,19 @@ block(void)
   const int a = 33, b = 26, c = 27, d = 29, s = 53;
   u64 t, v, w;
   xorgen_i = (xorgen_i+1)&(r-1);
-  t = u64state(xorgen_i);
-  v = u64state((xorgen_i+(r-s))&(r-1));   /* index is (i-s) mod r */
+  t = state[xorgen_i];
+  v = state[(xorgen_i+(r-s))&(r-1)];   /* index is (i-s) mod r */
   t ^= t<<a; t ^= t>>b;                   /* (I + L^a)(I + R^b) */
   v ^= v<<c; v ^= v>>d;                   /* (I + L^c)(I + R^d) */
   w = t^v;
-  u64stateset(xorgen_i, w);               /* update circular array */
-  return w;
+  return state[xorgen_i] = w;               /* update circular array */
 }
 
+/* v > 0 */
 static void
-init_xor4096i(ulong seed)
+init_xor4096i(u64 v)
 {
   const int r = 64;
-  u64 v = seed; /* v must be nonzero */
   int k;
 
   for (k = r; k > 0; k--) {/* avoid correlations for close seeds */
@@ -76,13 +70,13 @@ init_xor4096i(ulong seed)
   for (xorgen_w = v, k = 0; k < r; k++) { /* initialise circular array */
     v ^= v<<10; v ^= v>>15;
     v ^= v<<4;  v ^= v>>13;
-    u64stateset(k, v + (xorgen_w+=weyl));
+    state[k] = v + (xorgen_w+=weyl);
   }
   /* discard first 4*r results */
   for (xorgen_i = r-1, k = 4*r; k > 0; k--) (void)block();
 }
 
-void pari_init_rand(void) { init_xor4096i(1); }
+void pari_init_rand(void) { init_xor4096i(1UL); }
 
 static u64
 rand64(void)
@@ -101,22 +95,32 @@ void
 setrand(GEN x)
 {
   const int r2 = numberof(state);
-  ulong useed;
-  long i;
+  long i, lx;
+  u64 v;
   GEN xp;
-  if (typ(x)!=t_INT)
-    pari_err_TYPE("setrand",x);
-  if (signe(x) <= 0) return;
-  useed = itou_or_0(x);
-  if (useed > 0) { init_xor4096i(useed); return; }
-  if (lgefint(x)!=2+r2+2+(r2==128))
-    pari_err_TYPE("setrand",x);
+  if (typ(x)!=t_INT) pari_err_TYPE("setrand",x);
+  if (signe(x) <= 0) pari_err_DOMAIN("setrand","n", "<=", gen_0, x);
+  lx = lgefint(x);
+  if (lx == 3) { v = x[2]; init_xor4096i(v); return; }
+#ifndef LONG_IS_64BIT
+  if (lx == 4)
+  {
+    v = _32to64(*int_W(x,1),*int_W(x,0));
+    init_xor4096i(v); return;
+  }
+#endif
   xp = int_LSW(x);
-  for (i = 0; i < r2; i++) { state[i] = *xp; xp = int_nextW(xp); }
 #ifdef LONG_IS_64BIT
+  if (lx != 2 + r2+2)
+    pari_err_DOMAIN("setrand", "n", "!=", strtoGENstr("getrand()"), x);
+  for (i = 0; i < r2; i++, xp = int_nextW(xp)) state[i] = *xp;
   xorgen_w = *xp; xp = int_nextW(xp);
 #else
-  xorgen_w = _32to64(*xp, *int_nextW(xp)); xp = int_nextW(int_nextW(xp));
+  if (lx != 2 + 2*r2+3)
+    pari_err_DOMAIN("setrand", "n", "!=", strtoGENstr("getrand()"), x);
+  for (i = 0; i < r2; i++, xp = int_nextW(int_nextW(xp)))
+    state[i] = _32to64(*int_nextW(xp), *xp);
+  xorgen_w = _32to64(*int_nextW(xp), *xp); xp = int_nextW(int_nextW(xp));
 #endif
   xorgen_i =  (*xp) & 63;
 }
@@ -128,14 +132,17 @@ getrand(void)
   GEN x;
   ulong *xp;
   long i;
-  if (xorgen_i < 0) init_xor4096i(1);
+  if (xorgen_i < 0) init_xor4096i(1UL);
 
-  x = cgetipos(2+r2+2+(r2==128)); xp = (ulong *) int_LSW(x);
-  for (i = 0; i < r2; i++) { *xp = state[i]; xp = int_nextW(xp); }
 #ifdef LONG_IS_64BIT
+  x = cgetipos(2+r2+2); xp = (ulong *) int_LSW(x);
+  for (i = 0; i < r2; i++, xp = int_nextW(xp)) *xp = state[i];
   *xp = xorgen_w; xp = int_nextW(xp);
 #else
-  _64to32(xorgen_w, xp, int_nextW(xp)); xp = int_nextW(int_nextW(xp));
+  x = cgetipos(2+2*r2+3); xp = (ulong *) int_LSW(x);
+  for (i = 0; i < r2; i++, xp = int_nextW(int_nextW(xp)))
+    _64to32(state[i], int_nextW(xp), xp);
+  _64to32(xorgen_w, int_nextW(xp), xp); xp = int_nextW(int_nextW(xp));
 #endif
   *xp = xorgen_i? xorgen_i: 64; return x;
 }
@@ -298,11 +305,11 @@ genrand(GEN N)
         if (typ(a) != t_INT || typ(b) != t_INT) pari_err_TYPE("random", N);
         d = subii(b,a);
         if (signe(d) < 0) pari_err_TYPE("random([a,b]) (a > b)", N);
-        return gerepileuptoint(av, addii(a, randomi(addis(d,1))));
+        return gerepileuptoint(av, addii(a, randomi(addiu(d,1))));
       }
       return ellrandom(N);
     default:
       pari_err_TYPE("genrand",N);
-      return NULL;/*not reached*/
+      return NULL;/*LCOV_EXCL_LINE*/
   }
 }
