@@ -2198,7 +2198,7 @@ QX_complex_roots(GEN p, long l)
 /* Count sign changes in the coefficients of (x+1)^deg(P)*P(1/(x+1))
  * The inversion is implicit (we take coefficients backwards). Roots of P
  * at 0 and 1 (mapped to oo and 0) are ignored here and must be dealt with
- * by the caller */
+ * by the caller. Set root1 if P(1) = 0 */
 static long
 X2XP1(GEN P, long deg, int *root1, GEN *Premapped)
 {
@@ -2239,9 +2239,9 @@ X2XP1(GEN P, long deg, int *root1, GEN *Premapped)
     vlim--;
     if (gc_needed(av, 3))
     {
+      if (DEBUGMEM>1) pari_warn(warnmem, "X2XP1, i = %ld/%ld", i, deg-1);
       if (!Premapped) setlg(v, vlim + 2);
-      v = gerepileupto(av, v);
-      if (DEBUGMEM > 1) pari_warn(warnmem, "X2XP1");
+      v = gerepilecopy(av, v);
     }
   }
   if (vlim >= 2 && s == signe(gel(v, vlim))) nb++;
@@ -2662,12 +2662,16 @@ ZX_Uspensky(GEN P, GEN ab, long flag, long bitprec)
   if (typ(a) == t_INFINITY && typ(b) != t_INFINITY && gsigne(b))
   {
     fb = fujiwara_bound_real(Pcur, -1);
-    if (fb > -pariINFINITY) a = negi(int2n((long)ceil(fb))); else a = gen_0;
+    if (fb <= -pariINFINITY) a = gen_0;
+    else if (fb < 0) a = gen_m1;
+    else a = negi(int2n((long)ceil(fb)));
   }
   if (typ(b) == t_INFINITY && typ(a) != t_INFINITY && gsigne(a))
   {
     fb = fujiwara_bound_real(Pcur, 1);
-    if (fb > -pariINFINITY) b = int2n((long)ceil(fb)); else b = gen_0;
+    if (fb <= -pariINFINITY) b = gen_0;
+    else if (fb < 0) b = gen_1;
+    else b = int2n((long)ceil(fb));
   }
 
   if (typ(a) != t_INFINITY && typ(b) != t_INFINITY)
