@@ -16,6 +16,11 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 /**                        PARI CALCULATOR                        **/
 /**                                                               **/
 /*******************************************************************/
+#ifdef _WIN32
+#  include "../systems/mingw/pwinver.h"
+#  include <windows.h>
+#  include "../systems/mingw/mingw.h"
+#endif
 #include "pari.h"
 #include "paripriv.h"
 #include "gp.h"
@@ -106,12 +111,13 @@ gp_head(void)
 {
   pari_print_version();
   pari_putc('\n');
-  pari_center("Copyright (C) 2000-2017 The PARI Group");
+  pari_center("Copyright (C) 2000-2018 The PARI Group");
   pari_putc('\n');
   print_text("PARI/GP is free software, covered by the GNU General Public \
 License, and comes WITHOUT ANY WARRANTY WHATSOEVER.");
   pari_puts("\nType ? for help, \\q to quit.\n");
-  print_text("Type ?15 for how to get moral (and possibly technical) support.");
+  pari_printf("Type ?%d for how to get moral"
+              " (and possibly technical) support.\n", pari_community());
   if (pari_mainstack->vsize)
     pari_printf("\nparisizemax = %lu, primelimit = %lu",
                 pari_mainstack->vsize,GP_DATA->primelimit);
@@ -264,13 +270,7 @@ static int
 test_is_interactive(void) { return 0; }
 
 static void
-test_output(GEN z)
-{
-  init_linewrap(76);
-  gen_output(z, GP_DATA->fmt);
-  pari_putc('\n');
-  pari_flush();
-}
+test_output(GEN z) { init_linewrap(76); gen_output(z); }
 void
 init_test(void)
 {
@@ -343,7 +343,7 @@ gp_main_loop(long ismain)
       /* true error not from main instance, let caller sort it out */
       if (!ismain) { kill_buffers_upto_including(b); return NULL; }
     } else { /* allocatemem */
-      filestate_restore(rec.file);
+      tmp_restore(rec.file.file);
       gp_context_save(&rec);
     }
     avma = av = pari_mainstack->top;
@@ -360,7 +360,6 @@ gp_main_loop(long ismain)
       if (ismain) continue;
       pop_buffer(); return z;
     }
-    avma = av;
     if (ismain)
     {
       reset_ctrlc();
@@ -371,11 +370,10 @@ gp_main_loop(long ismain)
     z = pari_compile_str(b->buf);
     z = closure_evalres(z);
     if (!ismain) continue;
-    pari_alarm(0);
-
-    if (!pari_last_was_newline()) pari_putc('\n');
 
     t = timer_delay(GP_DATA->T);
+    if (!pari_last_was_newline()) pari_putc('\n');
+    pari_alarm(0);
     if (t && GP_DATA->chrono)
     {
       pari_puts("time = ");
@@ -384,6 +382,7 @@ gp_main_loop(long ismain)
     if (GP_DATA->simplify) z = simplify_shallow(z);
     pari_add_hist(z, t);
     if (z != gnil && ! is_silent(b->buf) ) gp_output(z);
+    avma = av;
     parivstack_reset();
   }
 }
@@ -488,13 +487,7 @@ break_loop(int numerr)
         killallfiles(); /* completely exit the debugger */
         go_on = 0; goto BR_EXIT;
     }
-
-    if (x != gnil && !is_silent(b->buf))
-    {
-      term_color(c_OUTPUT);
-      gen_output(x, GP_DATA->fmt);
-      pari_putc('\n');
-    }
+    if (x!=gnil && !is_silent(b->buf)) { term_color(c_OUTPUT); gen_output(x); }
   }
 BR_EXIT:
   s_env.n=nenv;
@@ -574,18 +567,18 @@ main(int argc, char **argv)
   (void)os_signal(SIGALRM,gp_alarm_handler);
 #endif
   pari_add_module(functions_gp);
-  pari_add_module(functions_highlevel);
 
-  init_graph();
+  pari_set_plot_engine(gp_get_plot);
   cb_pari_quit = gp_quit;
   cb_pari_whatnow = whatnow;
   cb_pari_sigint = gp_sigint_fun;
   cb_pari_handle_exception = gp_handle_exception;
   cb_pari_ask_confirm = gp_ask_confirm;
-  gp_expand_path(GP_DATA->path);
-  gp_expand_path(GP_DATA->sopath);
+  pari_init_paths();
   pari_mt_init(); /* MPI: will not return on slaves (pari_MPI_rank = 0) */
-
+#ifdef _WIN32
+  if (stdin_isatty) win32_set_codepage();
+#endif
 #ifdef READLINE
   init_readline();
 #endif
@@ -607,7 +600,8 @@ main(int argc, char **argv)
   }
   pari_stack_delete(&s_A);
   (void)gp_main_loop(1);
-  gp_quit(0); return 0; /* not reached */
+  gp_quit(0);
+  return 0; /* LCOV_EXCL_LINE */
 }
 
 void
@@ -643,7 +637,7 @@ dbg_up(long k)
 void
 gp_quit(long code)
 {
-  free_graph();
+  pari_kill_plot_engine();
   pari_close();
   kill_buffers_upto(NULL);
   if (!(GP_DATA->flags & gpd_QUIET)) pari_puts("Goodbye!\n");
@@ -655,5 +649,3 @@ void
 whatnow0(char *s) { whatnow(pariOut, s,0); }
 
 #include "gp_init.h"
-#include "../graph/rect.h"
-#include "highlvl.h"
