@@ -14,17 +14,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include "pari.h"
 #include "paripriv.h"
 #include "gp.h"
-
-typedef struct whatnow_t
-{
-  const char *name, *oldarg, *newarg;
-} whatnow_t;
-
-#define SAME NULL
-#define REMOV (char *)1L
-#define _REMOV {REMOV,NULL,NULL}
-#define _SAME  {SAME,NULL,NULL}
-
 #include "whatnow.h"
 
 static void
@@ -42,35 +31,21 @@ msg(PariOUT *out, const char *s)
 int
 whatnow(PariOUT *out, const char *s, int flag)
 {
-  int n;
   const char *def;
-  whatnow_t wp;
+  const whatnow_t *wp = whatnowlist;
   entree *ep;
 
-  if (flag && s[0] && !s[1]) return 0; /* special case "i" and "o" */
-  n = 0;
-  do
-    def = (oldfonctions[n++]).name;
-  while (def && strcmp(def,s));
-  if (!def)
-  {
-    int m = 0;
-    do
-      def = (functions_oldgp[m++]).name;
-    while (def && strcmp(def,s));
-    n += m - 1;
-  }
+  while (wp->old && strcmp(wp->old,s)) wp++;
   /* Above linear search is slow, esp. if the symbol is not found. BUT no
    * point in wasting time by preallocating [ or autoloading ] a hashtable:
    * whatnow() is never used in a case where speed would be necessary */
-  if (!def)
+  if (!wp->old)
   {
     if (!flag)
       msg(out, "As far as I can recall, this function never existed");
     return 0;
   }
-
-  wp = whatnowlist[n-1]; def = wp.name;
+  def = wp->name;
   if (def == SAME)
   {
     if (!flag)
@@ -80,23 +55,22 @@ whatnow(PariOUT *out, const char *s, int flag)
   if (flag)
   {
     out_term_color(out, c_NONE);
-    out_print_text(out, "\nA function with that name existed in GP-1.39.15; to run in backward compatibility mode, type \"default(compatible,3)\", or set \"compatible = 3\" in your GPRC file.");
+    out_print_text(out, "\nA function with that name existed in GP-1.39.15. Please update your script.");
     out_putc(out, '\n');
   }
 
   if (def == REMOV)
   {
-    msg(out, "This function was suppressed");
+    msg(out, "This function no longer exists");
     return 0;
   }
-  if (!strcmp(def,"x*y")) ep = NULL;
-  else {
-    ep = is_entry(wp.name);
-    if (!ep) pari_err_BUG("whatnow");
-  }
+  /* special case compimag -> x*y */
+  if (!strcmp(def,"x*y")) def = "_*_";
+  ep = is_entry(def);
+  if (!ep) pari_err_BUG("whatnow");
   out_puts(out, "New syntax: ");
   out_term_color(out, c_ERR);
-  out_printf(out, "%s%s ===> %s%s\n\n", s, wp.oldarg, wp.name, wp.newarg);
-  if (ep) msg(out, ep->help);
+  out_printf(out, "%s%s ===> %s%s\n\n", s, wp->oldarg, wp->name, wp->newarg);
+  msg(out, ep->help);
   out_term_color(out, c_NONE); return 1;
 }
