@@ -525,7 +525,7 @@ static const whatnow_t whatnowlist[]={
 {"rnfpseudobasis",_SAME},
 {"rnfsteinitz",_SAME},
 {"rootmod","polrootsmod","(x,p)","(x,p)"},
-{"rootmod2","polrootsmod","(x,p)","(x,p,1)"},
+{"rootmod2","polrootsmod","(x,p)","(x,p)"},
 {"rootpadic","polrootspadic","(x,p,r)","(x,p,r)"},
 {"roots","polroots","(x)","(x)"},
 {"rootsof1","nfrootsof1","(nf)","(nf)"},
@@ -627,9 +627,9 @@ static const whatnow_t whatnowlist[]={
 {"cursor","plotcursor","(w)","(w)"},
 {"default",_SAME},
 {"draw","plotdraw","(list)","(list)"},
-{"initrect","plotinit","(w,x,y)","(w,x,y)"},
+{"plotinit","plotinit","(w,x,y)","(w,x,y)"},
 {"kill",_SAME},
-{"killrect","plotkill","(w)","(w)"},
+{"plotkill","plotkill","(w)","(w)"},
 {"line","plotlines","(w,x2,y2)","(w,x2,y2)"},
 {"lines","plotlines","(w,x2,y2)","(w,x2,y2)"},
 {"move","plotmove","(w,x,y)","(w,x,y)"},
@@ -640,10 +640,10 @@ static const whatnow_t whatnowlist[]={
 {"plothraw",_SAME},
 {"point","plotpoints","(w,x,y)","(w,x,y)"},
 {"points","plotpoints","(w,x,y)","(w,x,y)"},
-{"postdraw","psdraw","(list)","(list)"},
-{"postploth","psploth","(X=a,b,expr)","(X=a,b,expr)"},
+{"psdraw","psdraw","(list)","(list)"},
+{"psploth","psploth","(X=a,b,expr)","(X=a,b,expr)"},
 {"postploth2","psploth","(X=a,b,expr)","(X=a,b,expr,1)"},
-{"postplothraw","psplothraw","(listx,listy)","(listx,listy)"},
+{"psplothraw","psplothraw","(listx,listy)","(listx,listy)"},
 {"pprint",_REMOV},
 {"pprint1",_REMOV},
 {"print",_SAME},
