@@ -107,7 +107,7 @@ new_chunk(size_t x) /* x is a number of longs */
     new_chunk_resize(x);
   avma = (pari_sp)z;
 #ifdef MEMSTEP
-  if (DEBUGMEM && pari_mainstack->memused != DISABLE_MEMUSED) {
+  if (DEBUGMEM>1 && pari_mainstack->memused != DISABLE_MEMUSED) {
     long d = (long)pari_mainstack->memused - (long)z;
     if (labs(d) > 4*MEMSTEP)
     {
@@ -127,6 +127,14 @@ stack_malloc(size_t N)
   return (char*)new_chunk(n);
 }
 
+INLINE char *
+stack_malloc_align(size_t N, long k)
+{
+  ulong d = ((ulong)avma) % k;
+  if (d) (void)new_chunk(d/sizeof(long));
+  return (char*) new_chunk(nchar2nlong(N));
+}
+
 INLINE char *
 stack_calloc(size_t N)
 {
@@ -239,6 +247,8 @@ INLINE GEN
 mpneg(GEN x) { GEN y = leafcopy(x); togglesign(y); return y; }
 INLINE GEN negi(GEN x) { return mpneg(x); }
 INLINE GEN negr(GEN x) { return mpneg(x); }
+INLINE GEN gmax_shallow(GEN x, GEN y) { return gcmp(x,y)<0? y: x; }
+INLINE GEN gmin_shallow(GEN x, GEN y) { return gcmp(x,y)<0? x: y; }
 
 /* negate in place */
 INLINE void
@@ -338,7 +348,7 @@ itou(GEN x)
     case 3: return x[2];
     default:
       pari_err_OVERFLOW("t_INT-->ulong assignment");
-      return 0; /* not reached */
+      return 0; /* LCOV_EXCL_LINE */
   }
 }
 
@@ -349,6 +359,24 @@ itou_or_0(GEN x) {
   return (ulong)x[2];
 }
 
+INLINE ulong
+umuluu_or_0(ulong x, ulong y)
+{
+  ulong z;
+  LOCAL_HIREMAINDER;
+  z = mulll(x, y);
+  return hiremainder? 0: z;
+}
+/* return x*y if <= n, else 0. Beware overflow */
+INLINE ulong
+umuluu_le(ulong x, ulong y, ulong n)
+{
+  ulong z;
+  LOCAL_HIREMAINDER;
+  z = mulll(x, y);
+  return (hiremainder || z > n)? 0: z;
+}
+
 INLINE GEN
 real_0_bit(long bitprec) { GEN x=cgetr(2); x[1]=evalexpo(bitprec); return x; }
 INLINE GEN
@@ -426,6 +454,19 @@ cmpsr(long x, GEN y)
 }
 INLINE int
 cmprs(GEN x, long y) { return -cmpsr(y,x); }
+/* compare x and y */
+INLINE int
+cmpui(ulong x, GEN y)
+{
+  ulong p;
+  if (!x) return -signe(y);
+  if (signe(y) <= 0) return 1;
+  if (lgefint(y) > 3) return -1;
+  p = y[2]; if (p == x) return 0;
+  return p < x ? 1 : -1;
+}
+INLINE int
+cmpiu(GEN x, ulong y) { return -cmpui(y,x); }
 /* compare x and |y| */
 INLINE int
 abscmpui(ulong x, GEN y)
@@ -471,6 +512,14 @@ mpcmp(GEN x, GEN y)
   return (typ(y)==t_INT) ? -cmpir(y,x) : cmprr(x,y);
 }
 
+/* x == y ? */
+INLINE int
+equalui(ulong x, GEN y)
+{
+  if (!x) return !signe(y);
+  if (signe(y) <= 0 || lgefint(y) != 3) return 0;
+  return ((ulong)y[2] == (ulong)x);
+}
 /* x == y ? */
 INLINE int
 equalsi(long x, GEN y)
@@ -495,6 +544,8 @@ INLINE int
 absequaliu(GEN x, ulong y) { return absequalui(y,x); }
 INLINE int
 equalis(GEN x, long y) { return equalsi(y,x); }
+INLINE int
+equaliu(GEN x, ulong y) { return equalui(y,x); }
 
 /* assume x != 0, is |x| == 2^n ? */
 INLINE int
@@ -671,14 +722,13 @@ ceildivuu(ulong a, ulong b)
   return (a%b)? c+1: c;
 }
 
-
 INLINE ulong
-udivui_rem(ulong x, GEN y, ulong *r)
+uabsdivui_rem(ulong x, GEN y, ulong *r)
 {
   long q, s = signe(y);
   LOCAL_HIREMAINDER;
 
-  if (!s) pari_err_INV("udivui_rem",gen_0);
+  if (!s) pari_err_INV("uabsdivui_rem",gen_0);
   if (!x || lgefint(y)>3) { *r = x; return 0; }
   hiremainder=0; q = (long)divll(x, (ulong)y[2]);
   if (s < 0) q = -q;
@@ -688,7 +738,7 @@ udivui_rem(ulong x, GEN y, ulong *r)
 /* assume d != 0 and |n| / d can be represented as an ulong.
  * Return |n|/d, set *r = |n| % d */
 INLINE ulong
-udiviu_rem(GEN n, ulong d, ulong *r)
+uabsdiviu_rem(GEN n, ulong d, ulong *r)
 {
   switch(lgefint(n))
   {
@@ -790,6 +840,11 @@ umodui(ulong x, GEN y)
   return x % (ulong)y[2];
 }
 
+INLINE ulong
+ugcdiu(GEN x, ulong y) { return ugcd(umodiu(x,y), y); }
+INLINE ulong
+ugcdui(ulong y, GEN x) { return ugcd(umodiu(x,y), y); }
+
 INLINE GEN
 remsi(long x, GEN y)
 { long r; (void)sdivsi_rem(x,y, &r); return stoi(r); }
@@ -899,7 +954,7 @@ dvdiuz(GEN x, ulong y, GEN z)
 {
   const pari_sp av = avma;
   ulong r;
-  GEN p1 = diviu_rem(x,y, &r);
+  GEN p1 = absdiviu_rem(x,y, &r);
   avma = av; if (r) return 0;
   affii(p1,z); return 1;
 }
@@ -940,7 +995,7 @@ Fl_mul_pre(ulong a, ulong b, ulong p, ulong pi)
 }
 
 INLINE ulong
-Fl_addmul_pre(ulong x0, ulong x1, ulong y0, ulong p, ulong pi)
+Fl_addmul_pre(ulong y0, ulong x0, ulong x1, ulong p, ulong pi)
 {
   ulong l0, h0;
   LOCAL_HIREMAINDER;
@@ -1324,11 +1379,11 @@ INLINE void divssz(long x, long y, GEN z)
 { affsi(x/y, z); }
 
 INLINE void modisz(GEN y, long s, GEN z)
-{ pari_sp av = avma; affii(modis(y,s),z); avma = av; }
+{ affsi(smodis(y,s),z); }
 INLINE void modsiz(long s, GEN y, GEN z)
 { pari_sp av = avma; affii(modsi(s,y),z); avma = av; }
 INLINE void modssz(long s, long y, GEN z)
-{ pari_sp av = avma; affii(modss(s,y),z); avma = av; }
+{ affsi(smodss(s,y),z); }
 
 INLINE void mpaddz(GEN x, GEN y, GEN z)
 { pari_sp av = avma; mpaff(mpadd(x,y),z); avma = av; }
