@@ -372,7 +372,7 @@ truncr(GEN x)
   GEN y;
 
   if ((s=signe(x)) == 0 || (e=expo(x)) < 0) return gen_0;
-  d = nbits2prec(e+1); m = remsBIL(e);
+  d = nbits2lg(e+1); m = remsBIL(e);
   if (d > lg(x)) pari_err_PREC( "truncr (precision loss in truncation)");
 
   y=cgeti(d); y[1] = evalsigne(s) | evallgefint(d);
@@ -392,7 +392,7 @@ floorr(GEN x)
 
   if (signe(x) >= 0) return truncr(x);
   if ((e=expo(x)) < 0) return gen_m1;
-  d = nbits2prec(e+1); m = remsBIL(e);
+  d = nbits2lg(e+1); m = remsBIL(e);
   lx=lg(x); if (d>lx) pari_err_PREC( "floorr (precision loss in truncation)");
   y = new_chunk(d);
   if (++m == BITS_IN_LONG)
@@ -524,14 +524,14 @@ umodiu(GEN y, ulong x)
 
 /* return |y| \/ x */
 GEN
-diviu_rem(GEN y, ulong x, ulong *rem)
+absdiviu_rem(GEN y, ulong x, ulong *rem)
 {
   long ly,i;
   GEN z;
   ulong xi;
   LOCAL_HIREMAINDER;
 
-  if (!x) pari_err_INV("diviu_rem",gen_0);
+  if (!x) pari_err_INV("absdiviu_rem",gen_0);
   if (!signe(y)) { *rem = 0; return gen_0; }
 
   ly = lgefint(y);
@@ -797,7 +797,7 @@ dvmdii(GEN x, GEN y, GEN *z)
       if (!rem) return gen_0;
       return (sx < 0)? utoineg(uel(y,2) - rem): utoipos(rem);
     }
-    q = diviu_rem(x, uel(y,2), &rem);
+    q = absdiviu_rem(x, uel(y,2), &rem);
     if (sx != sy) togglesign(q);
     if (!z) return q;
     if (!rem) *z = gen_0;
@@ -1072,7 +1072,12 @@ diviuexact_i(GEN x, ulong y)
 
   if (y == 1) return icopy(x);
   lx = lgefint(x);
-  if (lx == 3) return utoipos(uel(x,2) / y);
+  if (lx == 3)
+  {
+    q = uel(x,2) / y;
+    if (!q) pari_err_OP("exact division", x, utoi(y));
+    return utoipos(q);
+  }
   yinv = invmod2BIL(y);
   lz = (y <= uel(x,2)) ? lx : lx-1;
   z = new_chunk(lz);
@@ -1104,6 +1109,7 @@ diviuexact_i(GEN x, ulong y)
   z += i-2; lz -= i-2;
   z[0] = evaltyp(t_INT)|evallg(lz);
   z[1] = evalsigne(1)|evallg(lz);
+  if (lz == 2) pari_err_OP("exact division", x, utoi(y));
   avma = (pari_sp)z; return z;
 }
 
@@ -1120,6 +1126,7 @@ diviuexact(GEN x, ulong y)
   lx = lgefint(x);
   if (lx == 3) {
     ulong q = uel(x,2) / y;
+    if (!q) pari_err_OP("exact division", x, utoi(y));
     return (s > 0)? utoipos(q): utoineg(q);
   }
   av = avma; (void)new_chunk(lx); vy = vals(y);
@@ -1130,6 +1137,7 @@ diviuexact(GEN x, ulong y)
     if (lx == 3) {
       ulong q = uel(x,2) / y;
       avma = av;
+      if (!q) pari_err_OP("exact division", x, utoi(y));
       return (s > 0)? utoipos(q): utoineg(q);
     }
   } else x = icopy(x);
@@ -1154,6 +1162,7 @@ diviiexact(GEN x, GEN y)
   lx = lgefint(x);
   if (lx == 3) {
     q = uel(x,2) / uel(y,2);
+    if (!q) pari_err_OP("exact division", x, y);
     return (sx+sy) ? utoipos(q): utoineg(q);
   }
   vy = vali(y); av = avma;
@@ -1169,8 +1178,8 @@ diviiexact(GEN x, GEN y)
   ly = lgefint(y);
   if (ly == 3)
   {
-    x = diviuexact_i(x,uel(y,2)); /* x != 0 */
-    setsigne(x, (sx+sy)? 1: -1); return x;
+    z = diviuexact_i(x,uel(y,2)); /* x != 0 */
+    setsigne(z, (sx+sy)? 1: -1); return z;
   }
   y0inv = invmod2BIL(y[ly-1]);
   i=2; while (i<ly && y[i]==x[i]) i++;
@@ -1212,6 +1221,7 @@ diviiexact(GEN x, GEN y)
   z += i-2; lz -= (i-2);
   z[0] = evaltyp(t_INT)|evallg(lz);
   z[1] = evalsigne((sx+sy)? 1: -1) | evallg(lz);
+  if (lz == 2) pari_err_OP("exact division", x, y);
   avma = (pari_sp)z; return z;
 }
 
@@ -1387,7 +1397,7 @@ Zf_add(GEN a, GEN b, GEN M)
   long mod = lgefint(M)-3;
   long l = NLIMBS(z);
   if (l<=mod) return z;
-  y = subis(z, 1);
+  y = subiu(z, 1);
   if (NLIMBS(y)<=mod) return z;
   return int_normalize(y,1);
 }
