@@ -32,6 +32,10 @@ long Flx_SQR_HALFSQRI_LIMIT         = __Flx_SQR_HALFSQRI_LIMIT;
 long Flx_SQR_KARATSUBA_LIMIT        = __Flx_SQR_KARATSUBA_LIMIT;
 long Flx_SQR_SQRI2_LIMIT            = __Flx_SQR_SQRI2_LIMIT;
 long Flx_SQR_SQRI_LIMIT             = __Flx_SQR_SQRI_LIMIT;
+long F2xqX_BARRETT_LIMIT            = __F2xqX_BARRETT_LIMIT;
+long F2xqX_DIVREM_BARRETT_LIMIT     = __F2xqX_DIVREM_BARRETT_LIMIT;
+long F2xqX_INVBARRETT_LIMIT         = __F2xqX_INVBARRETT_LIMIT;
+long F2xqX_REM_BARRETT_LIMIT        = __F2xqX_REM_BARRETT_LIMIT;
 long FlxqX_BARRETT_LIMIT            = __FlxqX_BARRETT_LIMIT;
 long FlxqX_DIVREM_BARRETT_LIMIT     = __FlxqX_DIVREM_BARRETT_LIMIT;
 long FlxqX_EXTGCD_LIMIT             = __FlxqX_EXTGCD_LIMIT;
@@ -66,4 +70,5 @@ long RgX_MUL_LIMIT                  = __RgX_MUL_LIMIT;
 long RgX_SQR_LIMIT                  = __RgX_SQR_LIMIT;
 long SQRI_FFT_LIMIT                 = __SQRI_FFT_LIMIT;
 long SQRI_KARATSUBA_LIMIT           = __SQRI_KARATSUBA_LIMIT;
+long SQRR_SQRI_LIMIT                = __SQRR_SQRI_LIMIT;
 #endif
