@@ -44,15 +44,6 @@ get_sep(const char *t)
   }
 }
 
-static ulong
-safe_mul(ulong x, ulong y)
-{
-  ulong z;
-  LOCAL_HIREMAINDER;
-  z = mulll(x, y);
-  return hiremainder? 0: z;
-}
-
 /* "atoul" + optional [kmg] suffix */
 static ulong
 my_int(char *s)
@@ -71,11 +62,11 @@ my_int(char *s)
   {
     switch(*p)
     {
-      case 'k': case 'K': n = safe_mul(n,1000UL);       p++; break;
-      case 'm': case 'M': n = safe_mul(n,1000000UL);    p++; break;
-      case 'g': case 'G': n = safe_mul(n,1000000000UL); p++; break;
+      case 'k': case 'K': n = umuluu_or_0(n,1000UL);       p++; break;
+      case 'm': case 'M': n = umuluu_or_0(n,1000000UL);    p++; break;
+      case 'g': case 'G': n = umuluu_or_0(n,1000000000UL); p++; break;
 #ifdef LONG_IS_64BIT
-      case 't': case 'T': n = safe_mul(n,1000000000000UL); p++; break;
+      case 't': case 'T': n = umuluu_or_0(n,1000000000000UL); p++; break;
 #endif
     }
     if (!n) pari_err(e_SYNTAX,"integer too large",s,s);
@@ -127,7 +118,7 @@ pari_default_path(void) {
 #endif
 }
 
-void
+static void
 delete_dirs(gp_path *p)
 {
   char **v = p->dirs, **dirs;
@@ -139,8 +130,8 @@ delete_dirs(gp_path *p)
   }
 }
 
-void
-gp_expand_path(gp_path *p)
+static void
+expand_path(gp_path *p)
 {
   char **dirs, *s, *v = p->PATH;
   int i, n = 0;
@@ -174,6 +165,21 @@ gp_expand_path(gp_path *p)
   }
   dirs[i] = NULL; p->dirs = dirs;
 }
+void
+pari_init_paths(void)
+{
+  expand_path(GP_DATA->path);
+  expand_path(GP_DATA->sopath);
+}
+
+static void
+delete_path(gp_path *p) { delete_dirs(p); free(p->PATH); }
+void
+pari_close_paths(void)
+{
+  delete_path(GP_DATA->path);
+  delete_path(GP_DATA->sopath);
+}
 
 /********************************************************************/
 /*                                                                  */
@@ -265,6 +271,46 @@ sd_ulong(const char *v, long flag, const char *s, ulong *ptn, ulong Min, ulong M
   return gnil;
 }
 
+static void
+err_intarray(char *t, char *p, const char *s)
+{
+  char *b = stack_malloc(64 + strlen(s));
+  sprintf(b, "incorrect value for %s", s);
+  pari_err(e_SYNTAX, b, p, t);
+}
+static GEN
+parse_intarray(const char *v, const char *s)
+{
+  char *p, *t = gp_filter(v);
+  long i, l;
+  GEN w;
+  if (*t != '[') err_intarray(t, t, s);
+  if (t[1] == ']') { pari_free(t); return cgetalloc(t_VECSMALL, 1); }
+  for (p = t+1, l=2; *p; p++)
+    if (*p == ',') l++;
+    else if (*p < '0' || *p > '9') break;
+  if (*p != ']') err_intarray(t, p, s);
+  w = cgetalloc(t_VECSMALL, l);
+  for (p = t+1, i=0; *p; p++)
+  {
+    long n = 0;
+    while (*p >= '0' && *p <= '9') n = 10*n + (*p++ -'0');
+    w[++i] = n;
+  }
+  pari_free(t); return w;
+}
+GEN
+sd_intarray(const char *v, long flag, GEN *pz, const char *s)
+{
+  if (v) { GEN z = *pz; *pz = parse_intarray(v, s); pari_free(z); }
+  switch(flag)
+  {
+    case d_RETURN: return zv_to_ZV(*pz);
+    case d_ACKNOWLEDGE: pari_printf("   %s = %Ps\n", s, zv_to_ZV(*pz));
+  }
+  return gnil;
+}
+
 GEN
 sd_realprecision(const char *v, long flag)
 {
@@ -753,7 +799,7 @@ sd_PATH(const char *v, long flag, const char* s, gp_path *p)
     pari_free((void*)p->PATH);
     p->PATH = pari_strdup(v);
     if (flag == d_INITRC) return gnil;
-    gp_expand_path(p);
+    expand_path(p);
   }
   if (flag == d_RETURN) return strtoGENstr(p->PATH);
   if (flag == d_ACKNOWLEDGE)
@@ -845,7 +891,7 @@ setdefault(const char *s, const char *v, long flag)
   if (!ep)
   {
     pari_err(e_MISC,"unknown default: %s",s);
-    return NULL; /* not reached */
+    return NULL; /* LCOV_EXCL_LINE */
   }
   return call_f2(ep, v, flag);
 }
@@ -891,9 +937,7 @@ static char *
 init_help(void)
 {
   char *h = os_getenv("GPHELP");
-# ifdef GPHELP
-  if (!h) h = (char*)GPHELP;
-# endif
+  if (!h) h = (char*)paricfg_gphelp;
 #ifdef _WIN32
   win32_set_pdf_viewer();
 #endif
@@ -961,6 +1005,7 @@ default_gp_data(void)
   init_path(D->sopath, "");
   init_pp(D);
   init_graphs(D);
+  D->plothsizes = cgetalloc(t_VECSMALL, 1);
   D->prompt_comment = (char*)"comment> ";
   D->prompt = pari_strdup("? ");
   D->prompt_cont = pari_strdup("");
