@@ -1809,6 +1809,7 @@ escape(const char *tch, int ismain)
       switchin(s);
       if (file_is_binary(pari_infile))
       {
+        pari_sp av = avma;
         int vector;
         GEN x = readbin(s,pari_infile, &vector);
         popinfile();
@@ -1819,6 +1820,7 @@ escape(const char *tch, int ismain)
           pari_warn(warner,"setting %ld history entries", l-1);
           for (i=1; i<l; i++) pari_add_hist(gel(x,i), 0);
         }
+        avma = av;
       }
       break;
     case 's': dbg_pari_heap(); break;
