@@ -85,6 +85,7 @@ member_bid(GEN x)
   long t; (void)get_nf(x,&t);
   switch(t) {
     case typ_BNR: return bnr_get_bid(x);
+    case typ_BIDZ:
     case typ_BID: return x;
   }
   member_err("bid",x);
@@ -191,14 +192,22 @@ member_mod(GEN x) /* modulus */
   switch(t) {
     case typ_GAL: return gal_get_mod(x);
     case typ_BNR: return bnr_get_mod(x);
-    case typ_BID: return typ(gel(x,3)) == t_MAT? bid_get_mod(x)
-                                               : bid_get_ideal(x);
+    case typ_BIDZ: return bid_get_ideal(x);
+    case typ_BID: return bid_get_mod(x);
   }
   switch(typ(x))
   {
     case t_INTMOD: case t_POLMOD: case t_QUAD: break;
     case t_PADIC: return gel(x,3);
     case t_FFELT: return FF_mod(x);
+    case t_VEC:
+      if (checkmf_i(x))
+      {
+        GEN T = mf_get_field(x), CHI = mf_get_CHI(x), P = mfcharpol(CHI);
+        return (degpol(T) == 1)? P: (degpol(P) > 1? gmodulo(T, P): T);
+      }
+      else if (checkMF_i(x))
+        return mfcharpol(MF_get_CHI(x));
     default: member_err("mod",x);
   }
   return gel(x,1);
@@ -258,12 +267,12 @@ member_diff(GEN x) /* different */
 GEN
 member_codiff(GEN x) /* codifferent */
 {
-  long t; GEN T, D, DinvT, nf = get_nf(x,&t), y = nfmats(nf);
+  long t;
+  GEN T, d, Di, nf = get_nf(x,&t), y = nfmats(nf);
   if (!y) member_err("codiff",x);
   T = gel(y,4);
-  D = absi(nf_get_disc(nf));
-  DinvT = ZM_inv(T,D);
-  return gdiv(ZM_hnfmodid(DinvT, D), D);
+  Di = ZM_inv(T, &d); if (!d) return matid(lg(Di)-1);
+  return RgM_Rg_div(ZM_hnfmodid(Di, d), d);
 }
 
 GEN
@@ -303,6 +312,7 @@ _member_clgp(GEN x, GEN y, long t) /* class group (3-component row vector) */
     switch(t)
     {
       case typ_QUA: return mkvec3(gel(x,1), gel(x,2), gel(x,3));
+      case typ_BIDZ:
       case typ_BID: return gel(x,2);
     }
     if (typ(x)==t_VEC)
@@ -356,7 +366,7 @@ member_fu(GEN x) /* fundamental units */
 }
 
 /* torsion units. return [w,e] where w is the number of roots of 1, and e a
- * polymod generator */
+ * polmod (or integer) generator */
 GEN
 member_tu(GEN x)
 {
@@ -373,9 +383,10 @@ member_tu(GEN x)
   }
   else
   {
+    GEN z = bnf_get_tuU(bnf);
     if (t == typ_BNR) pari_err_IMPL("ray torsion units");
     gel(res,1) = utoipos( bnf_get_tuN(bnf) );
-    gel(res,2) = basistoalg(bnf, bnf_get_tuU(bnf));
+    gel(res,2) = typ(z)==t_INT? gen_m1: basistoalg(bnf,z);
   }
   return res;
 }
@@ -400,6 +411,7 @@ member_zkst(GEN x)
   long t; (void)get_nf(x,&t);
   switch(t)
   {
+    case typ_BIDZ:
     case typ_BID: return bid_get_grp(x);
     case typ_BNR: {
       GEN bid = bnr_get_bid(x);
@@ -407,7 +419,7 @@ member_zkst(GEN x)
     }
   }
   member_err("zkst",x);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 GEN
@@ -464,7 +476,7 @@ member_group(GEN x)
   if (t == typ_GAL) return gal_get_group(x);
   if (t == typ_ELL) return ellgroup0(x, NULL, 1);
   member_err("group",x);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 GEN
 member_orders(GEN x)
@@ -472,7 +484,7 @@ member_orders(GEN x)
   long t; (void)get_nf(x,&t);
   if (t == typ_GAL) return gal_get_orders(x);
   member_err("orders",x);
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 GEN
@@ -563,10 +575,15 @@ static int
 ell_is_complex(GEN x)
 { long t = ell_get_type(x); return t == t_ELL_Q || t == t_ELL_Rg; }
 
+static long
+ellnf_get_prec(GEN x) { return nf_get_prec(ellnf_get_nf(x)); }
+
 GEN
 member_omega(GEN x)
 {
   if (!is_ell(x)) member_err("omega",x);
+  if (ell_get_type(x)==t_ELL_NF)
+    return ellnf_vecomega(x, ellnf_get_prec(x));
   if (!ell_is_complex(x)) pari_err_TYPE("omega [not defined over C]",x);
   return ellR_omega(x, ellR_get_prec(x));
 }
@@ -575,6 +592,8 @@ GEN
 member_eta(GEN x)
 {
   if (!is_ell(x)) member_err("eta",x);
+  if (ell_get_type(x)==t_ELL_NF)
+    return ellnf_veceta(x, ellnf_get_prec(x));
   if (!ell_is_complex(x)) pari_err_TYPE("eta [not defined over C]",x);
   return ellR_eta(x, ellR_get_prec(x));
 }
@@ -582,15 +601,11 @@ member_eta(GEN x)
 GEN
 member_area(GEN x)
 {
-  GEN w, w1, w2, a,b,c,d;
-  long prec;
   if (!is_ell(x)) member_err("area",x);
+  if (ell_get_type(x)==t_ELL_NF)
+    return ellnf_vecarea(x, ellnf_get_prec(x));
   if (!ell_is_complex(x)) pari_err_TYPE("area [not defined over C]",x);
-  prec = ellR_get_prec(x);
-  w = ellR_omega(x, prec);
-  w1 = gel(w,1); a = real_i(w1); b = imag_i(w1);
-  w2 = gel(w,2); c = real_i(w2); d = imag_i(w2);
-  return gabs(gsub(gmul(a,d),gmul(b,c)), prec);
+  return ellR_area(x, ellR_get_prec(x));
 }
 
 GEN
