@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2010  The PARI group.
+/* Copyright (C) 2010  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -19,3 +17,6 @@ const char *paricfg_version = PARIVERSION;
 const char *paricfg_buildinfo = PARIINFO;
 const long  paricfg_version_code = PARI_VERSION_CODE;
 const char *paricfg_vcsversion = PARI_VCSVERSION;
+const char *paricfg_compiledate = __DATE__;
+const char *paricfg_mt_engine = PARI_MT_ENGINE;
+const char *paricfg_gphelp = GPHELP;
