@@ -18,9 +18,9 @@ typedef enum {Fseq,
               Fassign,
               Fmatcoeff,
               Fmatrixelts,Fmatrixlines,
-              Fmat,Fvec,Fnoarg,
+              Fmat,Fvec,Fnoarg,Fnorange,
               Flistarg,
-              Frefarg,
+              Frefarg, Fvararg,
               Fconst,Fsmall,
               Ftag,
               Fentry,Fcall,Ffunction,Flambda
