@@ -66,7 +66,7 @@ CharEval(GEN chi, GEN logelt)
   long nn = Fl_center(n,d,d>>1);
   GEN x = gel(chi,2);
   x = gpowgs(x, labs(nn));
-  if (nn < 0) x = gconj(x);
+  if (nn < 0) x = conj_i(x);
   return x;
 }
 
@@ -165,7 +165,7 @@ GEN
 cyc2elts(GEN cyc)
 {
   long i, n;
-  GEN rep;
+  GEN z;
   GROUP_t G;
 
   G.cyc = typ(cyc)==t_VECSMALL? cyc: gtovecsmall(cyc);
@@ -173,14 +173,14 @@ cyc2elts(GEN cyc)
   G.r = lg(cyc)-1;
   G.j = zero_zv(G.r);
 
-  rep = cgetg(n+1, t_VEC);
-  gel(rep,n) = leafcopy(G.j); /* trivial elt comes last */
+  z = cgetg(n+1, t_VEC);
+  gel(z,n) = leafcopy(G.j); /* trivial elt comes last */
   for  (i = 1; i < n; i++)
   {
     (void)NextElt(&G);
-    gel(rep,i) = leafcopy(G.j);
+    gel(z,i) = leafcopy(G.j);
   }
-  return rep;
+  return z;
 }
 
 /* Let Qt as given by InitQuotient, compute a system of
@@ -194,7 +194,7 @@ ComputeLift(GEN Qt)
   e = EltsOfGroup(h, gel(Qt,2));
   if (!RgM_isidentity(U))
   {
-    GEN Ui = ZM_inv(U, gen_1);
+    GEN Ui = ZM_inv(U,NULL);
     for (i = 1; i <= h; i++) gel(e,i) = ZM_ZC_mul(Ui, gel(e,i));
   }
   return e;
@@ -208,7 +208,7 @@ get_Char(GEN nchi, long prec)
 
 /* prime divisors of conductor */
 static GEN
-divcond(GEN bnr) { GEN bid = bnr_get_bid(bnr); return gmael(bid,3,1); }
+divcond(GEN bnr) {GEN bid = bnr_get_bid(bnr); return gel(bid_get_fact(bid),1);}
 
 /* vector of prime ideals dividing bnr but not bnrc */
 static GEN
@@ -328,7 +328,7 @@ IsGoodSubgroup(GEN H, GEN bnr, GEN map)
   if (gequal(gel(modH,1), gel(mod,1))) { avma = av; return 1; }
 
   /* need to check the splitting of primes dividing mod but not modH */
-  bnrH = Buchray(bnr, modH, nf_INIT|nf_GEN);
+  bnrH = Buchray(bnr, modH, nf_INIT);
   P = divcond(bnr);
   PH = divcond(bnrH);
   p2 = ZM_mul(bnrsurjection(bnr, bnrH), p1);
@@ -480,7 +480,7 @@ FindModulus(GEN bnr, GEN dtQ, long *newprec)
 
   for(;;)
   {
-    GEN listid = ideallist(nf, maxnorm); /* all ideals of norm <= maxnorm */
+    GEN listid = ideallist0(nf, maxnorm, 4+8); /* ideals of norm <= maxnorm */
     pari_sp av1 = avma;
     for (n = minnorm; n <= maxnorm; n++, avma = av1)
     {
@@ -499,7 +499,7 @@ FindModulus(GEN bnr, GEN dtQ, long *newprec)
           gel(arch,N+1-s) = gen_0;
 
           /* compute Clk(m), check if m is a conductor */
-          bnrm = Buchray(bnf, m, nf_INIT|nf_GEN);
+          bnrm = Buchray(bnf, m, nf_INIT);
           c = bnrisconductor(bnrm, NULL);
           gel(arch,N+1-s) = gen_1;
           if (!c) continue;
@@ -578,6 +578,28 @@ END:
 /*                      2nd part: compute W(X)                      */
 /********************************************************************/
 
+/* find ilambda s.t. Diff*f*ilambda integral and coprime to f
+   and ilambda >> 0 at foo, fa = factorization of f */
+static GEN
+get_ilambda(GEN nf, GEN fa, GEN foo)
+{
+  GEN x, w, E2, P = gel(fa,1), E = gel(fa,2), D = nf_get_diff(nf);
+  long i, l = lg(P);
+  if (l == 1) return gen_1;
+  w = cgetg(l, t_VEC);
+  E2 = cgetg(l, t_COL);
+  for (i = 1; i < l; i++)
+  {
+    GEN pr = gel(P,i), t = pr_get_tau(pr);
+    long e = itou(gel(E,i)), v = idealval(nf, D, pr);
+    if (v) { D = idealdivpowprime(nf, D, pr, utoipos(v)); e += v; }
+    gel(E2,i) = stoi(e+1);
+    if (typ(t) == t_MAT) t = gel(t,1);
+    gel(w,i) = gdiv(nfpow(nf, t, stoi(e)), powiu(pr_get_p(pr),e));
+  }
+  x = mkmat2(P, E2);
+  return idealchinese(nf, mkvec2(x, foo), w);
+}
 /* compute the list of W(chi) such that Ld(s,chi) = W(chi) Ld(1 - s, chi*),
  * for all chi in LCHI. All chi have the same conductor (= cond(bnr)).
  * if check == 0 do not check the result */
@@ -586,9 +608,9 @@ ArtinNumber(GEN bnr, GEN LCHI, long check, long prec)
 {
   long ic, i, j, nz, nChar = lg(LCHI)-1;
   pari_sp av = avma, av2;
-  GEN sqrtnc, dc, cond, condZ, cond0, cond1, lambda, nf, T;
-  GEN cyc, vN, vB, diff, vt, idg, idh, zid, gen, z, nchi;
-  GEN indW, W, classe, s0, s, den, muslambda, sarch;
+  GEN sqrtnc, cond, condZ, cond0, cond1, nf, T;
+  GEN cyc, vN, vB, diff, vt, idh, zid, gen, z, nchi;
+  GEN indW, W, classe, s0, s, den, ilambda, sarch;
   CHI_t **lC;
   GROUP_t G;
 
@@ -611,35 +633,12 @@ ArtinNumber(GEN bnr, GEN LCHI, long check, long prec)
   T     = nf_get_Tr(nf);
   cond  = bnr_get_mod(bnr);
   cond0 = gel(cond,1); condZ = gcoeff(cond0,1,1);
-  cond1 = vec01_to_indices(gel(cond,2));
+  cond1 = gel(cond,2);
 
   sqrtnc = gsqrt(idealnorm(nf, cond0), prec);
-  dc  = idealmul(nf, diff, cond0);
-
-  /* compute a system of elements congruent to 1 mod cond0 and giving all
-     possible signatures for cond1 */
-  sarch = nfarchstar(nf, cond0, cond1);
-
-  /* find lambda in diff.cond such that gcd(lambda.(diff.cond)^-1,cond0) = 1
-     and lambda >> 0 at cond1 */
-  lambda = idealappr(nf, dc);
-  lambda = set_sign_mod_divisor(nf, NULL, lambda, sarch);
-  idg = idealdivexact(nf, lambda, dc);
-
-  /* find mu in idg such that idh=(mu) / idg is coprime with cond0 and
-     mu >> 0 at cond1 */
-  if (!gequal1(gcoeff(idg, 1, 1))) {
-    GEN P = divcond(bnr);
-    GEN f = famat_mul_shallow(idealfactor(nf, idg),
-                          mkmat2(P, zerocol(lg(P)-1)));
-    GEN mu = set_sign_mod_divisor(nf, NULL, idealapprfact(nf, f), sarch);
-    idh = idealdivexact(nf, mu, idg);
-    muslambda = nfdiv(nf, mu, lambda);
-  } else { /* mu = 1 */
-    idh = idg;
-    muslambda = nfinv(nf, lambda);
-  }
-  muslambda = Q_remove_denom(muslambda, &den);
+  ilambda = get_ilambda(nf, bid_get_fact(bnr_get_bid(bnr)), cond1);
+  idh = idealmul(nf, ilambda, idealmul(nf, diff, cond0)); /* integral */
+  ilambda = Q_remove_denom(ilambda, &den);
   z = den? rootsof1_cx(den, prec): NULL;
 
   /* compute a system of generators of (Ok/cond)^*, we'll make them
@@ -648,10 +647,10 @@ ArtinNumber(GEN bnr, GEN LCHI, long check, long prec)
   cyc = abgrp_get_cyc(zid);
   gen = abgrp_get_gen(zid);
   nz = lg(gen) - 1;
+  sarch = nfarchstar(nf, cond0, vec01_to_indices(cond1));
 
   nchi = cgetg(nChar+1, t_VEC);
   for (ic = 1; ic <= nChar; ic++) gel(nchi,ic) = cgetg(nz + 1, t_VECSMALL);
-
   for (i = 1; i <= nz; i++)
   {
     if (is_bigint(gel(cyc,i)))
@@ -664,15 +663,14 @@ ArtinNumber(GEN bnr, GEN LCHI, long check, long prec)
     }
   }
 
-  /* Sum chi(beta) * exp(2i * Pi * Tr(beta * mu / lambda) where beta
+  /* Sum chi(beta) * exp(2i * Pi * Tr(beta * ilambda) where beta
      runs through the classes of (Ok/cond0)^* and beta cond1-positive */
-
   vt = gel(T,1); /* ( Tr(w_i) )_i */
-  if (typ(muslambda) == t_COL)
-    vt = ZV_ZM_mul(vt, zk_multable(nf, muslambda));
+  if (typ(ilambda) == t_COL)
+    vt = ZV_ZM_mul(vt, zk_multable(nf, ilambda));
   else
-    vt = ZC_Z_mul(vt, muslambda);
-  /*vt = den . (Tr(w_i mu/lambda))_i */
+    vt = ZC_Z_mul(vt, ilambda);
+  /*vt = den . (Tr(w_i * ilambda))_i */
   G.cyc = gtovecsmall(cyc);
   G.r = nz;
   G.j = zero_zv(nz);
@@ -719,7 +717,7 @@ ArtinNumber(GEN bnr, GEN LCHI, long check, long prec)
   }
 
   classe = isprincipalray(bnr, idh);
-  z = powIs(- (lg(cond1)-1));
+  z = powIs(- (lg(gel(sarch,1))-1));
 
   for (ic = 1; ic <= nChar; ic++)
   {
@@ -901,7 +899,7 @@ InitChar(GEN bnr, GEN listCR, long prec)
       }
       else
       {
-        ch_bnr(dtcr) = Buchray(bnf, cond, nf_INIT|nf_GEN);
+        ch_bnr(dtcr) = Buchray(bnf, cond, nf_INIT);
         ch_diff(dtcr) = get_prdiff(bnr, cond);
       }
     }
@@ -942,7 +940,7 @@ CharNewPrec(GEN dataCR, GEN nf, long prec)
   N     =  nf_get_degree(nf);
   prec2 = precdbl(prec) + EXTRA_PREC;
 
-  C = sqrtr(divir(absi(dk), powru(mppi(prec2), N)));
+  C = sqrtr(divir(absi_shallow(dk), powru(mppi(prec2), N)));
 
   l = lg(dataCR);
   for (j = 1; j < l; j++)
@@ -1171,15 +1169,6 @@ CopyCoeff(int** a, int** a2, long n, long m)
   }
 }
 
-/* return q*p if <= n. Beware overflow */
-static long
-next_pow(long q, long p, long n)
-{
-  const GEN x = muluu((ulong)q, (ulong)p);
-  const ulong qp = uel(x,2);
-  return (lgefint(x) > 3 || qp > (ulong)n)? 0: qp;
-}
-
 static void
 an_AddMul(int **an,int **an2, long np, long n, long deg, GEN chi, int **reduc)
 {
@@ -1193,7 +1182,7 @@ an_AddMul(int **an,int **an2, long np, long n, long deg, GEN chi, int **reduc)
     if (gequal1(chi2)) c = NULL; else { Polmod2Coeff(c2, chi2, deg); c = c2; }
     for(k = 1, qk = q; qk <= n; k++, qk += q)
       AddMulCoeff(an[qk], c, an2[k], reduc, deg);
-    if (! (q = next_pow(q,np, n)) ) break;
+    if (! (q = umuluu_le(q,np, n)) ) break;
 
     chi2 = gmul(chi2, chi);
   }
@@ -1315,11 +1304,10 @@ InitPrimesQuad(GEN bnr, ulong N0, LISTray *R)
       }
       break;
     default: /* ramified */
-      if (condZ % p == 0) deg0(R,p);
-      else {
-        GEN pr = gel(idealprimedec(nf,prime),1);
-        deg11(R, p, bnr, pr);
-      }
+      if (condZ % p == 0)
+        deg0(R,p);
+      else
+        deg11(R, p, bnr, idealprimedec_galois(nf,prime));
       break;
     }
   }
@@ -1372,7 +1360,7 @@ InitPrimes(GEN bnr, ulong N0, LISTray *R)
   }
 }
 
-static GEN /* cf polcoeff */
+static GEN /* cf polcoef */
 _sercoeff(GEN x, long n)
 {
   long i = n - valp(x);
@@ -1435,8 +1423,7 @@ ppgamma(ST_t *T, long prec)
   gamdm[1] = evalsigne(1) | _evalvalp(0) | evalvarn(0);
   gel(gamdm,2) = gen_0;
   gel(gamdm,3) = gneg(gadd(gmul2n(mplog2(prec), 1), eul));
-  for (i = 2; i <= r; i++)
-    gel(gamdm,i+2) = mulri(gel(gamun,i+2), subis(int2n(i), 1));
+  for (i = 2; i <= r; i++) gel(gamdm,i+2) = mulri(gel(gamun,i+2), int2um1(i));
   gamdm = gmul(sqpi, gexp(gamdm, prec)); /* Gamma(1/2 + x) */
 
  /* We simplify to get one of the following two expressions
@@ -1561,7 +1548,7 @@ GetValue(GEN dtcr, GEN W, GEN S, GEN T, long fl, long prec)
   { /* (W(chi).S(conj(chi)) + T(chi)) / (sqrt(Pi)^q 2^{r1 - q}) */
     cf = gmul2n(powruhalf(mppi(prec), q), b);
 
-    z = gadd(gmul(W, gconj(S)), gconj(T));
+    z = gadd(gmul(W, conj_i(S)), conj_i(T));
     if (isreal) z = real_i(z);
     z = gdiv(z, cf); r = 0;
     if (fl & 2) z = gmul(z, ComputeAChi(dtcr, &r, 0, prec));
@@ -1607,7 +1594,7 @@ TestOne(GEN plg, RC_data *d)
   GEN z = gsub(d->beta, gel(plg,v));
   if (expo(z) >= d->G) return 0;
   for (j = 1; j < lg(plg); j++)
-    if (j != v && mpcmp(d->B, mpabs(gel(plg,j))) < 0) return 0;
+    if (j != v && mpcmp(d->B, mpabs_shallow(gel(plg,j))) < 0) return 0;
   return 1;
 }
 
@@ -1644,9 +1631,9 @@ RecCoeff3(GEN nf, RC_data *d, long prec)
   d->G = minss(-10, -prec2nbits(prec) >> 4);
   BIG = maxss(32, -2*d->G);
   tB  = sqrtnr(real2n(BIG-N,DEFAULTPREC), N-1);
-  Bd  = grndtoi(gmin(B, tB), &e);
+  Bd  = grndtoi(gmin_shallow(B, tB), &e);
   if (e > 0) return NULL; /* failure */
-  Bd = addis(Bd, 1);
+  Bd = addiu(Bd, 1);
   prec2 = nbits2prec( expi(Bd) + 192 );
   prec2 = maxss(precdbl(prec), prec2);
   B2 = sqri(Bd);
@@ -1704,24 +1691,23 @@ RecCoeff2(GEN nf,  RC_data *d,  long prec)
 {
   pari_sp av;
   GEN vec, M = nf_get_M(nf), beta = d->beta;
-  long i, imin, imax, lM = lg(M);
+  long bit, min, max, lM = lg(M);
 
   d->G = minss(-20, -prec2nbits(prec) >> 4);
 
   vec  = shallowconcat(mkvec(gneg(beta)), row(M, d->v));
-  imin = (long)prec2nbits_mul(prec, .225);
-  imax = (long)prec2nbits_mul(prec, .315);
-
+  min = (long)prec2nbits_mul(prec, 0.75);
+  max = (long)prec2nbits_mul(prec, 0.98);
   av = avma;
-  for (i = imax; i >= imin; i-=16, avma = av)
+  for (bit = max; bit >= min; bit-=32, avma = av)
   {
     long e;
-    GEN v = lindep2(vec, i), z = gel(v,1);
+    GEN v = lindep_bit(vec, bit), z = gel(v,1);
     if (!signe(z)) continue;
     *++v = evaltyp(t_COL) | evallg(lM);
     v = grndtoi(gdiv(v, z), &e);
     if (e > 0) break;
-    if (TestOne(gmul(M, v), d)) return v;
+    if (TestOne(RgM_RgC_mul(M, v), d)) return v;
   }
   /* failure */
   return RecCoeff3(nf,d,prec);
@@ -1829,10 +1815,10 @@ computean(GEN dtcr, LISTray *R, long n, long deg)
     for (q=p;;)
     {
       an_set0_coprime(an, p,q,n,deg); /* v_p(q) odd */
-      if (! (q = next_pow(q,p, n)) ) break;
+      if (! (q = umuluu_le(q,p, n)) ) break;
 
       an_mul(an,p,q,n,deg,chi,reduc);
-      if (! (q = next_pow(q,p, n)) ) break;
+      if (! (q = umuluu_le(q,p, n)) ) break;
       chi = gmul(chi, chi1);
     }
   }
@@ -1847,7 +1833,7 @@ computean(GEN dtcr, LISTray *R, long n, long deg)
     for(q=p;;)
     {
       an_mul(an,p,q,n,deg,chi,reduc);
-      if (! (q = next_pow(q,p, n)) ) break;
+      if (! (q = umuluu_le(q,p, n)) ) break;
       chi = gmul(chi, chi1);
     }
   }
@@ -1871,7 +1857,7 @@ computean(GEN dtcr, LISTray *R, long n, long deg)
     for(q=p;;)
     {
       an_mul(an,p,q,n,deg,chi1,reduc);
-      if (! (q = next_pow(q,p, n)) ) break;
+      if (! (q = umuluu_le(q,p, n)) ) break;
       chi2 = gmul(chi2, chi12);
       chi1 = gadd(chi2, gmul(chi1, chi11));
     }
@@ -2006,7 +1992,7 @@ QuadGetST(GEN bnr, GEN *pS, GEN *pT, GEN dataCR, GEN vChar, long prec)
           if (++c == 256) { gerepileall(av2,2, &p1,&p2); c = 0; }
         }
       gaffect(gmul(cf0, p1), gel(S,t));
-      gaffect(gmul(cf1,  gconj(p2)), gel(T,t));
+      gaffect(gmul(cf1,  conj_i(p2)), gel(T,t));
       FreeMat(matan,NN); avma = av2;
     }
     if (DEBUGLEVEL>1) err_printf("\n");
@@ -2154,24 +2140,31 @@ zeta_get_N0(GEN C,  GEN limx)
   avma = av; return itos(z);
 }
 
-/* even i such that limx^i ( (i\2)! )^r1 ( i! )^r2 ~ B. Small. */
+static GEN
+eval_i(long r1, long r2, GEN limx, long i)
+{
+  GEN t = powru(limx, i);
+  if (!r1)      t = mulrr(t, powru(mpfactr(i  , DEFAULTPREC), r2));
+  else if (!r2) t = mulrr(t, powru(mpfactr(i/2, DEFAULTPREC), r1));
+  else {
+    GEN u1 = mpfactr(i/2, DEFAULTPREC);
+    GEN u2 = mpfactr(i,   DEFAULTPREC);
+    if (r1 == r2) t = mulrr(t, powru(mulrr(u1,u2), r1));
+    else t = mulrr(t, mulrr(powru(u1,r1), powru(u2,r2)));
+  }
+  return t;
+}
+
+/* "small" even i such that limx^i ( (i\2)! )^r1 ( i! )^r2 > B. */
 static long
 get_i0(long r1, long r2, GEN B, GEN limx)
 {
   long imin = 1, imax = 1400;
+  while (mpcmp(eval_i(r1,r2,limx, imax), B) < 0) { imin = imax; imax *= 2; }
   while(imax - imin >= 4)
   {
-    long i = (imax + imin) >> 1;
-    GEN t = powru(limx, i);
-    if (!r1)      t = mulrr(t, powru(mpfactr(i  , DEFAULTPREC), r2));
-    else if (!r2) t = mulrr(t, powru(mpfactr(i/2, DEFAULTPREC), r1));
-    else {
-      GEN u1 = mpfactr(i/2, DEFAULTPREC);
-      GEN u2 = mpfactr(i,   DEFAULTPREC);
-      if (r1 == r2) t = mulrr(t, powru(mulrr(u1,u2), r1));
-      else t = mulrr(t, mulrr(powru(u1,r1), powru(u2,r2)));
-    }
-    if (mpcmp(t, B) >= 0) imax = i; else imin = i;
+    long m = (imax + imin) >> 1;
+    if (mpcmp(eval_i(r1,r2,limx, m), B) >= 0) imax = m; else imin = m;
   }
   return imax & ~1; /* make it even */
 }
@@ -2259,7 +2252,7 @@ GetST0(GEN bnr, GEN *pS, GEN *pT, GEN dataCR, GEN vChar, long prec)
            if (++c == 256) { gerepileall(av2,2, &p1,&p2); c = 0; }
           }
         gaffect(p1,        gel(S,t));
-        gaffect(gconj(p2), gel(T,t));
+        gaffect(conj_i(p2), gel(T,t));
         FreeMat(matan, NN); avma = av2;
       }
       else if (DEBUGLEVEL>1)
@@ -2445,7 +2438,7 @@ LABDOUB:
       long r;
       GEN WW, A = ComputeAChi(gel(dataCR,i), &r, 0, newprec);
       WW = gmul(gel(C,i), gmul(A, gel(W,i)));
-      gel(Lp,i) = gdiv(gmul(WW, gconj(gel(L1,i))), p1);
+      gel(Lp,i) = gdiv(gmul(WW, conj_i(gel(L1,i))), p1);
     }
   }
 
@@ -2666,7 +2659,7 @@ bnrL1(GEN bnr, GEN subgp, long flag, long prec)
         gel(L1,i) = GetValue(gel(dataCR,a), gel(W,a), gel(S,a), gel(T,a),
                              flag, prec);
       else
-        gel(L1,i) = gconj(gel(L1,-a));
+        gel(L1,i) = conj_i(gel(L1,-a));
     }
   }
   if (!(flag & 1))
@@ -2727,7 +2720,7 @@ makescind(GEN nf, GEN P)
   G = galoisinit(pol, NULL);
   L = gal_get_group(G);
   p = gal_get_p(G);
-  a = FpX_quad_root(nfpol, p, 0);
+  a = FpX_oneroot(nfpol, p);
   /* P mod a prime \wp above p (which splits) */
   Pp = FpXY_evalx(P, a, p);
   roo = gal_get_roots(G);
@@ -2750,7 +2743,7 @@ makescind(GEN nf, GEN P)
     }
   }
   pari_err_BUG("makescind");
-  return NULL; /*not reached*/
+  return NULL; /*LCOV_EXCL_LINE*/
 }
 
 /* pbnf = NULL if no bnf is needed, f = NULL may be passed for a trivial
@@ -2806,7 +2799,7 @@ quadhilbertreal(GEN D, long prec)
   /* if the exponent of the class group is 2, use Genus Theory */
   if (absequaliu(gel(cyc,1), 2)) return gerepileupto(av, GenusFieldQuadReal(D));
 
-  bnr  = Buchray(bnf, gen_1, nf_INIT|nf_GEN);
+  bnr  = Buchray(bnf, gen_1, nf_INIT);
   M = diagonal_shallow(bnr_get_cyc(bnr));
   dtQ = InitQuotient(M);
   nf  = bnf_get_nf(bnf);
@@ -3335,8 +3328,8 @@ ellphistinit(GEN om, long prec)
   GEN res,om1b,om2b, om1 = gel(om,1), om2 = gel(om,2);
 
   if (gsigne(imag_i(gdiv(om1,om2))) < 0) { swap(om1,om2); om = mkvec2(om1,om2); }
-  om1b = gconj(om1);
-  om2b = gconj(om2); res = cgetg(4,t_VEC);
+  om1b = conj_i(om1);
+  om2b = conj_i(om2); res = cgetg(4,t_VEC);
   gel(res,1) = gdivgs(elleisnum(om,2,0,prec),12);
   gel(res,2) = gdiv(PiI2(prec), gmul(om2, imag_i(gmul(om1b,om2))));
   gel(res,3) = om2b; return res;
@@ -3511,7 +3504,7 @@ treatspecialsigma(GEN bnr)
   if (absequaliu(D,3)) /* Q(j) */
   {
     if (i == 4 || i == 5 || i == 7) return polcyclo(i,0);
-    if (!absequaliu(gcoeff(f,1,1),9) || !absequaliu(Q_content(f),3)) return NULL;
+    if (!absequaliu(gcoeff(f,1,1),9) || !absequaliu(Z_content(f),3)) return NULL;
     /* f = P_3^3 */
     p1 = mkpolmod(bnf_get_tuU(bnf), nf_get_pol(nf));
     return gadd(pol_xn(3,0), p1); /* x^3+j */
@@ -3538,7 +3531,7 @@ treatspecialsigma(GEN bnr)
   p2 = gcoeff(f,2,2); /* integer > 0 */
   if (is_pm1(p2)) fl = 0;
   else {
-    if (Ds % 16 != 8 || !absequaliu(Q_content(f),2)) return NULL;
+    if (Ds % 16 != 8 || !absequaliu(Z_content(f),2)) return NULL;
     fl = 1; tryf >>= 1;
   }
   if (tryf <= 3 || umodiu(D, tryf) || !uisprime(tryf)) return NULL;
