@@ -310,8 +310,8 @@ Baker(baker_s *BS)
   /* Compute a bound for the h_0 */
   hb0 = gadd(gmul2n(BS->hal,2), gmul2n(gadd(BS->Hmu,mplog2(DEFAULTPREC)), 1));
   tmp = gmul(BS->divro, gdiv(gel(BS->NE,i1), gel(BS->NE,i2)));
-  tmp = gmax(gen_1, abslog(tmp));
-  hb0 = gmax(hb0, gdiv(tmp, BS->bak));
+  tmp = gmax_shallow(gen_1, abslog(tmp));
+  hb0 = gmax_shallow(hb0, gdiv(tmp, BS->bak));
   c9 = gmul(BS->c91,hb0);
   c9 = gprec_w(myround(c9, 1), DEFAULTPREC);
   Indc11 = rtor(mulir(BS->Ind,BS->c11), DEFAULTPREC);
@@ -319,10 +319,9 @@ Baker(baker_s *BS)
   B0 = mulir(shifti(BS->Ind,1),
              divrr(addrr(mulrr(c9,mplog(divrr(mulir(BS->Ind, c9),BS->c10))),
                          mplog(Indc11)), BS->c10));
-  B0 = gmax(B0, dbltor(2.71828183));
-  B0 = gmax(B0, mulrr(divir(BS->Ind, BS->c10),
-                      mplog(divrr(Indc11, BS->Pi2))));
-
+  B0 = gmax_shallow(B0, dbltor(2.71828183));
+  B0 = gmax_shallow(B0, mulrr(divir(BS->Ind, BS->c10),
+                              mplog(divrr(Indc11, BS->Pi2))));
   if (DEBUGLEVEL>1) {
     err_printf("  B0  = %Ps\n",B0);
     err_printf("  Baker = %Ps\n",c9);
@@ -347,7 +346,7 @@ CF_1stPass(GEN *B0, GEN kappa, baker_s *BS)
   if (cmprr(mulrr(dbltor(0.1),sqrr(denbound)), BS->inverrdelta) > 0)
     return -1;
 
-  q = denom( bestappr(BS->delta, denbound) );
+  q = denom_i( bestappr(BS->delta, denbound) );
   qd = errnum(BS->delta, q);
   ql = errnum(BS->lambda,q);
 
@@ -591,8 +590,8 @@ GuessQi(GEN b, GEN c, GEN *eps)
   Q = gel(lllint(Lat),1);
   if (gequal0(gel(Q,2))) return NULL; /* FAIL */
 
-  *eps = gadd(gadd(gel(Q,3), gmul(gel(Q,1),b)), gmul(gel(Q,2),c));
-  *eps = mpabs(*eps); return Q;
+  *eps = mpadd(mpadd(gel(Q,3), mpmul(gel(Q,1),b)), mpmul(gel(Q,2),c));
+  *eps = mpabs_shallow(*eps); return Q;
 }
 
 /* x a t_REAL */
@@ -701,7 +700,7 @@ SmallSols(GEN S, GEN x3, GEN poly, GEN rhs)
   /* y = 0 first: solve X^n = rhs */
   if (odd(n))
   {
-    if (Z_ispowerall(absi(rhs), n, &X))
+    if (Z_ispowerall(absi_shallow(rhs), n, &X))
       add_sol(&S, signe(rhs) > 0? X: negi(X), gen_0);
   }
   else if (signe(rhs) > 0 && Z_ispowerall(rhs, n, &X))
@@ -838,14 +837,18 @@ thueinit(GEN pol, long flag, long prec)
     long k,l;
     if (!bnf)
     {
-      bnf = Buchall(pol, nf_FORCE, DEFAULTPREC);
-      if (flag) (void)bnfcertify(bnf);
+      bnf = gen_0;
+      if (expi(ZX_disc(pol)) <= 50)
+      {
+        bnf = Buchall(pol, nf_FORCE, DEFAULTPREC);
+        if (flag) (void)bnfcertify(bnf);
+      }
     }
-    ro = nf_get_roots(bnf_get_nf(bnf));
-    l = lg(ro);
-    c0 = imag_i(gel(ro,1));
+    ro = typ(bnf)==t_VEC? nf_get_roots(bnf_get_nf(bnf))
+                        : QX_complex_roots(pol, DEFAULTPREC);
+    l = lg(ro); c0 = imag_i(gel(ro,1));
     for (k = 2; k < l; k++) c0 = mulrr(c0, imag_i(gel(ro,k)));
-    c0 = invr(sqrr(c0)); tnf = mkvec3(pol,bnf,c0);
+    setsigne(c0,1); c0 = invr(c0); tnf = mkvec3(pol, bnf, c0);
   }
   gel(tnf,1) = mkvec3(gel(tnf,1), C, L);
   return gerepilecopy(av,tnf);
@@ -923,8 +926,8 @@ get_B0(long i1, GEN Delta2, GEN Lambda, GEN Deps5, long prec, baker_s *BS)
 
         if (! (Q = GuessQi(BS->delta, BS->lambda, &ep)) ) break;
 
-        denbound = gadd(B0, absi(gel(Q,1)));
-        q = denom( bestappr(BS->delta, denbound) );
+        denbound = mpadd(B0, absi_shallow(gel(Q,1)));
+        q = denom_i( bestappr(BS->delta, denbound) );
         l0 = subrr(errnum(BS->delta, q), ep);
         if (signe(l0) <= 0) break;
 
@@ -932,14 +935,15 @@ get_B0(long i1, GEN Delta2, GEN Lambda, GEN Deps5, long prec, baker_s *BS)
         if (DEBUGLEVEL>1) err_printf("Semirat. reduction: B0 -> %Ps\n",B0);
       }
       /* if no progress, stop */
-      if (gcmp(oldB0, gadd(B0,dbltor(0.1))) <= 0) return gmin(oldB0, B0);
+      if (gcmp(oldB0, gadd(B0,dbltor(0.1))) <= 0)
+        return gmin_shallow(oldB0, B0);
       else step++;
     }
     i2++; if (i2 == i1) i2++;
     if (i2 > BS->r) break;
   }
   pari_err_BUG("thue (totally rational case)");
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 static GEN
@@ -969,15 +973,16 @@ get_Bx_LLL(long i1, GEN Delta2, GEN Lambda, long prec, baker_s *BS)
       /* FIXME: TO BE COMPLETED */
       if (!step && cf == cfMAX)
       { /* Semirational or totally rational case */
-        GEN Q, ep, q, l0, denbound;
+        GEN Q, Q1, Q2, ep, q, l0, denbound;
 
         if (! (Q = GuessQi(BS->delta, BS->lambda, &ep)) ) break;
 
         /* Q[2] != 0 */
-        denbound = gadd(mulri(B0, absi(gel(Q,1))),
-                        mulii(BS->Ind, absi(gel(Q,2))));
-        q = denom( bestappr(BS->delta, denbound) );
-        l0 = divri(subrr(errnum(BS->delta, q), ep), absi(gel(Q,2)));
+        Q1 = absi_shallow(gel(Q,1));
+        Q2 = absi_shallow(gel(Q,2));
+        denbound = gadd(mulri(B0, Q1), mulii(BS->Ind, Q2));
+        q = denom_i( bestappr(BS->delta, denbound) );
+        l0 = divri(subrr(errnum(BS->delta, q), ep), Q2);
         if (signe(l0) <= 0) break;
 
         get_B0Bx(BS, l0, &B0, &Bx);
@@ -991,7 +996,7 @@ get_Bx_LLL(long i1, GEN Delta2, GEN Lambda, long prec, baker_s *BS)
     if (i2 > BS->r) break;
   }
   pari_err_BUG("thue (totally rational case)");
-  return NULL; /* not reached */
+  return NULL; /* LCOV_EXCL_LINE */
 }
 
 static GEN
@@ -1027,7 +1032,7 @@ START:
   ALH    = gel(tnf,4);
   MatFU  = gel(tnf,5);
   A      = gel(tnf,6);
-  c1     = gel(csts,1); c1 = gmul(absi(rhs), c1);
+  c1     = mpmul(absi_shallow(rhs), gel(csts,1));
   c2     = gel(csts,2);
   BS.hal = gel(csts,3);
   x0     = gel(csts,4);
@@ -1045,9 +1050,9 @@ START:
   c4 = mulur(n-1, c3);
   c14 = mulrr(c4, vecmax_shallow(RgM_sumcol(gabs(A,DEFAULTPREC))));
 
-  x1 = gmax(x0, sqrtnr(shiftr(tmp,1),n));
-  x2 = gmax(x1, sqrtnr(mulur(10,c14), n));
-  x3 = gmax(x2, sqrtnr(shiftr(c14, EXPO1+1),n));
+  x1 = gmax_shallow(x0, sqrtnr(shiftr(tmp,1),n));
+  x2 = gmax_shallow(x1, sqrtnr(mulur(10,c14), n));
+  x3 = gmax_shallow(x2, sqrtnr(shiftr(c14, EXPO1+1),n));
   c90 = gmul(shiftr(mulur(18,mppi(DEFAULTPREC)), 5*(4+r)),
                     gmul(gmul(mpfact(r+3), powiu(muliu(BS.bak,r+2), r+3)),
                          glog(muliu(BS.bak,2*(r+2)),DEFAULTPREC)));
@@ -1102,8 +1107,8 @@ START:
     for (k=1; k<=r; k++)
     {
       GEN z = gdiv(gcoeff(MatFU,iroot1,k), gcoeff(MatFU,iroot2,k));
-      z = gmax(gen_1, abslog(z));
-      c91 = gmul(c91, gmax(gel(ALH,k), gdiv(z, BS.bak)));
+      z = gmax_shallow(gen_1, abslog(z));
+      c91 = gmul(c91, gmax_shallow(gel(ALH,k), gdiv(z, BS.bak)));
     }
     BS.c91 = c91;
 
@@ -1150,12 +1155,12 @@ START:
       {
         if (! (Bx = get_Bx_LLL(i1, Delta2, Lambda, prec, &BS)) )
            goto START;
-        x3 = gerepileupto(av2, gmax(Bx, x3));
+        x3 = gerepileupto(av2, gmax_shallow(Bx, x3));
       }
     }
     ine = 1;
   }
-  x3 = gmax(x0, MiddleSols(&S, x3, ro, P, rhs, s, c1));
+  x3 = gmax_shallow(x0, MiddleSols(&S, x3, ro, P, rhs, s, c1));
   return SmallSols(S, x3, P, rhs);
 }
 
@@ -1247,6 +1252,16 @@ sols_from_R(GEN Rab, GEN *pS, GEN P, GEN POL, GEN rhs)
   for (k = 1; k < l; k++)
     if (typ(gel(ry,k)) == t_INT) check_y(pS, P, POL, gel(ry,k), rhs);
 }
+static GEN
+Z_factor_if_easy(GEN rhs)
+{
+  GEN F, P;
+  long l;
+  if (expi(rhs) < 150) return Z_factor(rhs);
+  F = Z_factor_limit(rhs, 500000);
+  P = gel(F,1); l = lg(P);
+  return (l == 1 || BPSW_psp(gel(P,l-1)))? F: NULL;
+}
 /* Given a tnf structure as returned by thueinit, a RHS and
  * optionally the solutions to the norm equation, returns the solutions to
  * the Thue equation F(x,y)=a */
@@ -1286,20 +1301,12 @@ thue(GEN tnf, GEN rhs, GEN ne)
   { /* Case s=0. All solutions are "small". */
     GEN bnf = tnf_get_bnf(tnf);
     GEN c0 = gel(tnf,3), F;
-    x3 = sqrtnr(mulir(absi(rhs),c0), degpol(POL));
+    x3 = sqrtnr(mulir(absi_shallow(rhs),c0), degpol(POL));
     x3 = addrr(x3, dbltor(0.1)); /* guard from round-off errors */
     S = cgetg(1,t_VEC);
-    if (!ne && expo(x3) > 10)
+    if (!ne && typ(bnf) == t_VEC && expo(x3) > 10)
     {
-      long l;
-      GEN P;
-      if (expi(rhs) < 150) F = Z_factor(rhs);
-      else
-      {
-        F = Z_factor_limit(rhs, 500000);
-        P = gel(F,1); l = lg(P);
-        if (!is_pm1(rhs) && !BPSW_psp(gel(P,l-1))) F = NULL;
-      }
+      F = Z_factor_if_easy(rhs);
       if (F) ne = get_ne(bnf, F, gen_1);
     }
     if (ne)
@@ -1307,8 +1314,25 @@ thue(GEN tnf, GEN rhs, GEN ne)
       if (lg(ne) == 1) { avma = av; return cgetg(1,t_VEC); }
       if (degpol(POL) == 2) /* quadratic imaginary */
       {
-        GEN u = bnf_get_tuU(bnf);
-        long w =  bnf_get_tuN(bnf);
+        GEN u = NULL;
+        long w = 2;
+        if (typ(bnf) == t_VEC)
+        {
+          u = bnf_get_tuU(bnf);
+          w =  bnf_get_tuN(bnf);
+        }
+        else
+        {
+          GEN D = coredisc(ZX_disc(POL));
+          if (cmpis(D, -4) >= 0)
+          {
+            GEN F, T = quadpoly(D);
+            w = equalis(D, -4)? 4: 6;
+            setvarn(T, fetch_var_higher());
+            F = gcoeff(nffactor(POL, T), 1, 1);
+            u = gneg(lift_shallow(gel(F,2))); delete_var();
+          }
+        }
         if (w == 4) /* u = I */
           ne = shallowconcat(ne, RgXQV_RgXQ_mul(ne,u,POL));
         else if (w == 6) /* u = j */
